@@ -5,6 +5,7 @@ import importlib
 import datetime
 import inspect
 import builtins
+import copy
 import json
 import linecache
 import math
@@ -15,30 +16,30 @@ import re
 import shlex
 import struct
 import gc
+import numbers
 import subprocess
 import sys
 import threading
 import time
 import traceback
 import types
+import warnings
 from enum import Enum
 from inspect import signature
 from pydoc import locate
-from typing import Union, Tuple, Any, overload,NewType,Text
-
-
 
+from typing import Iterable,Generator,Sequence,Iterator,Union, Tuple, Any, overload, NewType, Text
 import numpy as np
 
-__all__ = ['get_session','set_session','get_session_value','get_backend','get_image_backend', 'get_trident_dir', 'epsilon', 'floatx','import_or_install',
-           'check_keys','make_sure', 'if_else', 'camel2snake', 'snake2camel', 'to_onehot', 'to_list', 'addindent', 'format_time',
+__all__ = ['get_session','set_session','get_session_value','is_autocast_enabled','set_autocast_enabled','get_backend','get_plateform','get_image_backend', 'get_trident_dir', 'epsilon', 'floatx','import_or_install',
+           'check_keys','make_sure', 'if_none', 'camel2snake', 'snake2camel', 'to_onehot', 'to_list', 'addindent', 'format_time',
            'get_time_suffix', 'get_file_modified_time','get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime',
            'next_prime', 'prev_prime', 'nearest_prime', 'PrintException','TensorShape', 'unpack_singleton', 'enforce_singleton',
-           'OrderedDict','map_function_arguments', 'ClassfierType', 'PaddingMode','Signature',
+           'OrderedDict','map_function_arguments', 'ClassfierType', 'PaddingMode','Signature','is_iter','get_string_actual_length',
            'Interpolation','is_numpy','find_minimal_edit_distance_key','jaccard_similarity','text_similarity','levenshtein',
 
-           'GetImageMode', 'split_path', 'make_dir_if_need', 'sanitize_path', 'ShortcutMode',
-          'get_args_spec', 'get_gpu_memory_map','get_memory_profile','get_gpu_memory_map']
+           'GetImageMode', 'split_path', 'make_dir_if_need', 'sanitize_path', 'ShortcutMode','adaptive_format','num_cpus',
+          'get_args_spec', 'get_gpu_memory_map','get_memory_profile','get_gpu_memory_map','dtype']
 
 
 # In some cases, these basic types are shadowed by corresponding
@@ -46,7 +47,7 @@ __all__ = ['get_session','set_session','get_session_value','get_backend','get_im
 # types.  See https://github.com/python/mypy/issues/4146 for why these
 # workarounds is necessary
 import six
-
+from trident import context
 
 _int = builtins.int
 _float = builtins.float
@@ -54,7 +55,7 @@ _bool = builtins.bool
 
 
 
-_SESSION = threading.local()
+_SESSION = context._context()
 
 
 
@@ -117,63 +118,14 @@ def make_dir_if_need(path):
     return sanitize_path(path)
 
 
-def if_else(a, b):
-    """
-    Syntax suggar for  value assigment with None check
-    if_else(a, b)  means if a is None else b
 
-    Args:
-        a (obj):
-        b (obj):
 
-    Returns:
-        None replacement
+def if_none(a, b):
+    "`b` if `a` is None else `a`"
+    return b if a is None else a
 
-    Examples:
-        >>> d=dict({'a':None})
-        >>> d['a']=if_else(d['a'],5)
-        >>> print(d)
-        {'a': 5}
-        >>> d['a']=if_else(d['a'],3)
-        >>> print(d)
-        {'a': 5}
-    """
 
-    if a is None:
-        return b
-    else:
-        return a
 
-def _get_trident_dir():
-    """Get or create trident directory
-    1)  read from enviorment variable 'TRIDENT_HOME'
-    2) use default directory '~/.trident'
-    3) if the directory not exist, create it!
-
-    Returns:
-        the  trident directory path
-
-    """
-    _trident_dir = ''
-    if 'TRIDENT_HOME' in os.environ:
-        _trident_dir = os.environ.get('TRIDENT_HOME')
-    else:
-        _trident_base_dir = os.path.expanduser('~')
-        if not os.access(_trident_base_dir, os.W_OK):
-            _trident_dir = '/tmp/.trident'
-        else:
-            _trident_dir = os.path.expanduser('~/.trident')
-
-    _trident_dir = sanitize_path(_trident_dir)
-    if not os.path.exists(_trident_dir):
-        try:
-            os.makedirs(_trident_dir)
-        except OSError as e:
-            # Except permission denied and potential race conditions
-            # in multi-threaded environments.
-            print(e)
-
-    return _trident_dir
 def get_trident_dir():
     """Method for access trident_dir attribute in session
 
@@ -184,7 +136,7 @@ def get_trident_dir():
         '~/.trident'
 
     """
-    return _SESSION.trident_dir
+    return context._context().trident_dir
 
 
 def get_plateform():
@@ -206,61 +158,7 @@ def get_plateform():
 
 
 
-def _initialize_session():
-    """
-    1) load session config from config file
-    2) if no exist value, assign the default value
 
-    """
-    global _SESSION
-    _SESSION.trident_dir = _get_trident_dir()
-    _SESSION.backend ='pytorch'
-
-    _SESSION.image_backend ='pillow'
-    _SESSION.epoch_equivalent =1000
-    _SESSION.floatx = 'float32'
-    _SESSION.epsilon = 1e-8
-    _SESSION.working_directory =os.getcwd()
-    _SESSION.plateform = get_plateform()
-    _SESSION.numpy_print_format = '{0:.4e}'
-    _SESSION.amp_available=False
-    _config_path = os.path.expanduser(os.path.join(_SESSION.trident_dir, 'trident.json'))
-    if os.path.exists(_config_path):
-        _config = {}
-        try:
-            with open(_config_path) as f:
-                _config = json.load(f)
-                for k, v in _config.items():
-                    try:
-                        if k == 'floatx':
-                            assert v in {'float16', 'float32', 'float64'}
-                        if k not in  ['trident_dir','device','working_directory']:
-                            _SESSION.__setattr__(k, v)
-                    except Exception as e:
-                        print(e)
-        except ValueError as ve:
-            print(ve)
-    if 'TRIDENT_WORKING_DIR' in os.environ:
-        _SESSION.working_directory = os.environ['TRIDENT_WORKING_DIR']
-        os.chdir(os.environ['TRIDENT_WORKING_DIR'])
-
-    if 'TRIDENT_BACKEND' in os.environ:
-        if _SESSION.backend != os.environ['TRIDENT_BACKEND']:
-            _SESSION.backend = os.environ['TRIDENT_BACKEND']
-    else:
-        try:
-            import torch
-            os.environ['TRIDENT_BACKEND'] = 'pytorch'
-        except:
-            try:
-                import tensorflow
-                os.environ['TRIDENT_BACKEND'] = 'tensorflow'
-            except:
-                pass
-    np.set_printoptions(formatter={'float_kind': lambda x: _SESSION.numpy_print_format.format(x)})
-    _SESSION.device = None
-
-_initialize_session()
 
 
 def get_session():
@@ -299,23 +197,20 @@ def set_session(key, value):
     return _SESSION
 
 def get_backend():
-    global  _SESSION
-    if hasattr(_SESSION,'backend'):
-        return _SESSION.backend
-    else:
-        return os.environ['TRIDENT_BACKEND']
+    return context._context().get_backend()
 
 def get_image_backend():
-    global  _SESSION
-    return _SESSION.image_backend
+    return context._context().image_backend
 
 def get_device():
-    global  _SESSION
-    if hasattr(_SESSION,'device'):
-        return _SESSION.device
-    else:
+    return context._context().device
+
+def is_autocast_enabled():
+    return get_session_value('is_autocast_enabled')
+
+def set_autocast_enabled(enabled:bool):
+    set_session('is_autocast_enabled',enabled)
 
-        _SESSION.device
 
 def _is_c_contiguous(data):
     while isinstance(data, list):
@@ -384,7 +279,46 @@ def snake2camel(string1):
     else:
         return ''.join(x.capitalize() or '_' for x in string1.split('_'))
 
+def adaptive_format(num:numbers.Number, prev_value:numbers.Number=None,value_type=None):
+    if num is None:
+        return 'nan'
+    format_string= '.3f'
+    if value_type=='metric':
+        format_string = '.3%'
+        if builtins.abs(num)>2:
+            format_string = '.3f'
+    elif value_type!='loss' and ((prev_value is None or prev_value==num) and 1e-3<=builtins.abs(num)<1.2):
+        format_string ='.3%'
+    elif  (prev_value is None or prev_value==num) and builtins.abs(num)<1e-3:
+        format_string = '.3e'
+    elif prev_value is  not None and builtins.abs(prev_value)>0 and builtins.abs(num) < 1e-3:
+        diff=builtins.min(builtins.max(3,-1*math.log10(builtins.abs(builtins.abs(prev_value)-builtins.abs(num)))),8)
+        format_string = '.{0}e'.format(diff)
+    elif prev_value is not None:
+        diff=prev_value-num
+        if  1e-3 < builtins.abs(diff) < 1.2:
+            format_string = '.3%'
+        elif builtins.abs(diff) <= 1e-3:
+            format_string = '.3e'
+    return '{0:{1}}'.format(num, format_string)
 
+def get_string_actual_length(input_string:str):
+    """Get the string acutal length (considering Chinese double byte)
+
+    Args:
+        input_string ():
+
+    Returns:
+
+    Examples:
+        >>> get_string_actual_length('你好')
+        4
+        >>> get_string_actual_length('深度學習deep learning')
+        21
+
+    """
+
+    return builtins.sum([len(input_string[i].encode("UTF-8")) if len(input_string[i].encode("UTF-8")) ==3 else len(input_string[i].encode("UTF-8"))  for i in range(len(input_string))])
 
 def PrintException():
     """
@@ -416,6 +350,103 @@ class DeviceType(object):
     index: _int  # THPDevice_index
 
 
+class dtype:
+    backend=get_backend()
+    if backend == 'pytorch':
+        import torch
+        # type definition
+        bool = torch.bool
+
+        int8 = torch.int8
+        byte = torch.int8
+        int16 = torch.int16
+        short = torch.int16
+        int32 = torch.int32
+        intc = torch.int32
+        int64 = torch.int64
+        intp = torch.int64
+
+        uint8 = torch.uint8
+        ubyte = torch.uint8
+        float16 = torch.float16
+        half = torch.float16
+        float32 = torch.float32
+        single = torch.float32
+        float64 = torch.float64
+        double = torch.float64
+        long = torch.int64
+        float = torch.float32
+    elif backend == 'tensorflow':
+        import tensorflow as tf
+        bool = tf.bool
+
+        int8 = tf.int8
+        byte = tf.int8
+        int16 = tf.int16
+        short = tf.int16
+        int32 = tf.int32
+        intc = tf.int32
+        int64 = tf.int64
+        intp = tf.int64
+
+        uint8 = tf.uint8
+        ubyte = tf.uint8
+        float16 = tf.float16
+        half = tf.float16
+        float32 = tf.float32
+        single = tf.float32
+        float64 = tf.float64
+        double = tf.float64
+        long = tf.int64
+        float = tf.float32
+    elif backend== 'onnx':
+        import onnx
+        from onnx import helper, onnx_pb, defs, numpy_helper
+        bool = onnx_pb.TensorProto.BOOL
+        int8 =  onnx_pb.TensorProto.INT8
+        byte = onnx_pb.TensorProto.INT8
+        int16 = onnx_pb.TensorProto.INT16
+        short = onnx_pb.TensorProto.INT16
+        int32 = onnx_pb.TensorProto.INT32
+        intc = onnx_pb.TensorProto.INT32
+        int64 = onnx_pb.TensorProto.INT64
+        intp =onnx_pb.TensorProto.INT64
+
+        uint8 = onnx_pb.TensorProto.UINT8
+        ubyte =onnx_pb.TensorProto.UINT8
+        float16 =onnx_pb.TensorProto.FLOAT1
+        half =onnx_pb.TensorProto.FLOAT1
+        float32 =onnx_pb.TensorProto.FLOAT
+        single = onnx_pb.TensorProto.FLOAT
+        float64 =onnx_pb.TensorProto.DOUBLE
+        double =onnx_pb.TensorProto.DOUBLE
+        long =onnx_pb.TensorProto.INT64
+        float =onnx_pb.TensorProto.FLOAT
+    elif backend == 'numpy':
+        bool = np.bool
+
+        int8 = np.int8
+        byte = np.int8
+        int16 = np.int16
+        short = np.int16
+        int32 = np.int32
+        intc = np.int32
+        int64 = np.int64
+        intp = np.int64
+
+        uint8 = np.uint8
+        ubyte = np.uint8
+        float16 = np.float16
+        half = np.float16
+        float32 = np.float32
+        single = np.float32
+        float64 = np.float64
+        double = np.float64
+        long = np.int64
+        float = np.float32
+
+
+
 # class Device(object):
 #     '''
 #     Describes device type and device id
@@ -473,18 +504,19 @@ class TensorShape(object):
         Raises:
           TypeError: If dims cannot be converted to a list of dimensions.
         """
-        if isinstance(dims, (tuple, list)):  # Most common case.
-            self._dims = [d for d in dims]
-        elif dims is None:
+
+        if dims is None:
             self._dims = None
         elif 'tensor' in dims.__class__.__name__.lower():
             if hasattr(dims,'cpu'):
                 dims.cpu()
             if hasattr(dims, 'detach'):
-                dims.detach()
+                dims.detach().numpy()
             if hasattr(dims, 'numpy'):
-                dims=dims.numpy()
+                dims=[d for d in dims]
             self._dims = [d for d in dims]
+        elif isinstance(dims, (tuple, list)):  # Most common case.
+            self._dims = [d if isinstance(d, numbers.Integral) else None if d is None else d.item() if hasattr(d,"item") else d.numpy() for d in dims]
         elif isinstance(dims,TensorShape):
             self._dims = dims.dims
         else:
@@ -565,6 +597,9 @@ class TensorShape(object):
         else:
             return None
 
+    def copy(self):
+        return copy.deepcopy(self)
+
 
 
     def __len__(self):
@@ -679,12 +714,17 @@ class TensorShape(object):
               return False
         return True
 
+
     def get_dummy_tensor(self):
-        shape=self._dims
-        shape[0]=2
+        shape=[d for d in self._dims]
+        if shape[0] is None:
+            shape[0]=2
+        else:
+            shape=[2,]+shape
         return np.random.standard_normal(shape)
 
 
+
 def as_shape(shape):
     """Converts the given object to a TensorShape."""
     if isinstance(shape, TensorShape):
@@ -694,6 +734,10 @@ def as_shape(shape):
 
 
 
+
+
+
+
 class OrderedDict(collections.OrderedDict):
     """ more easy-to-use OrderedDict"""
     def __init__(self, *args, **kwds):
@@ -731,35 +775,50 @@ class OrderedDict(collections.OrderedDict):
 
 
 class Signature(object):
-    """ more easy-to-use OrderedDict"""
-
-    def __init__(self, name=None):
+    def __init__(self, input_spec=None, output_spec=None,name=None):
         super().__init__()
         self.name = name
         self.inputs = OrderedDict()
         self.outputs = OrderedDict()
+        if input_spec is not None:
+            self.inputs[input_spec.name if input_spec.name is not None else 'input']=input_spec
+        if output_spec is not None:
+            self.outputs[output_spec.name if output_spec.name is not None else 'output']=output_spec
+
+    # @classmethod
+    # def get_signature(cls, fn:callable):
+    #
+    #     if "Layer"==fn.__class__.__base__:
+    #
+    #     if inspect.isfunction(fn)
+
+
+
 
     def maybe_not_complete(self):
-        if len(self.inputs)<1 and len(self.outputs)<1:
+        if len(self.inputs)<1 or len(self.outputs)<1:
             return True
-
-        elif any([item is not None or 'TensorSpec' not in item.__class__.__name__  for item in self.inputs.value_list ]):
-            return True
-        elif any([item is not None or 'TensorSpec' not in item.__class__.__name__  for item in self.outputs.value_list ]):
-            return True
-        elif any([ 'TensorSpec' in item.__class__.__name__ and item.shape is None  for item in self.inputs.value_list ]):
-            return True
-        elif any([ 'TensorSpec' in item.__class__.__name__ and item.shape is None  for item in self.outputs.value_list ]):
-            return True
-        else:
+        completeness=0
+        if self.inputs.value_list[0] is not None and self.inputs.value_list[0].__class__.__name__=='TensorSpec':
+            if isinstance(self.inputs.value_list[0].shape,TensorShape):
+                completeness+=1
+        if self.outputs.value_list[0] is not None and self.outputs.value_list[0].__class__.__name__=='TensorSpec':
+            if isinstance(self.outputs.value_list[0].shape,TensorShape):
+                completeness+=1
+        if completeness==2:
             return False
-    def get_kvsting(self, k, v):
+        else:
+            True
+
+    def _get_kvsting(self, k, v):
         if v is None:
             return '{0}'.format(k)
         elif isinstance(v, (list, tuple)):
             return '{0}: Tensor[{1}]'.format(k, v)
+        elif  v.__class__.__name__== "TensorSpec" and v.object_type is None:
+            return '{0}: Tensor[{1}] '.format(k, v._shape_tuple)
         elif  v.__class__.__name__== "TensorSpec":
-            return '{0}: Tensor[{1}]'.format(k, v._shape_tuple)
+            return '{0}: Tensor[{1}] ({2})'.format(k, v._shape_tuple,v.object_type)
         else:
             return '{0}:{1}'.format(k, v)
 
@@ -768,8 +827,8 @@ class Signature(object):
 
     def __repr__(self):
         # ElementTimes(x: Tensor[13]) -> Tensor[13]
-        input_str = ', '.join([self.get_kvsting(k, v) for k, v in self.inputs.item_list]) if len(self.inputs.item_list) > 0 else ''
-        output_str = ', '.join([self.get_kvsting(k, v) for k, v in self.outputs.item_list]) if len(self.outputs.item_list) > 0 else ''
+        input_str = ', '.join([self._get_kvsting(k, v) for k, v in self.inputs.item_list]) if len(self.inputs.item_list) > 0 else ''
+        output_str = ', '.join([self._get_kvsting(k, v) for k, v in self.outputs.item_list]) if len(self.outputs.item_list) > 0 else ''
         return '{0}( {1}) -> {2} '.format(self.name, input_str, output_str)
 
 class AverageMeter(object):
@@ -832,6 +891,11 @@ def to_onehot(label, classes):
     onehot[label] = 1
     return onehot
 
+def is_iter(x):
+    "Test whether `x 'can be used in a `for` loop"
+    #Rank 0 tensors in PyTorch are not really iterable
+    return isinstance(x, (Iterable,Generator)) and getattr(x,'ndim',1)
+
 
 def to_list(x):
     """
@@ -873,6 +937,8 @@ def to_list(x):
         return [x[i] for i in range(len(x))]
     elif 'tensor' in x.__class__.__name__.lower():
         return [x[i] for i in range(len(x))]
+    elif is_iter(x):
+        return list(x)
     elif hasattr(x, 'tolist') and callable(x.tolist):
         return x.tolist()
     elif isinstance(x, (int, float)):
@@ -964,7 +1030,7 @@ def check_keys(model, pretrained_state_dict):
     print('Unused checkpoint keys:{}'.format(len(unused_pretrained_keys)))
     print('\n\t'.join(unused_pretrained_keys))
     print('Used keys:{}'.format(len(used_pretrained_keys)))
-    print('\n\t'.join(used_pretrained_keys))
+    #print('\n\t'.join(used_pretrained_keys))
 
     assert len(used_pretrained_keys) > 0, 'load NONE from pretrained checkpoint'
     return True
@@ -1339,7 +1405,7 @@ def isprime(n):
     if n >= 9:
         divisors = [d for d in range(2, int(math.sqrt(n))) if n % d == 0]
         return all(n % od != 0 for od in divisors if od != n)
-    elif n in [1, 2, 3, 5, 7]:
+    elif n in [1, 2,3, 5, 7]:
         return True
     else:
         return False
@@ -1361,6 +1427,7 @@ def prev_prime(n):
         pos -= 1
 
 
+
 def nearest_prime(n):
     nextp = next_prime(n)
     prevp = prev_prime(n)
@@ -1369,6 +1436,10 @@ def nearest_prime(n):
     else:
         return nextp
 
+def num_cpus():
+    "Get number of cpus"
+    try:                   return len(os.sched_getaffinity(0))
+    except AttributeError: return os.cpu_count()
 
 
 def get_memory_profile(mode):
