@@ -17,8 +17,12 @@ import json
 from typing import List, Callable
 
 import numpy as np
-from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session,get_backend, \
-    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path,make_dir_if_need,Signature
+from trident.data.vision_transforms import Unnormalize
+
+from trident.backend.opencv_backend import array2image
+
+from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, get_backend, \
+    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path, make_dir_if_need, Signature, adaptive_format
 from trident.backend.tensorspec import *
 from trident.data.image_common import *
 from trident.callbacks import LambdaCallback, UnfreezeModelCallback
@@ -75,7 +79,6 @@ class ModelBase(object):
 
 
         self._preprocess_flow = []
-        self.input_spec=None
 
         self.current_save_path=None
 
@@ -98,7 +101,8 @@ class ModelBase(object):
                                  'tmp_metrics': HistoryBase('tmp_metrics'),
                                  'out_sample_metrics': HistoryBase('out_sample_metrics'),
                                  'print_progress_frequency': 10,
-                                'print_progress_unit': 'batch',
+                                  'collect_data_inteval':1,
+                                  'print_progress_unit': 'batch',
                                 'optimizer':None,
                                  'warmup':0,
                                  'grads': None,
@@ -107,7 +111,10 @@ class ModelBase(object):
                                  'total_batch': -1,  # current_batch
                                  'current_epoch': -1,  # current_epoch
                                  'current_batch': -1,  # current_batch
+                                 'data_feed':None,
                                  'current_model': None,  # current model
+                                'train_data':OrderedDict(),
+                                'test_data': OrderedDict(),
                                  'current_input': None,  # current input
                                  'current_target': None,  # current target
                                  'steps': 0,
@@ -138,7 +145,13 @@ class ModelBase(object):
         self._targets = OrderedDict()
 
         if isinstance(value, Layer):
-            value.signature = None
+            if not value.is_root or value.nodes.key_list[0]!=value.uuid:
+                value.nodes = OrderedDict([(mod.uuid, mod) for mod in list(value.modules()) if isinstance(mod, Layer)])
+                for name, mod in value.named_modules():
+                    if isinstance(mod, Layer):
+                        mod.nodes = value.nodes
+                        mod.relative_name = name
+
             inp_shape=None
             if hasattr(value,'signature') and value.signature is not None and len(value.signature.inputs)>0:
                 inp_shape=value.input_shape
@@ -148,7 +161,7 @@ class ModelBase(object):
             self._initial_graph(input_shape=inp_shape,output=value)
         elif isinstance(value, np.ndarray) or 'tensor' in value.__name__.lower():
             self.inputs = OrderedDict()
-            self._initial_graph(input_shape=value.input_shape,output= to_tensor(value))
+            self._initial_graph(input_shape=int_shape(value),output= to_tensor(value))
         else:
             raise ValueError('Only Layer, Module, Image and Tensor can be valid model')
 
@@ -231,38 +244,51 @@ class ModelBase(object):
     def update_signature(self, arg_names):
         if self.model is not None and hasattr(self.model, 'signature') and self.signature != self.model.signature:
             self.signature = self.model.signature
-        if self.signature is None or len(self.signature.inputs.key_list)+len(self.signature.outputs.key_list) == len(arg_names):
 
-            new_inputs =  OrderedDict()
-            for i in range(len(arg_names[:len(self.inputs)])):
-                arg = arg_names[:len(self.inputs)][i]
-                new_inputs[arg] = self.inputs.value_list[0]
-            self.inputs = new_inputs
+        if self.signature is not None and  len(self.signature.inputs.key_list) == len(arg_names):
+            old_inputs = copy.deepcopy(self.signature.inputs)
+            self.signature.inputs = OrderedDict()
+            self.inputs= OrderedDict()
+            for i in range(len(old_inputs)):
+                spec=old_inputs[i]
+                spec.name=arg_names[i]
+                self.signature.inputs[arg_names[i]]=spec
+                self.inputs[arg_names[i]]=spec
+            if len(old_inputs)==1:
+                self.input_spec=self.signature.inputs.value_list[0]
 
-            new_outputs = OrderedDict()
-            new_target = OrderedDict()
-            outputs_args = arg_names[len(self.signature.inputs.key_list):]
-            outputs=self._outputs
-            targets=self._targets
-            for i in range(len(outputs_args)):
-                arg = outputs_args[i]
-                new_outputs[arg] = outputs.value_list[0]
-                target_arg = arg.replace('output', 'target')
+        elif self.signature is not None and len(self.signature.inputs.key_list)+len(self.signature.outputs.key_list) == len(arg_names):
+
+            old_inputs = copy.deepcopy(self.signature.inputs)
+            old_outputs = copy.deepcopy(self.signature.outputs)
+            self.signature.inputs = OrderedDict()
+            self.signature.outputs = OrderedDict()
+            self.inputs= OrderedDict()
+            self._outputs = OrderedDict()
+            self._targets = OrderedDict()
+            for i in range(len(old_inputs)):
+                spec = old_inputs[i]
+                spec.name = arg_names[i]
+                self.signature.inputs[arg_names[i]] = spec
+                self.inputs[arg_names[i]] = spec
+            if len(old_inputs) == 1:
+                self.input_spec = self.signature.inputs.value_list[0]
+            for i in range(len(old_inputs)):
+                target_arg = arg_names[i ].replace('output', 'target')
                 if 'target' not in target_arg:
                     target_arg = 'target_' + target_arg
-                new_target[target_arg] = targets.value_list[0]
-            self._outputs = new_outputs
-            self._targets = new_target
-            # if self.model is not None:
-            #     self.signature = get_signature(self._model.forward, 'model')
-            #     self.signature.inputs = copy.deepcopy(self.inputs)
-            #     self.signature.outputs = copy.deepcopy(self._outputs)
+
+                spec = old_outputs[i]
+                spec.name = arg_names[i+len(old_inputs)]
+                self.signature.outputs[arg_names[i+len(old_inputs)]] = spec
+                self._outputs[arg_names[i+len(old_inputs)]] = spec
+                self._targets[target_arg] = spec
 
             print(self.signature)
         elif not isinstance(arg_names, (list, tuple)):
             raise ValueError('arg_names should be list or tuple')
         elif len(self.signature) != len(arg_names):
-            raise ValueError('data deed and arg_names should be the same length')
+            raise ValueError('data feed and arg_names should be the same length')
 
     @property
     def preprocess_flow(self):
@@ -271,17 +297,15 @@ class ModelBase(object):
     @preprocess_flow.setter
     def preprocess_flow(self,value):
         self._preprocess_flow=value
-        if isinstance(self.input_spec,TensorSpec):
-            self.input_spec=None
+
 
     @property
     def reverse_preprocess_flow(self):
-        return_list = []
-        return_list.append(reverse_image_backend_adaption)
-        for i in range(len(self.preprocess_flow)):
-            fn = self.preprocess_flow[-1 - i]
-            if fn.__qualname__ == 'normalize.<locals>.img_op':
-                return_list.append(unnormalize(fn.mean, fn.std))
+        return_list = [reverse_image_backend_adaption]
+        for i in range(len(self._preprocess_flow)):
+            fn = self._preprocess_flow[-1 - i]
+            if (inspect.isfunction(fn) and fn.__qualname__ == 'normalize.<locals>.img_op') or (fn.__class__.__name__ == 'Normalize'):
+                return_list.append(Unnormalize(fn.mean, fn.std))
         return_list.append(array2image)
         return return_list
 
@@ -293,8 +317,10 @@ class ModelBase(object):
             return np.expand_dims(image_backend_adaption(img_data),0)
         if isinstance(img_data, np.ndarray):
             for fc in self.preprocess_flow:
-                if self._model is not None and self.signature is not None and len(self.signature) > 1 and self.input_spec is not None:
+                if self._model is not None and  self.input_spec is not None:
                     img_data = fc(img_data,spec=self.input_spec)
+                elif self._model is not None and self.signature is not None and len(self.signature.inputs) > 1 :
+                    img_data = fc(img_data,spec=self.signature.inputs.value_list[0])
                 else:
                     img_data = fc(img_data)
             img_data = np.expand_dims(image_backend_adaption(img_data),0)
@@ -353,10 +379,17 @@ class ModelBase(object):
 
 
     def __getattr__(self, name):
+        if name in ['_input_shape','_output_shape','_class_names','class_names','output_fn']:
+            return self.__dict__[name]
+        elif name in ['reverse_preprocess_flow']:
+            return self.__getattribute__(name)
+
         if name == 'signature' or name == '_signature':
             _model = self.__dict__['_model']
             if _model is not None and isinstance(_model, Layer):
                 return _model.signature
+            else:
+                return Signature()
         if 'training_context' in self.__dict__:
             if name in  self.__dict__['training_context']:
                 return  self.__dict__['training_context'][name]
@@ -380,9 +413,18 @@ class ModelBase(object):
         raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
 
     def __setattr__(self, name, value):
-        if name in ['_input_shape','_output_shape']:
-            pass
+        if name in ['_input_shape','_output_shape','_class_names','class_names','output_fn']:
+            object.__setattr__(self, name, value)
+
+        elif name in ['_model']:
+            object.__setattr__(self, '_model', value)
+            if value is not None and value.signature is None and hasattr(value,'_built') and value._built==True:
+                value.signature=Signature()
+                value.signature.inputs['input']=TensorSpec(shape=value.input_shape,name='input')
+                value.signature.outputs['output'] = TensorSpec(shape=value.output_shape, name='output')
+
         else:
+
             if name=='signature' or name=='_signature':
                 _model = self.__dict__['_model']
                 if _model is not None and isinstance(_model ,Layer):
@@ -408,7 +450,13 @@ class ModelBase(object):
                 object.__setattr__(self, name, value)
 
     def __call__(self, *input, **kwargs):
-        return self._model(*input, **kwargs)
+        if self._model.training:
+            input = enforce_singleton(input)
+            return self._model(input, **kwargs)
+        else:
+            input=enforce_singleton(input)
+            input=self.data_preprocess(input)
+            return self._model(input, **kwargs)
 
     def with_optimizer(self, optimizer, **kwargs):
         return self
@@ -425,6 +473,9 @@ class ModelBase(object):
     def with_constraint(self, constraint, **kwargs):
         return self
 
+    def with_initializer(self, initializer, **kwargs):
+        return self
+
     def with_model_save_path(self, save_path, **kwargs):
         return self
 
@@ -471,37 +522,40 @@ class ModelBase(object):
 
     def reset_training_context(self):
         self.training_context = {
-            'losses': HistoryBase('losses'),  # loss_wrapper
-            'metrics': HistoryBase('metrics'),  # loss_wrapper
-            'epoch_losses': HistoryBase('epoch_losses'),  # loss_wrapper
-            'epoch_metrics': HistoryBase('epoch_metrics'),  # loss_wrapper
-            'grads_state': OrderedDict(),  # loss_wrapper
-            'tmp_losses': HistoryBase('tmp_losses'),  # loss_wrapper
-            'tmp_metrics': HistoryBase('tmp_metrics'),  # loss_wrapper
-            'out_sample_metrics': HistoryBase('out_sample_metrics'),
-            'print_progress_frequency': 10,
-            'print_progress_unit': 'batch',
-            'optimizer': None,
-            'warmup': 0,
-            'grads': None,
-            'stop_training': False,  # stop training
-            'total_epoch': -1,  # current_epoch
-            'total_batch': -1,  # current_batch
-            'current_epoch': -1,  # current_epoch
-            'current_batch': -1,  # current_batch
-            'current_model': None,  # current model
-            'current_input': None,  # current input
-            'current_target': None,  # current target
-            'steps': 0,
-            'current_output': None,  # current output
-            'current_loss': None,  # current loss
-            'best_metric': None,  # current loss
-            'best_model': None,  # current model
-            'loss_history': None, 'metric_history': None, 'base_lr': None,  # current loss
-            'current_lr': None,  # current loss
-            'save_path': os.path.join(working_directory,'Models'), 'is_collect_data': True, 'callbacks': [],
-            'stop_update': 0, 'retain_graph': False,
-            'skip_generate_output': False}
+                                 'losses': HistoryBase('losses'),
+                                 'metrics': HistoryBase('metrics'),
+                                 'epoch_losses': HistoryBase('epoch_losses'),
+                                 'epoch_metrics': HistoryBase('epoch_metrics'),
+                                 'grads_state': OrderedDict(),
+                                 'tmp_losses': HistoryBase('tmp_losses'),
+                                 'tmp_metrics': HistoryBase('tmp_metrics'),
+                                 'out_sample_metrics': HistoryBase('out_sample_metrics'),
+                                 'print_progress_frequency': 10,
+                                  'collect_data_inteval':1,
+                                  'print_progress_unit': 'batch',
+                                'optimizer':None,
+                                 'warmup':0,
+                                 'grads': None,
+                                 'stop_training': False,  # stop training
+                                 'total_epoch': -1,  # current_epoch
+                                 'total_batch': -1,  # current_batch
+                                 'current_epoch': -1,  # current_epoch
+                                 'current_batch': -1,  # current_batch
+                                 'data_feed':None,
+                                 'current_model': None,  # current model
+                                 'current_input': None,  # current input
+                                 'current_target': None,  # current target
+                                 'steps': 0,
+                                 'current_output': None,  # current output
+                                 'current_loss': None,  # current loss
+                                 'best_metric': None,  # current loss
+                                 'best_model': None,  # current model
+                                'loss_history': None, 'metric_history': None, 'base_lr': None,  # current loss
+                                'current_lr': None,  # current loss
+                                'save_path': os.path.join(working_directory,'Models'), 'is_collect_data': True, 'callbacks': [],
+                                 'stop_update': 0, 'retain_graph': False,
+                                 'skip_generate_output':False}
+
 
     def adjust_learning_rate(self, lr):
         raise NotImplementedError
@@ -520,9 +574,7 @@ class ModelBase(object):
         pass
 
     def do_on_epoch_start(self):
-        self.training_context['time_epoch_start'] = time.time()
-        if self.training_context['steps'] == 0:
-            self.training_context['time_epoch_progress'] = self.training_context['time_epoch_start']
+        pass
 
     def do_on_epoch_end(self):
         # set model as training state
@@ -530,14 +582,10 @@ class ModelBase(object):
         pass
 
     def do_on_batch_start(self):
-        self.training_context['time_batch_start'] = time.time()
-        if self.training_context['steps'] == 0:
-            self.training_context['time_batch_progress'] = self.training_context['time_batch_start']
+        pass
 
 
     def do_on_batch_end(self):
-        # set model as training state
-        # zero grad
         pass
 
     def do_on_data_received(self, train_data, test_data):
@@ -559,12 +607,8 @@ class ModelBase(object):
         pass
 
     def do_post_gradient_update(self):
+        self.training_context['tmp_losses'].collect('total_losses',self.training_context['steps'],self.training_context['current_loss'].item)
 
-        self.training_context['tmp_losses'].collect('total_losses',self.training_context['steps'],self.training_context['current_loss'])
-        if self.training_context['is_collect_data'] == True:
-            steps,values=self.training_context['tmp_losses'].get_series('total_losses')
-            self.training_context['losses'].collect('total_losses',self.training_context['steps'],float(np.asarray(values).mean()))
-            self.training_context['tmp_losses'].reset()
 
     def do_on_metrics_evaluation_start(self):
         pass
@@ -642,44 +686,51 @@ class ModelBase(object):
         if 'max_name_length' not in self.training_context:
             self.training_context['max_name_length']=len(self.name)+1
         metric_strings=[]
-        for k, v in self._metrics.items():
-            collect_history = self._metrics[k].collect_history
-            metric_value=None
-            batch_steps, batch_values = self.batch_metric_history.get_series(k)
-            if collect_history != False:
-                if k in self.batch_metric_history and len(batch_values)>=print_batch_progress_frequency:
-                    metric_value=np.array(batch_values[-1*print_batch_progress_frequency:]).mean()
-                elif k in self.training_context['tmp_metrics']:
-                    tmp_steps,tmp_values=self.training_context['tmp_metrics'].get_series(k)
-                    metric_value = np.array(tmp_values).mean()
-                    self.training_context['tmp_metrics'][k]=[]
-                format_string='.3%'
-                if metric_value > 3:
-                    format_string = '.3f'
-                elif metric_value< 1e-3:
-                    format_string = '.3e'
-                metric_strings.append('{0}: {1:<8{2}}'.format(k, metric_value, format_string))
+        slice_length = print_batch_progress_frequency // self.training_context['collect_data_inteval']
+        for k in  self.batch_metric_history.key_list:
+            collect_history =True
 
+            if k in self._metrics:
+                collect_history=self._metrics[k].collect_history
+            metric_value=None
+            if collect_history:
+                batch_steps, batch_values = self.batch_metric_history.get_series(k)
+                if len(batch_values)==0:
+                    batch_steps, batch_values = self.tmp_metrics.get_series(k)
+                    metric_value = np.array(batch_values).mean()
+                else:
+
+                    if len(batch_values) > slice_length:
+                        metric_value = np.array(batch_values[-1 * slice_length:]).mean()
+                    else:
+                        metric_value=np.array(batch_values).mean()
+                metric_strings.append('{0}: {1} '.format(k, adaptive_format(metric_value, value_type='metric')))
+        loss_value=None
         loss_steps,loss_values=self.batch_loss_history.get_series('total_losses')
-        loss_value=float(np.array(loss_values[-1*print_batch_progress_frequency:]).mean())
-        progress_start = self.training_context['time_batch_progress']
-        progress_end = time.time()
-        step_time = progress_end - progress_start
-        self.training_context['time_batch_progress'] = progress_end
+        if len(loss_values) == 0:
+            loss_steps, loss_values = self.tmp_losses.get_series('total_losses')
+            loss_value = np.array(loss_values).mean()
+        else:
+            if len(loss_values) > slice_length:
+                loss_value= to_numpy(loss_values[-1*slice_length:]).astype(np.float32).mean()
+            else:
+                loss_value = to_numpy(loss_values).astype(np.float32).mean()
+        step_time =  self.training_context['time_batch_progress']
         progress_bar(step_time,self.training_context['current_batch'], self.training_context['total_batch'],
-                 'Loss: {0:<8.5f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(loss_value, ','.join(metric_strings), self.training_context['current_lr'],
+                 'Loss: {0} | {1} | lr: {2:<10.3e} | epoch: {3}'.format(adaptive_format(loss_value,value_type='loss'), ', '.join(metric_strings), self.training_context['current_lr'],
                      self.training_context['current_epoch']), name=self.name.ljust(self.training_context['max_name_length']+1,' '))
+        self.training_context['time_batch_progress'] = 0
 
     def print_epoch_progress(self, print_epoch_progress_frequency):
         if 'max_name_length' not in self.training_context:
             self.training_context['max_name_length']=len(self.name)+1
         metric_strings=[]
-        loss_steps, loss_values = self.epoch_loss_history.get_series('total_losses')
+        loss_steps, loss_values = self.batch_loss_history.get_series('total_losses')
         if print_epoch_progress_frequency>len(loss_values):
             print_epoch_progress_frequency=len(loss_values)
 
-        loss_value = float(np.array(loss_values[-1 * print_epoch_progress_frequency:]).mean())
-        for k, v in self.epoch_metric_history.items():
+        loss_value = to_numpy(loss_values[-1*int(print_epoch_progress_frequency):]).astype(np.float32).mean()
+        for k  in self.epoch_metric_history.key_list:
             format_string='.3%'
             steps,values=self.epoch_metric_history.get_series(k)
             metric_value=to_numpy(values)
@@ -688,13 +739,12 @@ class ModelBase(object):
             elif np.max(metric_value)<1e-3:
                 format_string = '.3e'
 
-            metric_strings.append('{0}: {1:<8{2}}'.format(k,float(metric_value[-1*int(print_epoch_progress_frequency):].mean()),format_string))
-        progress_start = self.training_context['time_epoch_progress']
-        progress_end = time.time()
-        step_time = progress_end-progress_start
-        self.training_context['time_epoch_progress']=progress_end
+            metric_strings.append('{0}: {1}'.format(k,adaptive_format(float(metric_value[-1*int(print_epoch_progress_frequency):].mean()),value_type='metric')))
+        step_time = self.training_context['time_epoch_progress']
+
         progress_bar(step_time,self.training_context['current_epoch'], self.training_context['total_epoch'],
-                     'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}'.format(loss_value, ','.join(metric_strings), self.training_context['current_lr']), name=self.name.ljust(self.training_context['max_name_length']+1,' '))
+                     'Loss: {0}| {1} | lr: {2:<10.3e}'.format(adaptive_format(loss_value,value_type='loss'), ', '.join(metric_strings), self.training_context['current_lr']), name=self.name.ljust(self.training_context['max_name_length']+1,' '))
+        self.training_context['time_epoch_progress'] = 0
 
     #@pysnooper.snoop()
     def train_model(self, train_data, test_data, current_epoch, current_batch, total_epoch, total_batch,
@@ -712,6 +762,7 @@ class ModelBase(object):
             self.training_context['current_lr'] = self.optimizer.lr
             self.training_context['train_data'] = train_data
             self.training_context['test_data'] = test_data
+            self.training_context['is_out_sample_evaluation']=is_out_sample_evaluation
 
 
 
@@ -763,14 +814,14 @@ class ModelBase(object):
                                 self.train_data[k] = v
                         elif 'tensor' in output.__class__.__name__.lower():
                             self.train_data[self.outputs.key_list[0]] = output
-                            if self.use_output_as_loss==True:
+                            if self.use_output_as_loss:
 
                                 this_loss=output.sum()
                                 self.training_context['losses'].collect(self.outputs.key_list[0],self.training_context['steps'],this_loss)
                                 self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
                         else:
                             self.train_data[self.outputs.key_list[0]] = output
-                            if self.use_output_as_loss==True:
+                            if self.use_output_as_loss:
 
                                 this_loss=output.sum()
                                 self.training_context['losses'].collect(self.outputs.key_list[0], self.training_context['steps'], this_loss)
@@ -790,6 +841,8 @@ class ModelBase(object):
 
             # confirm singleton
             # output=unpack_singleton(output)
+            for callback in self.callbacks:
+                callback.on_loss_calculation_start(self.training_context)
 
             # losss
             for k, v in self._losses.items():
@@ -831,7 +884,7 @@ class ModelBase(object):
             for callback in self.callbacks:
                 callback.on_loss_calculation_end(self.training_context)
 
-            if accumulate_grads == False:
+            if not accumulate_grads:
                 # regularizer
                 for k, v in self._regs.items():
                     this_loss=to_tensor(0.0,requires_grad=True)
@@ -862,12 +915,7 @@ class ModelBase(object):
                 # ON_POSTBACKWARD_CALCULATION
                 self.do_post_gradient_update()
 
-                if isinstance(self._model, Layer) and any_abnormal_number(self._model):
-                    for para in self._model.parameters():
-                        if any_abnormal_number(para):
-                            para.data.copy_(where(is_nan(para), random_normal_like(para, mean=0, std=0.02).to(get_device()), para))
 
-                # model comfirm
                 for k, v in self._constraints.items():
                     if self.training_context['stop_update'] == 0 :
                         v(self._model)
@@ -901,9 +949,9 @@ class ModelBase(object):
 
                 for k, v in self._metrics.items():
                     collect_history =getattr(v,'collect_history') if  hasattr(v,'collect_history') else True
-                    if not collect_history == False:
+                    if collect_history:
                         self.training_context['metrics'].regist(k)
-                        self.training_context['tmp_metrics'].regist(k)
+                    self.training_context['tmp_metrics'].regist(k)
 
                     this_metric = try_map_args_and_call(v, self.train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else to_tensor(0)
                     self.training_context['tmp_metrics'].collect(k, self.training_context['steps'], this_metric)
@@ -925,9 +973,22 @@ class ModelBase(object):
                     #aggregate tmp data and move to metrics history
                     for k, v in self.training_context['tmp_metrics'].items():
                         steps,values=self.training_context['tmp_metrics'].get_series(k)
-                        self.training_context['metrics'].collect(k, self.training_context['steps'], np.asarray(values).mean())
+
+                        #check
+                        if len(steps) > 0:
+                            values = np.mean(to_numpy(values))
+                            self.training_context['metrics'].collect(k, self.training_context['steps'], values)
                     self.training_context['tmp_metrics'].reset()
 
+                    for k, v in self.training_context['tmp_losses'].items():
+                        steps, values = self.training_context['tmp_losses'].get_series(k)
+
+                        if len(steps)>0:
+                            values =np.mean(to_numpy(values))
+                            self.training_context['losses'].collect(k, self.training_context['steps'], values)
+                    self.training_context['tmp_losses'].reset()
+
+
                 # ON_BATCH_END
                 self.do_on_batch_end()
                 for callback in self.training_context['callbacks']:
@@ -955,34 +1016,34 @@ class ModelBase(object):
                         metric_value= test_values[-1]
                         history_metric_value=np.array(test_values).mean()
 
-                        format_string = '.3%'
-                        if history_metric_value > 3:
-                            format_string = '.3f'
-                        elif history_metric_value < 1e-3:
-                            format_string = '.3e'
-                        verbose.append('{0}: {1:<8{2}}'.format(k, metric_value, format_string))
+                        verbose.append('{0}: {1}'.format(k, adaptive_format(metric_value)))
                     print(self.training_context['model_name'] + ': out-of-sample evaluation: ',','.join(verbose))
-            self.training_context['steps'] += 1
+            #self.training_context['steps'] += 1
             if self.training_context['current_batch'] == self.training_context['total_batch'] - 1:
                 self.do_on_epoch_end()
                 batch_steps,batch_values=self.training_context['losses'].get_series('total_losses')
                 if not hasattr(self.training_context['losses'],'last_aggregate_idx'):
-                    self.epoch_loss_history.collect('total_losses',self.training_context['current_epoch'],np.asarray(batch_values).mean())
-                    self.training_context['losses'].last_aggregate_idx=len(batch_values)
+                    if len(batch_values)>0:
+                        self.epoch_loss_history.collect('total_losses',self.training_context['current_epoch'],batch_values)
+                        self.training_context['losses'].last_aggregate_idx=len(batch_values)
                 else:
-                    self.epoch_loss_history.collect('total_losses', self.training_context['current_epoch'], np.asarray(batch_values[self.training_context['losses'].last_aggregate_idx:]).mean())
-                    self.training_context['losses'].last_aggregate_idx = len(batch_values)
+                    if len(batch_values) > 0:
+                        self.epoch_loss_history.collect('total_losses', self.training_context['current_epoch'],batch_values[self.training_context['losses'].last_aggregate_idx:])
+                        self.training_context['losses'].last_aggregate_idx = len(batch_values)
 
 
 
                 for k, v in self.training_context['metrics'].items():
                     metric_steps, metric_values = self.training_context['metrics'].get_series(k)
                     if not hasattr(self.training_context['metrics'], 'last_aggregate_idx'):
-                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.asarray(metric_values).mean())
-                        self.training_context['metrics'].last_aggregate_idx = len(metric_values)
+                        if len(metric_values) > 0:
+                            self.epoch_metric_history.collect(k, self.training_context['current_epoch'], metric_values)
+                            self.training_context['metrics'].last_aggregate_idx = len(metric_values)
                     else:
-                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.asarray(metric_values[self.training_context['metrics'].last_aggregate_idx:]).mean())
-                        self.training_context['metrics'].last_aggregate_idx = len(metric_values)
+                        if len(metric_values) > 0:
+                            if self.training_context['metrics'].last_aggregate_idx<len(metric_values):
+                                self.epoch_metric_history.collect(k, self.training_context['current_epoch'], metric_values[self.training_context['metrics'].last_aggregate_idx:])
+                                self.training_context['metrics'].last_aggregate_idx = len(metric_values)
 
 
 
@@ -1030,7 +1091,7 @@ class ModelBase(object):
         raise NotImplementedError
 
     def trigger_when(self, when='on_batch_end',epoch=None,batch=None,epoch_frequency=None,batch_frequency=None,action=None):
-        new_callbacks=LambdaCallback(when,epoch=epoch,batch=batch,epoch_frequency=epoch_frequency,batch_frequency=batch_frequency,function=action)
+        new_callbacks=LambdaCallback(when,epoch=epoch,batch=batch,epoch_frequency=epoch_frequency,batch_frequency=batch_frequency,action=action)
         self.with_callbacks(new_callbacks)
         return self
 
@@ -1049,6 +1110,7 @@ class ModelBase(object):
             set_device('cuda')
         elif self._model is not None and isinstance(self._model, Tensor):
             self._model.cuda()
+
     def gpu(self):
         self.cuda()
 
