@@ -22,6 +22,11 @@ from copy import copy
 from functools import update_wrapper, partial
 from typing import List, Tuple, Optional, Union, Callable, Any, Iterable, Mapping, TypeVar
 import typing
+
+try:
+    import _pickle as pickle
+except:
+    import pickle
 from itertools import islice
 from distutils.version import Version, LooseVersion
 import torch.nn as nn
@@ -32,13 +37,18 @@ from torch._six import container_abcs
 from torch.nn.parameter import Parameter
 from trident.backend.tensorspec import *
 from trident.backend.common import to_list, addindent, camel2snake, unpack_singleton, enforce_singleton, OrderedDict, get_session, set_session, get_session_value, \
-    PrintException, Signature,TensorShape,split_path,make_dir_if_need,sanitize_path
+    PrintException, Signature, TensorShape, split_path, make_dir_if_need, sanitize_path
 from trident.backend.tensorspec import *
 from trident.backend import iteration_tools
 from trident.backend.pytorch_ops import *
 from trident.backend import pytorch_ops as tops
-from trident.backend import dtype as Dtype
-__all__ = ['get_device', 'set_device', 'Layer', 'Sequential', 'ModuleList', 'Parameter', 'ModuleDict', 'print_network', 'summary', 'load', 'save', 'Combine', 'try_map_args_and_call',
+from trident.backend.common import dtype as Dtype
+from trident import context
+ctx = context._context()
+_backend = ctx.get_backend()
+
+__all__ = ['get_device', 'set_device', 'Layer', 'Sequential', 'ModuleList', 'Parameter', 'ModuleDict', 'print_network', 'summary', 'load', 'save', 'Combine',
+           'try_map_args_and_call',
            'print_mem_stack',
            'normalize_padding', 'fix_layer']
 
@@ -47,7 +57,6 @@ _FUN_NAMES = [
 for target_fun_name, source_fun in _FUN_NAMES:
     setattr(Tensor, target_fun_name, source_fun)
 
-
 version = torch.__version__
 sys.stdout.write('Pytorch version:{0}.\n'.format(version))
 
@@ -59,7 +68,7 @@ if pt_version.version < base_version.version:
     raise ValueError('Not support Pytorch older then version 1.4')
 elif pt_version.version >= amp_version.version:
     set_session('amp_available', True if torch.cuda.is_available() and pt_version >= amp_version else False)
-    if get_session_value('amp_available') == True:
+    if get_session_value('amp_available'):
         sys.stdout.write('Automatic Mixed Precision Support:{0}.\n'.format(True))
     else:
         sys.stdout.write('Automatic Mixed Precision Support:{0}.\n'.format(False))
@@ -87,7 +96,7 @@ def set_device(device='cpu'):
             for i in range(len(gcitems)):
                 obj = gcitems[i]
                 try:
-                    if torch.is_tensor(obj) or (hasattr(obj, 'data') and torch.is_tensor(obj.data)):
+                    if is_tensor(obj) or hasattr(obj, 'data'):
                         obj.to(device)
                     elif isinstance(obj, nn.Module):
                         obj.to(device)
@@ -101,7 +110,7 @@ if torch.cuda.is_available() and get_device() == 'cuda':
     torch.backends.cudnn.enabled = True
     torch.backends.cudnn.benchmark = True
     torch.backends.cudnn.deterministic = False
-    os.environ["CUDA_VISIBLE_DEVICES"]='0'
+    os.environ["CUDA_VISIBLE_DEVICES"] = '0'
 
 
 def load(f):
@@ -135,7 +144,7 @@ def save(obj, f, is_compressed=False):
     Returns:
 
     """
-    torch.save(obj, f, _use_new_zipfile_serialization=is_compressed)
+    torch.save(obj, f, pickle_module=pickle, _use_new_zipfile_serialization=is_compressed)
     return True
 
 
@@ -177,7 +186,6 @@ _global_forward_pre_hooks = OrderedDict()
 _global_forward_hooks = OrderedDict()
 
 
-
 def register_module_forward_pre_hook(hook: Callable[..., None]) -> RemovableHandle:
     r"""Registers a forward pre-hook common to all modules.
 
@@ -241,8 +249,9 @@ def register_module_forward_hook(hook: Callable[..., None]) -> RemovableHandle:
     _global_forward_hooks[handle.id] = hook
     return handle
 
+
 def register_module_backward_hook(
-    hook: Callable[['Module', _grad_t, _grad_t], Optional[Tensor]]
+        hook: Callable[['Module', _grad_t, _grad_t], Optional[Tensor]]
 ) -> RemovableHandle:
     r"""Registers a backward hook common to all the modules.
 
@@ -292,11 +301,8 @@ class Parameter(nn.Parameter):
         return super().requires_grad
 
     @trainable.setter
-    def trainable(self,value):
-        super().requires_grad=value
-
-
-
+    def trainable(self, value):
+        super().requires_grad = value
 
 
 class Layer(nn.Module):
@@ -371,14 +377,14 @@ class Layer(nn.Module):
         self.default_name = camel2snake(prefix) + '_' + str(get_global_uid(camel2snake(prefix)))
         self.relative_name = ''
         reset_name(self, self._uid_prefixs)
-        self._input_shape:Optional[None,TensorShape,List[TensorShape]] = None
-        self._output_shape:Optional[None,TensorShape,List[TensorShape]]  = None
+        self._input_shape: Optional[None, TensorShape, List[TensorShape]] = None
+        self._output_shape: Optional[None, TensorShape, List[TensorShape]] = None
 
         self.input_filters = None
         self.input_spec = None
 
         self.keep_output = keep_output
-        self.register_buffer('_output_tensor',None,False)
+        self.register_buffer('_output_tensor', None, False)
 
         self._signature = None
         self._device = get_device()
@@ -388,7 +394,7 @@ class Layer(nn.Module):
     # Trick mypy into not applying contravariance rules to inputs by defining
     # forward as a value, rather than a function.  See also
     # https://github.com/python/mypy/issues/8795
-    def _forward_unimplemented(self, *input: Any) -> None:
+    def _forward_unimplemented(self, *input: Any, **kwargs) -> None:
         raise NotImplementedError
 
     r"""Defines the computation performed at every call.
@@ -404,15 +410,16 @@ class Layer(nn.Module):
     forward: Callable[..., Any] = _forward_unimplemented
 
     def get_root(self):
-        if not hasattr(self, '_nodes') or self._nodes is None or len(self._nodes) < 2:
+        if not hasattr(self, '_nodes') or self._nodes is None:
             self.is_root = True
             return self
-        elif self._nodes.value_list[0].is_root == True:
+        elif len(self._nodes) > 0 and self._nodes.value_list[0].is_root:
             return self._nodes.value_list[0]
         else:
             for name, node in self._nodes.item_list:
-                if node.is_root == True:
+                if node.is_root:
                     return node
+            return self
 
     # def forward(self, *input,**kwargs):
     #     r"""Defines the computation performed at every call.
@@ -508,7 +515,7 @@ class Layer(nn.Module):
         else:
             raise ValueError('Not valid module')
 
-    def build(self, input_shape:TensorShape):
+    def build(self, *input_shape: TensorShape):
         """ Do the shape inference and initialize weights and bias.
 
         `build' is a key method in trident, you can use  property `built' to check whether the layer do the build process.
@@ -520,7 +527,7 @@ class Layer(nn.Module):
         """
         pass
 
-    def rebuild(self, input_shape):
+    def rebuild(self, *input_shape):
         """ Do the shape inference and initialize weights and bias.
 
         `build' is a key method in trident, you can use  property `built' to check whether the layer do the build process.
@@ -542,9 +549,6 @@ class Layer(nn.Module):
             dummay_input = to_tensor(np.random.standard_normal((1,) + tuple(input_shape)).astype(np.float32)).to(get_device())
             out = self.forward(dummay_input)
 
-
-
-
     @property
     def trainable_weights(self) -> List[nn.Parameter]:
         """The list of trainable variables (parameters) of the module.
@@ -556,7 +560,7 @@ class Layer(nn.Module):
             necessary to ensure uniqueness.
 
         """
-        paras=[]
+        paras = []
         for p in self.parameters():
             if p.requires_grad:
                 paras.append(p)
@@ -636,25 +640,31 @@ class Layer(nn.Module):
         return self._device
 
     @device.setter
-    def device(self, value:str):
-        if isinstance(value,str):
+    def device(self, value: str):
+        if isinstance(value, str):
             self._device = value
+            self.to(value)
         elif isinstance(value, torch.device):
             self._device = value.type
+            self.to(value)
         else:
             print(value)
+            self.to(value)
 
     def cuda(self, device=None):
-        self.get_root().device ='cuda'
-        super().cuda(device=device)
+        if self.get_root().device != 'cuda':
+            self.get_root().device = 'cuda'
+            super().cuda(device=device)
 
     def cpu(self):
-        self.get_root().device='cpu'
-        super().cpu()
+        if self.get_root().device != 'cpu':
+            self.get_root().device = 'cpu'
+            super().cpu()
 
     def gpu(self, device=None):
-        self.get_root().device = 'cuda'
-        super().cuda(device)
+        if self.get_root().device != 'cuda':
+            self.get_root().device = 'cuda'
+            super().cuda(device)
 
     def to(self, *args, **kwargs):
         r"""Moves and/or casts the parameters and buffers.
@@ -733,8 +743,8 @@ class Layer(nn.Module):
             if not dtype.is_floating_point:
                 raise TypeError('nn.Module.to only accepts floating point '
                                 'dtypes, but got desired dtype={}'.format(dtype))
-        # if device is not None:
-        #     self.get_root().device=device.type
+        if device is not None and self.get_root()._device != device.type:
+            self.get_root()._device = device.type
 
         def convert(t):
             if convert_to_format is not None and t.dim() == 4:
@@ -752,21 +762,23 @@ class Layer(nn.Module):
         """Shape of input tensor,not including the batch axis."""
         return self._input_shape
 
-
     @input_shape.setter
     def input_shape(self, value) -> TensorShape:
         """ Setting the input_shape, means the layer get shape information and start to do the shape inferrence """
 
         if is_tensor(value) and value.ndim == 1 and value.dtype == torch.int32:
-            value = TensorShape(value)
+            value = TensorShape( [None,]+to_list(to_numpy(value)))
         elif isinstance(value, torch.Size):
-            value =TensorShape(value)
+            dims = [None,]+[d for d in value]
+            value = TensorShape(dims)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([isinstance(item, numbers.Integral) for item in value]):
-            value = TensorShape(value)
+            value = TensorShape((None,)+value)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([is_tensor(item) and ndim(item) == 1 and item.dtype == torch.int32 for item in value]):
-            value =[TensorShape(sh) for sh in value]
+            value = [TensorShape(sh) for sh in value]
+        elif isinstance(value, TensorShape):
+            pass
         else:
-            value =TensorShape(value)
+            value = TensorShape(value)
 
         if self._built == False or self._input_shape is None or self.input_filters is None:
             self._input_shape = value
@@ -777,27 +789,29 @@ class Layer(nn.Module):
                 if self._signature is None:
                     self._signature = Signature(name=self.name)
                 self._signature.inputs = OrderedDict()
-                if isinstance(self._input_shape,TensorShape):
+                if isinstance(self._input_shape, TensorShape):
                     self._signature.inputs['input'] = TensorSpec(shape=self._input_shape, name='input')
                 elif isinstance(self._input_shape, list):
                     for k in range(len(self._input_shape)):
                         self._signature.inputs['input_{0}'.format(k)] = TensorSpec(shape=self._input_shape[k], name='input_{0}'.format(k))
 
-
     @property
-    def output_shape(self) :
+    def output_shape(self):
         return self._output_shape
 
     @output_shape.setter
     def output_shape(self, value):
         if is_tensor(value) and value.ndim == 1 and value.dtype == torch.int32:
-            value = TensorShape(value)
+            value = TensorShape( [None,]+to_list(to_numpy(value)))
         elif isinstance(value, torch.Size):
-            value = TensorShape(value)
+            dims = [None, ] + [d for d in value]
+            value = TensorShape(dims)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([isinstance(item, numbers.Integral) for item in value]):
-            value = TensorShape(value)
+            value = TensorShape((None,) + value)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([is_tensor(item) and ndim(item) == 1 and item.dtype == torch.int32 for item in value]):
             value = [TensorShape(sh) for sh in value]
+        elif isinstance(value, TensorShape):
+            pass
         else:
             value = TensorShape(value)
 
@@ -806,8 +820,11 @@ class Layer(nn.Module):
             if self._signature is None:
                 self._signature = Signature(name=self.name)
             self._signature.outputs = OrderedDict()
-            if is_tensor(self._output_shape):
+            if isinstance(self._output_shape, TensorShape):
                 self._signature.outputs['output'] = TensorSpec(shape=self._output_shape, name='output')
+
+            elif is_tensor(self._output_shape):
+                self._signature.outputs['output'] = TensorSpec(shape=TensorShape(to_list(to_numpy(self._output_shape))), name='output')
             else:
                 for k in range(len(self._output_shape)):
                     self._signature.outputs['output_{0}'.format(k)] = TensorSpec(shape=self._output_shape[k], name='output_{0}'.format(k))
@@ -820,7 +837,7 @@ class Layer(nn.Module):
 
         """
         if self.is_root:
-            if self._signature is None or len(self._signature) == 0 or len(self._signature.outputs) == 0:
+            if self._signature is None or len(self._signature) == 0 or len(self._signature.inputs) == 0:
                 self._signature = Signature(name=self.name)
 
                 if self._input_shape is not None:
@@ -831,7 +848,7 @@ class Layer(nn.Module):
                             self._signature.inputs["input_{0}".format(i)] = TensorSpec(shape=TensorShape(self._input_shape[i]), name="input_{0}".format(i))
 
                 if self._output_shape is not None:
-                    if isinstance(self._output_shape,TensorShape) :
+                    if isinstance(self._output_shape, TensorShape):
                         self._signature.outputs["output"] = TensorSpec(shape=TensorShape(self._output_shape), name="output")
                     elif isinstance(self._output_shape, tuple):
                         for i in range(len(self._output_shape)):
@@ -859,7 +876,7 @@ class Layer(nn.Module):
             Output tensor or list of output tensors.
 
         """
-        if self.keep_output == False:
+        if not self.keep_output:
             return None
         return list(self._output_tensor) if isinstance(self._output_tensor, tuple) else self._output_tensor
 
@@ -876,9 +893,9 @@ class Layer(nn.Module):
 
     def save_onnx(self, file_path=''):
         input_shape = self.signature.inputs.value_list[0].shape.dims
-        input_shape[0]=1
+        input_shape[0] = 1
         self.eval()
-        x = cast(torch.randn(*input_shape, requires_grad=False),self.input_spec.dtype)
+        x = cast(torch.randn(*input_shape, requires_grad=False), self.input_spec.dtype)
         torch_out = self(x)
         folder, filename, ext = split_path(file_path)
         if filename == '':
@@ -902,18 +919,21 @@ class Layer(nn.Module):
         shutil.copy(save_path, save_path.replace('.onnx_', '.onnx'))
         os.remove(save_path)
 
-
-
     def _call_impl(self, *input, **kwargs):
-        is_all_numpy = True
-        is_built=self._built
+        is_all_numpy = False
+        is_built = self._built
 
-        #only do in the root
+        # only do in the root
         if self.is_root:
-            if isinstance(input,np.ndarray):
-                to_tensor(input)
+
+            if isinstance(input, (tuple)):
+                is_all_numpy = all([isinstance(inp, np.ndarray) for inp in input])
+                input = tuple([to_tensor(inp, device=get_device()) for inp in input])
             else:
-                is_all_numpy = False
+                if isinstance(input, np.ndarray):
+                    is_all_numpy = True
+                input = to_tensor(input, device=get_device())
+                input = (input,)
         for hook in itertools.chain(
                 _global_forward_pre_hooks.values(),
                 self._forward_pre_hooks.values()):
@@ -922,25 +942,25 @@ class Layer(nn.Module):
                 if not isinstance(result, tuple):
                     result = (result,)
                 input = result
-        if self._built == False:
+        if not self._built:
             inp = unpack_singleton(input)
             if is_tensor(inp):
-                shp= tensor_to_shape(inp)
+                shp = tensor_to_shape(inp,need_exclude_batch_axis=True)
                 self.input_filters = shp[self.filter_index]
-                self.input_shape =shp
-                self.input_spec=TensorSpec.tensor_to_spec(inp)
+                self.input_shape = shp
+                self.input_spec = TensorSpec.tensor_to_spec(inp)
                 del inp
             elif isinstance(input, (tuple, list)):
                 if isinstance(input[0], numbers.Number):
                     self.input_shape = TensorShape(list(input))
                 else:
-                    self.input_shape = tuple([tensor_to_shape(i.copy().detach()) for i in input if not isinstance(i, (list, tuple))])
+                    self.build(*[tensor_to_shape(inp, need_exclude_batch_axis=True) for inp in input])
 
             else:
                 self.input_shape = TensorShape(list(input))
                 print('input shou be tensor or tuple of tensor')
 
-            self._built=True
+            self._built = True
 
         if torch._C._get_tracing_state():
             result = self._slow_forward(*input, **kwargs)
@@ -950,12 +970,12 @@ class Layer(nn.Module):
             if hasattr(self, 'keep_output') and self.keep_output == True:
                 # make a op
                 self._output_tensor = identity(unpack_singleton(result))
-            if self._output_shape is None or is_built==False:
-                output= unpack_singleton(result)
+            if self._output_shape is None or is_built == False:
+                output = unpack_singleton(result)
                 if is_tensor(output):  # one output
                     self._output_shape = tensor_to_shape(output)
                 elif isinstance(output, (list, tuple)):
-                    output_shape =tuple([tensor_to_shape(item) for item in output  if not isinstance(item, (list, tuple))])
+                    output_shape = tuple([tensor_to_shape(item) for item in output if not isinstance(item, (list, tuple))])
                     # if not isinstance(item, (list,tuple)) lstm
                     self._output_shape = unpack_singleton(output_shape)
 
@@ -982,7 +1002,7 @@ class Layer(nn.Module):
                     functools.update_wrapper(wrapper, hook)
                     grad_fn.register_hook(wrapper)
 
-        if is_all_numpy == True and self.training == False and self.is_root==True:
+        if is_all_numpy == True and self.training == False and self.is_root == True:
             if is_tensor(result):
                 return to_numpy(result)
             elif isinstance(result, (list, tuple)):
@@ -1137,7 +1157,7 @@ class Sequential(Layer):
                     self.add_module(str(idx), module)
         self.to(self.device)
 
-    def build(self, input_shape:TensorShape):
+    def build(self, input_shape: TensorShape):
         """
 
         Args:
@@ -1166,20 +1186,18 @@ class Sequential(Layer):
             dummay_input = to_tensor(last_output.get_dummy_tensor()).to(self.device)
             out = module(dummay_input)
             self._modules[name] = module
-            self._output_shape = module.output_shape
+            self._output_shape = tensor_to_shape(out, need_exclude_batch_axis=True, is_singleton=False)
+            self.get_root().signature.outputs.value_list[0].shape=self._output_shape.copy()
         else:
             super(Sequential, self).add_module(name, module)
 
-        self._signature = None
-
     def remove_at(self, idx):
         self.__delitem__(idx)
         if len(self._modules) > 0:
             self._output_shape = self[-1]._output_shape
-            if isinstance(self._signature ,Signature) and len(self._signature.outputs)>0:
-                self._signature.outputs[self._signature.outputs.key_list[0]]=TensorSpec(shape=self[-1]._output_shape)
-            else:
-                self._signature = None
+            if isinstance(self._signature, Signature):
+                self._signature.outputs = OrderedDict()
+                self._signature.outputs['output'] = TensorSpec(shape=self[-1]._output_shape)
 
     def _get_item_by_idx(self, iterator, idx):
         """Get the idx-th item of the iterator"""
@@ -1190,6 +1208,25 @@ class Sequential(Layer):
         idx %= size
         return next(islice(iterator, idx, None))
 
+    def __getattr__(self, name):
+        if name in ['output', 'output_shape', '_output_shape']:
+            return self[-1].__getattr__(name)
+        if '_parameters' in self.__dict__:
+            _parameters = self.__dict__['_parameters']
+            if name in _parameters:
+                return _parameters[name]
+        if '_buffers' in self.__dict__:
+            _buffers = self.__dict__['_buffers']
+            if name in _buffers:
+                return _buffers[name]
+        if '_modules' in self.__dict__:
+            modules = self.__dict__['_modules']
+            if name in modules:
+                return modules[name]
+        if name in self.__dict__:
+            return self.__dict__[name]
+        raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
+
     def __getitem__(self, idx):
         if isinstance(idx, slice):
             returnDict = OrderedDict()
@@ -1273,7 +1310,9 @@ class ModuleList(Layer):
         if isinstance(idx, slice):
             return self.__class__(list(self._modules.values())[idx])
         else:
-            return self._modules[self._get_abs_string_index(idx)]
+            layer = self._modules[self._get_abs_string_index(idx)]
+            layer._nodes = OrderedDict()
+            return layer
 
     def __setitem__(self, idx, module):
         idx = self._get_abs_string_index(idx)
@@ -1384,10 +1423,9 @@ class ModuleDict(Layer):
         super(ModuleDict, self).__init__(name=None, keep_output=False, **kwargs)
         self.is_multicasting = is_multicasting
         if modules is not None:
-            if len(modules)>0:
+            if len(modules) > 0:
                 self.update(modules)
 
-
     # @_copy_to_script_wrapper
     def __getitem__(self, key: str) -> Layer:
         return self._modules[key]
@@ -1403,7 +1441,7 @@ class ModuleDict(Layer):
         return len(self._modules)
 
     # @_copy_to_script_wrapper
-    def __iter__(self) :
+    def __iter__(self):
         return iter(self._modules)
 
     # @_copy_to_script_wrapper
@@ -1478,7 +1516,7 @@ class ModuleDict(Layer):
                                      "; 2 is required")
                 self[m[0]] = m[1]
 
-    def build(self, input_shape:TensorShape):
+    def build(self, input_shape: TensorShape):
         """
 
         Args:
@@ -1488,23 +1526,24 @@ class ModuleDict(Layer):
 
         """
         if self._built == False and len(self._modules) > 0:
-            self._input_shape = input_shape
-            input_shape = tuple(to_numpy(input_shape))
-            dummay_input = random_normal((2,) + input_shape).to(self.device)
+            if self.is_root:
+                self._input_shape = input_shape
 
-            for name, module in self.items():
-                out = module(dummay_input)
-                module.input_shape = input_shape
-                module.output_shape = tensor_to_shape(out)
-            self._built = True
+                dummay_input = to_tensor(self.input_shape.get_dummy_tensor()).to(self.device)
+
+                for name, module in self.items():
+                    out = module(dummay_input)
+                    module.input_shape = input_shape
+                    module.output_shape = tensor_to_shape(out)
+                self._built = True
 
     def forward(self, x, **kwargs):
         if self.is_multicasting == True:
-            x=enforce_singleton(x)
+            x = enforce_singleton(x)
             results = OrderedDict()
             for name, module in self.items():
-                out= module(x)
-                results[name]=out
+                out = module(x)
+                results[name] = out
             return results
         else:
             raise NotImplementedError()
@@ -1601,7 +1640,7 @@ def print_network(net, verbose=False):
     logging.info('Total number of parameters: %d\n' % num_params)
 
 
-def summary(model, input_size, batch_size=-1, device="cuda"):
+def summary(model, input_size, batch_size=1, device="cuda"):
     def register_hook(module):
         def hook(module, input, output):
             # class_name =module.re    module.name   # str(module.__class__).split(".")[-1].split("'")[0]
@@ -1668,13 +1707,12 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
         module.relative_name = name
     # multiple inputs to the network
 
-
     # prevent pytorch 'ValueError: Expected more than 1 value per channel when training, got input size ....
     model.to(get_device())
     model.eval()
 
     # batch_size of 2 for batchnorm
-    x = [ to_tensor(shape.get_dummy_tensor()).to(get_device()) for shape in input_size]
+    x = [to_tensor(shape.get_dummy_tensor()).to(get_device()) for shape in input_size]
     # p    rint(type(x[0]))
 
     # create properties
@@ -1693,7 +1731,7 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
         h.remove()
 
     print("--------------------------------------------------------------------------------------------------------------------------------")
-    line_new = "{0:^50s} {1:^25s}  {2:^20s} {3:^8s}  {4:^8s}  {5:^25s}".format("Layer (type)", "Output Shape", "Weight ", "Bias", "Param #", "FLOPS #")
+    line_new = "{0:^50s} {1:<25s}  {2:<20s} {3:<8s}  {4:<8s}  {5:<25s}".format("Layer (type)", "Output Shape", "Weight ", "Bias", "Param #", "FLOPS #")
     print(line_new)
     print("==============================================================================")
     total_params = 0
@@ -1714,12 +1752,13 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
         #     summary[layer]["flops"][0]
         # )
 
-        line_new = (layer + "  " + class_name).ljust(50, ' ') \
-                   + (is_keep + str(summary[layer]["output_shape"])).ljust(25, ' ') \
-                   + str(summary[layer]["weight"] if 'weight' in summary[layer] else '').ljust(20, ' ') \
-                   + str(summary[layer]["bias"] if 'bias' in summary[layer] else '').ljust(8, ' ') \
-                   + '{:,}'.format(summary[layer]["nb_params"]).ljust(8, ' ') \
-                   + '{:,}'.format(summary[layer]["flops"].sum()).ljust(25, ' ')
+        line_new = "{0:<50s} {1:<25s}  {2:<20s} {3:^8s}  {4:,}  {5:,}  ".format((layer + "  [" + class_name + "]").ljust(50, ' '),
+                                                                                (is_keep + str([None] + summary[layer]["output_shape"][1:])).ljust(25, ' '),
+                                                                                str(summary[layer]["weight"] if 'weight' in summary[layer] else '').ljust(20, ' '),
+                                                                                str(summary[layer]["bias"] if 'bias' in summary[layer] else '').ljust(8, ' '),
+                                                                                summary[layer]["nb_params"],
+                                                                                summary[layer]["flops"].sum()
+                                                                                )
 
         total_params += summary[layer]["nb_params"]
         flops += float(summary[layer]["flops"])
@@ -1794,7 +1833,6 @@ def normalize_padding(padding, rank):
     return padding
 
 
-
 def print_mem_stack():  # pragma: no cover
     """
 
@@ -1824,7 +1862,6 @@ def count_mem_items():  # pragma: no cover
     return nb_params, nb_tensors
 
 
-
 def get_human_readable_count(number):
     """
     Abbreviates an integer number with K, M, B, T for thousands, millions,
@@ -1868,15 +1905,16 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
         try:
             arg_map = OrderedDict()
             if isinstance(fn, Layer):
+                _device = fn.get_root().device
                 for arg in fn.signature.inputs.key_list:
                     if arg in data_feed:
-                        arg_map[arg] = to_tensor(data[data_feed[arg]]).to(get_device())
+                        arg_map[arg] = to_tensor(data[data_feed[arg]]).to(_device)
                     elif arg in data:
-                        arg_map[arg] = to_tensor(data[arg]).to(get_device())
+                        arg_map[arg] = to_tensor(data[arg]).to(_device)
                     else:
                         raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
                 # print('arg_map',arg_map.key_list)
-                if get_session_value('amp_available') == True and get_session_value('is_amp_enable') == True and get_device()=='cuda':
+                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1887,8 +1925,10 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
                 return out
             elif hasattr(fn, 'signature') and callable(fn):
                 for arg in fn.signature.inputs.key_list:
+
                     if arg in data_feed:
-                        arg_map[arg] = data[data_feed[arg]].to(get_device())
+                        if data_feed[arg] in data:
+                            arg_map[arg] = data[data_feed[arg]].to(get_device())
                     elif arg in data:
                         arg_map[arg] = data[arg].to(get_device())
 
@@ -1896,7 +1936,7 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
 
                         raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
                 # print('arg_map', arg_map.key_list)
-                if get_session_value('amp_available') == True and get_session_value('is_amp_enable') == True and get_device()=='cuda' :
+                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1916,7 +1956,7 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
                     else:
                         arg_map[arg] = ''
                 # print('arg_map', arg_map.key_list)
-                if get_session_value('amp_available') == True and get_session_value('is_amp_enable') == True and get_device()=='cuda':
+                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1976,18 +2016,35 @@ def fix_layer(layer: Layer):
         layer._uid_prefixs = {}
     reset_name(layer, layer._uid_prefixs)
 
-    if layer._input_shape is not None and not isinstance(layer._input_shape, TensorShape):
-        layer._input_shape =TensorShape(layer._input_shape)
+    if 'ssd' in layer.__class__.__base__.__name__.lower() or 'yolo' in layer.__class__.__base__.__name__.lower():
+        if hasattr(layer, 'nms_threshold'):
+            delattr(layer, 'nms_threshold')
+        if hasattr(layer, 'detection_threshold'):
+            delattr(layer, 'detection_threshold')
 
-    if layer._output_shape is not None and not isinstance(layer._output_shape, TensorShape):
-        layer._output_shape =TensorShape(layer._output_shape)
+    if layer._input_shape is not None and isinstance(layer._input_shape, tuple):
+        layer._input_shape = tuple([TensorShape(to_numpy(item)) for item in TensorShape(layer._input_shape)])
+    elif layer._input_shape is not None and not isinstance(layer._input_shape, TensorShape):
+        layer._input_shape = TensorShape(to_numpy(layer._input_shape))
+    elif layer._input_shape is not None and isinstance(layer._input_shape, TensorShape) and is_tensor(layer._input_shape.dims[0]):
+        layer._input_shape = TensorShape([d.item() for d in layer._input_shape.dims])
+
+    if layer._output_shape is not None and isinstance(layer._output_shape, tuple):
+        layer._output_shape = tuple([TensorShape(to_numpy(item)) for item in TensorShape(layer._output_shape)])
+    elif layer._output_shape is not None and not isinstance(layer._output_shape, TensorShape):
+        layer._output_shape = TensorShape(layer._output_shape)
+    elif layer._output_shape is not None and isinstance(layer._output_shape, TensorShape) and isinstance(layer._output_shape.dims[0], torch.Size) and len(
+            layer._output_shape.dims[0]) == 1:
+        layer._output_shape = TensorShape([None if d is None else d[0] for d in layer._output_shape.dims])
+    elif layer._output_shape is not None and isinstance(layer._output_shape, TensorShape) and isinstance(layer._output_shape.dims[0], torch.Size):
+        layer._output_shape = tuple([TensorShape(to_numpy(d)) for d in layer._output_shape.dims])
 
     if not hasattr(layer, 'get_toot'):
         setattr(layer, 'get_root', MethodType(get_root, layer))
 
     for module in layer.modules():
         class_name = module.__class__.__name__
-        #check for root
+        # check for root
         if module.uuid == layer.uuid:
             module.is_root = True
         else:
@@ -2026,10 +2083,11 @@ def fix_layer(layer: Layer):
             if module.input_shape is not None:
                 module.input_spec = TensorSpec(shape=TensorShape(module._input_shape))
         # fix for shape definition
-        if not isinstance(module._input_shape,TensorShape):
-            module._input_shape=TensorShape(module._input_shape)
-        if not isinstance(module._output_shape,TensorShape):
-            module._output_shape=TensorShape(module._output_shape)
+        if not isinstance(module._input_shape, TensorShape):
+            module._input_shape = TensorShape(to_numpy(module._input_shape))
+
+        if not isinstance(module._output_shape, TensorShape):
+            module._output_shape = TensorShape(to_numpy(module._output_shape))
 
         if not hasattr(module, 'batch_index'):
             setattr(module, 'batch_index', 0)
@@ -2057,7 +2115,7 @@ def fix_layer(layer: Layer):
             if not hasattr(module, 'use_spectral'):
                 module.use_spectral = False
 
-    if layer.is_root == True and not hasattr(layer, 'signature'):
+    if layer.is_root == True and (hasattr(layer, 'signature') or layer._signature is None or (is_tensor(layer._signature.inputs[0].shape))):
         layer._signature = Signature()
         if layer._input_shape is not None:
             if is_tensor(layer._input_shape):
