@@ -5,10 +5,16 @@ from __future__ import print_function
 
 import pickle
 import io
+import re
+import struct
+from xml.etree import ElementTree
 import numpy as np
 import copy
+import cv2
+from trident.data.utils import _delete_h
 
-from trident.data.image_common import list_pictures
+from trident.backend.opencv_backend import image2array, array2image
+from trident.data.image_common import list_images
 from trident.data.data_provider import *
 from trident.data.dataset import *
 from trident.data.mask_common import *
@@ -360,7 +366,7 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
     base_folder = sanitize_path(base_folder)
     if base_folder is not None and os.path.exists(base_folder):
         print(base_folder)
-        if folder_as_label == True:
+        if folder_as_label:
             class_names = []
             if classes is not None and isinstance(classes, list) and len(classes) > 0:
                 class_names.extend(classes)
@@ -376,15 +382,17 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
             imgs = []
             for i in range(len(class_names)):
                 class_name = class_names[i]
-                class_imgs = glob.glob(base_folder + '/{0}/*.*g'.format(class_name))
+                class_imgs = list_images(base_folder + '/{0}'.format(class_name))
                 if len(class_imgs)==0:
                     class_imgs = glob.glob(base_folder + '/{0}/*/*.*g'.format(class_name))
-                print(base_folder + '/{0}/*.*g'.format(class_name))
+
+                else:
+                    print(base_folder + '/{0}/*.*g'.format(class_name))
                 print(len(class_imgs))
                 labels.extend([i] * len(class_imgs))
                 imgs.extend(class_imgs)
 
-            imagedata = ImageDataset(imgs, object_type=ObjectType.rgb, get_image_mode=GetImageMode.processed)
+            imagedata = ImageDataset(imgs, object_type=ObjectType.rgb)
             print('extract {0} images...'.format(len(imagedata)))
             labelsdata = LabelDataset(labels,object_type=ObjectType.classification_label)
             labelsdata.binding_class_names(class_names)
@@ -394,8 +402,8 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
             dataset.binding_class_names(class_names)
 
         else:
-            imgs = glob.glob(base_folder + '/*.*g')
-            imagedata = ImageDataset(imgs, object_type=ObjectType.rgb, get_image_mode=GetImageMode.processed)
+            imgs = list_images(base_folder)
+            imagedata = ImageDataset(imgs, object_type=ObjectType.rgb)
             traindata = Iterator(data=imagedata)
             dataset = DataProvider(dataset_name, traindata=traindata)
         return dataset
@@ -484,6 +492,7 @@ def load_lfw(format='aligned_face', is_paired=False):
 
     tar_file_path = os.path.join(dirname, 'lfw.tgz')
     if format=='aligned_face':
+        tar_file_path = os.path.join(dirname, 'lfw-crop.tar')
         download_file_from_google_drive('1sVbU8NHC7kzDkqByTtPiib9_EY23eeW7', dirname, 'lfw-crop.tar')
     elif format=='raw':
         download_file('http://vis-www.cs.umass.edu/lfw/lfw.tgz', dirname, 'lfw.tgz')
@@ -493,9 +502,9 @@ def load_lfw(format='aligned_face', is_paired=False):
 
     # if only_face_area:
     #     if only_aligned_face:
-    tar_file_path = os.path.join(dirname, 'lfw.tgz')
+
     extract_archive(tar_file_path, dirname, archive_format='tar')
-    extract_path = os.path.join(dirname,'lfw-crop' if format=='aligned_face' else 'lfw')
+
     # if _backend=='tensorflow':
     #     import trident.models.tensorflow_mtcnn as mtcnn
     # else:
@@ -504,8 +513,8 @@ def load_lfw(format='aligned_face', is_paired=False):
     # detector = mtcnn.Mtcnn(pretrained=True,verbose=False)
     # detector.minsize = 70
     #
-    # detector.detection_threshould = [0.8, 0.8, 0.9]
-    # detector.nms_threshould = [0.5, 0.5, 0.3]
+    # detector.detection_threshold = [0.8, 0.8, 0.9]
+    # detector.nms_threshold = [0.5, 0.5, 0.3]
     # faces = glob.glob(os.path.join(dirname, 'lfw-deepfunneled' if only_aligned_face else 'lfw') + '/*/*.*g')
     # faces_crop=glob.glob(os.path.join(dirname, 'lfw-crop')+ '/*/*.*g')
     # if len(faces_crop)==0 or len(faces_crop)!=len(faces):
@@ -518,10 +527,10 @@ def load_lfw(format='aligned_face', is_paired=False):
     #         else:
     #             make_dir_if_need(new_path)
     #             img = read_image(img_path)
-    #             detector.detection_threshould = [0.8, 0.8, 0.9]
+    #             detector.detection_threshold = [0.8, 0.8, 0.9]
     #             results = detector.infer_single_image(img_path)
     #             if len(results)==0 or results is None:
-    #                 detector.detection_threshould = [0.5, 0.7, 0.8]
+    #                 detector.detection_threshold = [0.5, 0.7, 0.8]
     #                 results = detector.infer_single_image(img_path)
     #
     #             results = detector.rerec(to_tensor(results), img.shape)
@@ -552,7 +561,7 @@ def load_lfw(format='aligned_face', is_paired=False):
     #         sys.stdout.flush()
     #
     #
-    data_provider = load_folder_images(dataset_name, os.path.join(dirname, 'lfw-crop'))
+    data_provider = load_folder_images(dataset_name,os.path.join(dirname,'lfw-crop' if format=='aligned_face' else 'lfw-deepfunneled'))
 
     if is_paired:
         storage=OrderedDict()
@@ -645,28 +654,43 @@ def load_examples_data(dataset_name):
         dataset = load_folder_images(dataset_name, dirname, folder_as_label=True)
         return dataset
     elif dataset_name == 'examples_nsfw':
-        download_file_from_google_drive('1EXpV2QUrSFJ7zJn8NqtqFl1k6HvXsUzp', dirname, 'nsfw.tar')
         tar_file_path = os.path.join(dirname, 'nsfw.tar')
+        if os.path.exists(tar_file_path) and get_file_create_time(tar_file_path)<datetime.datetime(2021, 2, 20, 0, 0, 0).timestamp():
+            os.remove(tar_file_path)
+            if os.path.exists(os.path.join(dirname,'porn_detection_data.pkl')):
+                os.remove(os.path.join(dirname,'porn_detection_data.pkl'))
+            _delete_h(dirname)
+        download_file_from_google_drive('1EXpV2QUrSFJ7zJn8NqtqFl1k6HvXsUzp', dirname, 'nsfw.tar')
+
         extract_path = os.path.join(dirname, 'nsfw')
         extract_archive(tar_file_path, dirname, archive_format='tar')
-        trainData = np.load(os.path.join(dirname, 'train_porn_detector64_small.npy'), allow_pickle=True)
-        testData = np.load(os.path.join(dirname, 'test_porn_detector64_small.npy'), allow_pickle=True)
+        folders = ['drawings', 'hentai', 'neutral', 'porn', 'sexy']
+        data=unpickle(os.path.join(dirname,'porn_detection_data.pkl'))
 
-        trainarray = ImageDataset(np.array(trainData[0].tolist()).transpose([0, 2, 3, 1]),
-                                  object_type=ObjectType.rgb, get_image_mode=GetImageMode.processed)
-        trainlabel = LabelDataset(trainData[1].tolist(),object_type=ObjectType.classification_label)
+        trainData = []
+        testData = []
+        trainLabel = []
+        testLabel = []
+        for n  in range(5):
+            folder=folders[n]
+            trainData.extend(data[folder]['train'])
+            trainLabel.extend([n]*len(data[folder]['train']))
+            testData.extend(data[folder]['test'])
+            testLabel.extend([n] * len(data[folder]['test']))
+
+        trainarray = ImageDataset(trainData,object_type=ObjectType.rgb)
+        trainlabel = LabelDataset(trainLabel,object_type=ObjectType.classification_label)
         train_iter = Iterator(data=trainarray, label=trainlabel)
 
-        testarray = ImageDataset(np.array(testData[0].tolist()).transpose([0, 2, 3, 1]),
-                                 object_type=ObjectType.rgb, get_image_mode=GetImageMode.processed)
-        testlabel = LabelDataset(testData[1].tolist(),object_type=ObjectType.classification_label)
+        testarray = ImageDataset(testData,object_type=ObjectType.rgb)
+        testlabel = LabelDataset(testLabel,object_type=ObjectType.classification_label)
         test_iter = Iterator(data=testarray, label=testlabel)
         print('training images: {0}  test images:{1}'.format(len(trainarray), len(testarray)))
 
         dataset = DataProvider(dataset_name, traindata=train_iter, testdata=test_iter)
         dataset.binding_class_names(['drawing', 'hentai', 'neutral', 'porn', 'sexy'], 'en-us')
-        dataset.binding_class_names(['繪畫', '色情漫畫', '中性', '色情', '性感'], 'zh-tw')
         dataset.binding_class_names(['绘画', '色情漫画', '中性', '色情', '性感'], 'zh-cn')
+        dataset.binding_class_names(['繪畫', '色情漫畫', '中性', '色情', '性感'], 'zh-tw')
         dataset.scenario = 'train'
         return dataset
     elif dataset_name == 'examples_simpsons':
@@ -674,22 +698,22 @@ def load_examples_data(dataset_name):
         tar_file_path = os.path.join(dirname, 'simpsons.tar')
         extract_path = os.path.join(dirname, 'simpsons')
         extract_archive(tar_file_path, extract_path, archive_format='tar')
-        dataset = load_folder_images(dataset_name, extract_path, folder_as_label=False)
-        dataset.traindata.label = RandomNoiseDataset(shape=(100), random_mode='normal')
-        print('get simpsons images :{0}'.format(len(dataset)))
-        return dataset
+        data_provider = load_folder_images(dataset_name, extract_path, folder_as_label=False)
+        data_provider.traindata.unpair = RandomNoiseDataset(shape=(100), random_mode='normal')
+        print('get simpsons images :{0}'.format(len(data_provider.traindata.data.items)))
+        return data_provider
     elif dataset_name == 'examples_horse2zebra':
         download_file_from_google_drive('1pqj-T90Vh4wVNBV09kYZWgVPsZUA2f7U', dirname, 'horse2zebra.tar')
         tar_file_path = os.path.join(dirname, 'horse2zebra.tar')
         extract_path = os.path.join(dirname, 'horse2zebra')
         extract_archive(tar_file_path, dirname, archive_format='tar')
-        trainA = ImageDataset(list_pictures(os.path.join(dirname, 'trainA')), object_type=ObjectType.rgb,
+        trainA = ImageDataset(list_images(os.path.join(dirname, 'trainA')), object_type=ObjectType.rgb,
                               get_image_mode=GetImageMode.processed)
-        trainB = ImageDataset(list_pictures(os.path.join(dirname, 'trainB')), object_type=ObjectType.rgb,
+        trainB = ImageDataset(list_images(os.path.join(dirname, 'trainB')), object_type=ObjectType.rgb,
                               get_image_mode=GetImageMode.processed)
-        testA = ImageDataset(list_pictures(os.path.join(dirname, 'testA')), object_type=ObjectType.rgb,
+        testA = ImageDataset(list_images(os.path.join(dirname, 'testA')), object_type=ObjectType.rgb,
                              get_image_mode=GetImageMode.processed)
-        testB = ImageDataset(list_pictures(os.path.join(dirname, 'testB')), object_type=ObjectType.rgb,
+        testB = ImageDataset(list_images(os.path.join(dirname, 'testB')), object_type=ObjectType.rgb,
                              get_image_mode=GetImageMode.processed)
         train_iter = Iterator(data=trainA, unpair=trainB)
         test_iter = Iterator(data=testA, unpair=testB)
@@ -722,8 +746,8 @@ def load_examples_data(dataset_name):
         imgs = glob.glob(os.path.join(dirname, 'images', '*.*g'))
         masks = glob.glob(os.path.join(dirname, 'masks', '*.png'))
 
-        imgdata = ImageDataset(images=imgs, object_type=ObjectType.rgb)
-        mskdata = MaskDataset(masks=masks, object_type=ObjectType.color_mask)
+        imgdata = ImageDataset(images=imgs, object_type=ObjectType.rgb,symbol='image')
+        mskdata = MaskDataset(masks=masks, object_type=ObjectType.color_mask,symbol='mask')
 
         def parse_code(l):
             if len(l.strip().split("\t")) == 2:
@@ -781,37 +805,89 @@ def load_examples_data(dataset_name):
         return data_provider
 
     elif dataset_name == 'examples_facelandmarks':
-        download_file_from_google_drive('1aJhxN9IqsxuayhRTm-gmxk6PiLe5wm9X', dirname, 'beauty.tar')
-        tar_file_path = os.path.join(dirname, 'beauty.tar')
+        download_file_from_google_drive('1GtswQBAHPa_bXaB4tW2uOOQ8Lxfz2L5B', dirname, 'ibug_300W.tar')
+        tar_file_path = os.path.join(dirname, 'ibug_300W.tar')
         extract_archive(tar_file_path, dirname, archive_format='tar')
-        # 讀取圖片數據
-        images_dict = {}
-        with open(os.path.join(dirname, 'images_dict.pkl'), 'rb') as fp:
-            images_dict = pickle.load(fp)
+        root_dir=os.path.join(dirname, 'ibug_300W_large_face_landmark_dataset')
+        image_paths = {}
+        landmarks = {}
+        crops = {}
 
-        f = open(os.path.join(dirname, 'All_Ratings.txt'), encoding='utf-8-sig').readlines()
-        imgs = []
-        landmarks = []
-        ratings = []
-        for row in f:
-            data = row.strip().split('\t')
-            if 'images\\' + data[0] in images_dict:
-                img = images_dict['images\\' + data[0]][0]
-                img = img.transpose([2, 0, 1])[::-1].transpose([1, 2, 0])
-                imgs.append(img)
-                landmark = images_dict['images\\' + data[0]][1].astype(np.float32)
-                landmarks.append(landmark)
-                rating = np.zeros(2)
-                if 'm' in data[0]:
-                    rating[0] = 1
-                if 'w' in data[0]:
-                    rating[1] = 1
-                ratings.append(rating)
-        print('{0} faces loaded...'.format(len(imgs)))
-        imgdata = ImageDataset(images=imgs, object_type=ObjectType.rgb, symbol='faces')
-        landmarkdata = LandmarkDataset(landmarks=landmarks, object_type=ObjectType.landmarks, symbol='target_landmarks')
-        labeldata = LabelDataset(data=ratings, object_type=ObjectType.classification_label, symbol='target_label')
-        data_provider = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=Dataset.zip(landmarkdata, labeldata)))
+        for mode in ['train','test']:
+            make_dir_if_need(os.path.join(dirname, 'crops',mode))
+            tree = ElementTree.parse(os.path.join(root_dir, 'labels_ibug_300W_{0}.xml'.format(mode)))
+            root = tree.getroot()
+            image_paths[mode]=[]
+            landmarks[mode] = []
+            crops[mode] = []
+
+            offset=5
+            for j in tqdm(range(len(root[2]))):
+                try:
+                    filename=root[2][j]
+                    landmark = []
+                    for num in range(68):
+                        x_coordinate = int(filename[0][num].attrib['x'])
+                        y_coordinate = int(filename[0][num].attrib['y'])
+                        landmark.append([x_coordinate, y_coordinate])
+                    landmark=np.asarray(landmark)
+
+                    crop = filename[0].attrib
+                    for k in crop.keys():
+                        crop[k] = int(crop[k]) if isinstance(crop[k], str) else crop[k]
+                    for k in crop.keys():
+                        if k=='top' and int(landmark[:,1].min())<int(crop[k]):
+                            crop[k] = int( landmark[:,1].min())
+                            crop[ 'height']+=crop[k]-int(landmark[:,1].min())
+                        elif k=='left' and int(landmark[:,0].min())<int(crop[k]):
+                            crop[k] = int( landmark[:,0].min())
+                            crop['width']+= crop[k] - int(landmark[:, 0].min())
+                        elif k == 'width' and int(landmark[:, 0].max()-landmark[:, 0].min()) > int(crop[k]):
+                            crop[k] = int(landmark[:, 0].max()-landmark[:, 0].min())
+                        elif k == 'height' and int(landmark[:, 1].max()-landmark[:, 1].min()) > int(crop[k]):
+                            crop[k] = int(landmark[:, 1].max()-landmark[:, 1].min())
+
+                    crop['left']-=offset
+                    crop['top'] -= offset
+                    crop['width'] += 2*offset
+                    crop['height'] += 2*offset
+
+
+                    landmark[:,0]-=crop['left']
+                    landmark[:, 1] -= crop['top']
+
+
+                    if not os.path.exists(os.path.join(dirname, 'crops', mode, '{0}.png'.format(j))):
+                        im=image2array(os.path.join(root_dir, filename.attrib['file']))
+                        if im.ndim==2:
+                            im=cv2.cvtColor(im,cv2.COLOR_GRAY2RGB)
+                        im=im[crop['top']:min(crop['top']+crop['height'],im.shape[0]),crop['left']:min(crop['left']+crop['width'],im.shape[1]),:]
+
+                        if max(im.shape[:2])/max(min(im.shape[:2]),0)<=5:
+
+                            array2image(im).save(os.path.join(dirname, 'crops',mode,'{0}.png'.format(j)))
+                            image_paths[mode].append(os.path.join(dirname, 'crops', mode, '{0}.png'.format(j)))
+                            crops[mode].append(crop)
+                            landmarks[mode].append(landmark)
+                        del im
+                    else:
+                        #im = image2array(os.path.join(dirname, 'crops',mode,'{0}.png'.format(j)))
+                        image_paths[mode].append(os.path.join(dirname, 'crops',mode,'{0}.png'.format(j)))
+                        crops[mode].append(crop)
+                        landmarks[mode].append(landmark)
+
+                    if j%100==0:
+                        gc.collect()
+                except Exception as e:
+                   pass
+
+        print('ibug 300w train dataset: images: {0} landmarks:{1} \n'.format(len(image_paths['train']),len(landmarks['train'])))
+        print('ibug 300w test dataset: images: {0} landmarks:{1} \n'.format(len(image_paths['test']), len(landmarks['test'])))
+        imdata=ImageDataset(images=image_paths['train'],symbol='faces',object_type=ObjectType.rgb)
+        landmarkdata = LandmarkDataset(landmarks=landmarks['train'], symbol='landmarks',object_type=ObjectType.landmarks)
+        imtestdata = ImageDataset(images=image_paths['test'], symbol='faces',object_type=ObjectType.rgb)
+        landmarktestdata = LandmarkDataset(landmarks=landmarks['test'], symbol='landmarks',object_type=ObjectType.landmarks)
+        data_provider=DataProvider(traindata=Iterator(data=imdata,label=landmarkdata),testdata=Iterator(data=imtestdata,label=landmarktestdata))
         return data_provider
 
     elif dataset_name == 'examples_antisproofing':
