@@ -1,22 +1,33 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import builtins
+import copy
+import inspect
 import itertools
 import locale
 import os
 import random
 import string
+import uuid
 import warnings
 from typing import List
 
 import numpy as np
+from trident import context
+
+from trident.misc.ipython_utils import is_in_ipython
+
+from trident.data.vision_transforms import Unnormalize, Normalize
+
+from trident.data.transform import Transform
 
 try:
     from urllib.request import urlretrieve
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 from trident.backend.common import *
+from trident.backend.opencv_backend import *
 from trident.data.text_common import *
 from trident.data.image_common import *
 from trident.data.label_common import *
@@ -40,7 +51,7 @@ class ImageDataProvider(object):
 
     def __init__(self, dataset_name='', traindata=None, testdata=None, minibatch_size=8, mode='tuple', **kwargs):
         self.__name__ = dataset_name
-
+        self.uuid = uuid.uuid4().node
         self.traindata = traindata
         self.testdata = testdata
         self.annotations = {}
@@ -73,6 +84,8 @@ class ImageDataProvider(object):
         self._label_transform_funcs = []
         self._paired_transform_funcs = []
         self.spatial_transform_funcs = []
+        cxt=context._context()
+        cxt.regist_data_provider(self)
 
     @property
     def signature(self):
@@ -131,14 +144,12 @@ class ImageDataProvider(object):
     def image_transform_funcs(self, value):
         self._image_transform_funcs = value
         if self.traindata is not None and hasattr(self.traindata.data, 'transform_funcs'):
-            self.traindata.data.transform_funcs = self._image_transform_funcs
-            if len(self.traindata.unpair) > 0:
-                self.traindata.unpair.transform_funcs = self._image_transform_funcs
+            self.traindata.data.transform_funcs =copy.deepcopy(value)
+
             self.traindata.update_data_template()
         if self.testdata is not None and len(self.testdata.data) > 0 and hasattr(self.testdata.data, 'transform_funcs'):
-            self.testdata.data.transform_funcs = self._image_transform_funcs
-            if len(self.testdata.unpair) > 0:
-                self.testdata.unpair.transform_funcs = self._image_transform_funcs
+            self.testdata.data.transform_funcs =copy.deepcopy(value)
+
             self.testdata.update_data_template()
 
     def image_transform(self, img_data):
@@ -148,10 +159,8 @@ class ImageDataProvider(object):
             return image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
             for fc in self.image_transform_funcs:
-                if not fc.__qualname__.startswith('random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
-                        fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
+                if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption:
                     img_data = fc(img_data)
-
             img_data = image_backend_adaption(img_data)
 
             return img_data
@@ -160,25 +169,24 @@ class ImageDataProvider(object):
 
     @property
     def reverse_image_transform_funcs(self):
-        return_list = []
-        return_list.append(reverse_image_backend_adaption)
+        return_list = [reverse_image_backend_adaption]
         for i in range(len(self.image_transform_funcs)):
             fn = self.image_transform_funcs[-1 - i]
-            if fn.__qualname__ == 'normalize.<locals>.img_op':
-                return_list.append(unnormalize(fn.mean, fn.std))
-        # return_list.append(array2image)
+            if (inspect.isfunction(fn) and fn.__qualname__ == 'normalize.<locals>.img_op') or (isinstance(fn, Transform) and fn.name == 'normalize'):
+                return_list.append(Unnormalize(fn.mean, fn.std))
+        #return_list.append(array2image)
         return return_list
 
     def reverse_image_transform(self, img_data: np.ndarray):
         if img_data.ndim == 4:
-            return np.asarray([self.reverse_image_transform(im) for im in img_data])
+            return np.array([self.reverse_image_transform(im) for im in img_data])
         if len(self.reverse_image_transform_funcs) == 0:
             return reverse_image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 img_data = fc(img_data)
-            img_data = reverse_image_backend_adaption(img_data)
+            #img_data = reverse_image_backend_adaption(img_data)
 
         return img_data
 
@@ -205,17 +213,20 @@ class ImageDataProvider(object):
         self._paired_transform_funcs = value
 
         if self.traindata is not None and hasattr(self.traindata, 'paired_transform_funcs'):
-            self.traindata.paired_transform_funcs = self._paired_transform_funcs
+            self.traindata.paired_transform_funcs =copy.deepcopy(value)
+
+
             self.traindata.update_data_template()
 
         if self.testdata is not None and hasattr(self.testdata, 'paired_transform_funcs'):
-            self.testdata.paired_transform_funcs = self._paired_transform_funcs
+            self.testdata.paired_transform_funcs =copy.deepcopy(value)
             self.testdata.update_data_template()
 
     @property
     def class_names(self):
         return self._class_names
 
+
     @class_names.setter
     def class_names(self, value):
         self._class_names = value
@@ -228,11 +239,12 @@ class ImageDataProvider(object):
         if not isinstance(self.traindata.data,ImageDataset):
             return None
         else:
-            if key is None :
-                data, label = self.next()
+            if key is None and isinstance(self.traindata.data,ImageDataset):
+                data= self.next()
+                data=enforce_singleton(data)
                 data = self.reverse_image_transform(data)
                 if is_concate:
-                    data = np.concatenate([img for img in data], axis=1)
+                    data = np.concatenate([img for img in data], axis=-1 if data[0].ndim==2 or (data[0].ndim==3 and data[0].shape[0] in [1,3,4]) else -2)
                     return array2image(data)
                 else:
                     return [array2image(img) for img in data]
@@ -241,17 +253,36 @@ class ImageDataProvider(object):
                 stop = key.stop
                 results=[]
                 for k in range(start,stop,1):
-                    img= self.reverse_image_transform(self.traindata.data.__getitem__(k))
+                    img=self.traindata.data.__getitem__(k)
+
+                    if isinstance(img, np.ndarray):
+                        for fc in self.image_transform_funcs:
+                            if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption and  fc is not Normalize and fc is not normalize:
+                                img = fc(img)
                     results.append(img)
                 if is_concate:
-                    results = np.concatenate(results, axis=1)
+                    results = np.concatenate(results, axis=-1 if results[0].ndim==2 or (results[0].ndim==3 and results[0].shape[0] in [1,3,4]) else -2)
                     return array2image(results)
                 else:
                     return [array2image(img) for img in results]
             elif isinstance(key, int):
-                img=self.reverse_image_transform(self.traindata.data.__getitem__(key))
+                img=self.traindata.data.__getitem__(key)
+                if isinstance(img, np.ndarray):
+                    for fc in self.image_transform_funcs:
+                        if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption and fc is not Normalize and fc is not normalize:
+                            img = fc(img)
                 return array2image(img)
 
+    def label_statistics(self):
+        if isinstance(self.traindata.label,LabelDataset):
+            unique, counts = np.unique(np.array(self.traindata.label.items), return_counts=True)
+            max_len=builtins.max([get_string_actual_length(item) for item in self.class_names[self.__default_language__]])+5
+            for i in range(len(unique)):
+                class_names=''.join([' ']*max_len) if self.class_names is None or len(self.class_names) == 0 else self.index2label(unique[i])+''.join([' ']*(max_len-get_string_actual_length(self.index2label(unique[i]))))
+                bar=['█']*int(builtins.round(50*counts[i] / float(len(self.traindata.label.items))))
+                print('{0:<10} {1} {2:<50} {3:,} ({4:.3%})'.format(unique[i],class_names,''.join(bar),counts[i],counts[i]/float(len(self.traindata.label.items))))
+
+
 
 
     def _next_index(self):
@@ -298,22 +329,18 @@ class ImageDataProvider(object):
         else:
             return None
 
-    def get_all_data(self, is_shuffle=False, get_image_mode=GetImageMode.expect, topk=-1):
-        orig_get_image_mode = None
-        if hasattr(self.traindata.data, 'get_image_mode'):
-            orig_get_image_mode = self.traindata.data.get_image_mode
-            self.traindata.data.get_image_mode = get_image_mode
-
+    def get_all_data(self, is_shuffle=False,topk=-1):
         idxes = np.arange(len(self.traindata.data))
-        if is_shuffle == True:
+        if is_shuffle :
             np.random.shuffle(idxes)
         data = []
         if topk == -1:
             topk = len(self.traindata.data)
+
         for i in range(topk):
-            data.append(self.traindata.data[idxes[i]])
-        if hasattr(self.traindata.data, 'get_image_mode'):
-            self.traindata.data.get_image_mode = orig_get_image_mode
+            data.append(self.traindata.data.__getitem__(idxes[i]))
+
+
         return data
 
     def binding_class_names(self, class_names=None, language=None):
@@ -358,6 +385,19 @@ class ImageDataProvider(object):
     def get_language(self):
         return self.__default_language__
 
+    def __setattr__(self, name: str, value) -> None:
+        object.__setattr__(self, name, value)
+        if name=='mode':
+            if isinstance(self.traindata,Iterator):
+                self.traindata.mode=value
+                self.traindata.batch_sampler.mode=value
+            if isinstance(self.testdata,Iterator):
+                self.testdata.mode=value
+                self.testdata.batch_sampler.mode = value
+
+
+
+
 
 DataProvider = ImageDataProvider
 
@@ -498,7 +538,7 @@ class TextSequenceDataProvider(object):
         for i in range(len(self.text_transform_funcs)):
             fn = self.text_transform_funcs[-1 - i]
             if fn.__qualname__ == 'normalize.<locals>.text_op':
-                return_list.append(unnormalize(fn.mean, fn.std))
+                return_list.append(Unnormalize(fn.mean, fn.std))
         # return_list.append(array2text)
         return return_list
 
@@ -534,10 +574,10 @@ class TextSequenceDataProvider(object):
         self._paired_transform_funcs = value
 
         if self.traindata is not None and hasattr(self.traindata, 'paired_transform_funcs'):
-            self.traindata.paired_transform_funcs = self._paired_transform_funcs
+            self.traindata.paired_transform_funcs = value
 
         if self.testdata is not None and hasattr(self.testdata, 'paired_transform_funcs'):
-            self.testdata.paired_transform_funcs = self._paired_transform_funcs
+            self.testdata.paired_transform_funcs = value
 
     def _next_index(self):
         return self.__next__()
