@@ -19,7 +19,7 @@ from trident.backend.common import get_function, get_class, camel2snake,snake2ca
 from trident.backend.pytorch_backend import Layer,Parameter
 from trident.backend.pytorch_ops import *
 
-__all__ = ['Identity', 'Sigmoid', 'Tanh', 'Relu', 'Relu6', 'LeakyRelu', 'LeakyRelu6', 'SmoothRelu', 'PRelu', 'Swish',
+__all__ = ['Identity', 'Sigmoid', 'Tanh', 'Relu', 'Relu6', 'LeakyRelu', 'LeakyRelu6', 'SmoothRelu','CRelu','Silu', 'PRelu', 'Swish',
            'Elu', 'HardSigmoid', 'HardSwish', 'Selu', 'LecunTanh', 'SoftSign', 'SoftPlus', 'HardTanh', 'Logit',
            'LogLog', 'Mish','HardMish', 'Softmax', 'Gelu', 'GptGelu','SIREN', 'LogSoftmax', 'get_activation']
 
@@ -178,6 +178,65 @@ class SmoothRelu(Layer):
         return smooth_relu(x)
 
 
+class CRelu(Layer):
+    """Computes Concatenated ReLU.
+
+    Concatenates a ReLU which selects only the positive part of the activation
+    with a ReLU which selects only the *negative* part of the activation.
+    Note that as a result this non-linearity doubles the depth of the activations.
+    Source: [Understanding and Improving Convolutional Neural Networks via
+    Concatenated Rectified Linear Units. W. Shang, et
+    al.](https://arxiv.org/abs/1603.05201)
+
+    References:
+      Understanding and Improving Convolutional Neural Networks via Concatenated
+      Rectified Linear Units:
+        [Shang et al., 2016](http://proceedings.mlr.press/v48/shang16)
+        ([pdf](http://proceedings.mlr.press/v48/shang16.pdf))
+    """
+
+    def __init__(self, axis=1,name=None):
+        super(CRelu, self).__init__()
+        self._built = True
+        self.axis=axis
+
+    def forward(self, x, **kwargs):
+        return crelu(x,axis=self.axis)
+
+
+class Silu(Layer):
+    """Applies the silu function, element-wise.
+
+    .. math::
+        \text{silu}(x) = x * \sigma(x), \text{where } \sigma(x) \text{ is the logistic sigmoid.}
+
+    .. note::
+        See `Gaussian Error Linear Units (GELUs) <https://arxiv.org/abs/1606.08415>`_
+        where the SiLU (Sigmoid Linear Unit) was originally coined, and see
+        `Sigmoid-Weighted Linear Units for Neural Network Function Approximation
+        in Reinforcement Learning <https://arxiv.org/abs/1702.03118>`_ and `Swish:
+        a Self-Gated Activation Function <https://arxiv.org/abs/1710.05941v1>`_
+        where the SiLU was experimented with later.
+
+    Shape:
+        - Input: :math:`(N, *)` where `*` means, any number of additional
+          dimensions
+        - Output: :math:`(N, *)`, same shape as the input
+
+    Examples::
+
+        >>> m = Silu()
+        >>> input = torch.randn(2)
+        >>> output = m(input)
+    """
+
+    def __init__(self, name=None):
+        super(Silu, self).__init__()
+        self._built = True
+
+    def forward(self, x, **kwargs):
+        return torch.nn.functional.silu(x)
+
 class PRelu(Layer):
     """Parametric Rectified Linear Unit.
 
@@ -654,8 +713,8 @@ def get_activation(fn_name):
                     return activation_fn()
         elif getattr(fn_name, '__module__', None) == 'trident.layers.pytorch_activations':
             if inspect.isfunction(fn_name):
-                return fn_name
-            elif inspect.isclass(fn_name) and inspect._is_type(fn_name):
+                return partial(fn_name)
+            elif inspect.isclass(fn_name) and  fn_name.__class__.__name__=="type":
                 return fn_name()
             elif isinstance(fn_name, Layer):
                 return fn_name
