@@ -103,7 +103,10 @@ class BatchNorm(Layer):
 
         """
 
-        super().__init__(in_sequence=in_sequence,name=name)
+        super().__init__(name=name)
+        self.in_sequence=in_sequence
+        if self.in_sequence:
+            self.filter_index=-1
         self.eps = eps
         self.momentum = momentum
         self.affine = affine
@@ -139,7 +142,7 @@ class BatchNorm(Layer):
             missing_keys, unexpected_keys, error_msgs)
 
     def build(self, input_shape:TensorShape):
-        if self._built == False:
+        if not self._built:
             if self.affine:
                 self.register_parameter('weight', Parameter(ones(self.input_filters)))
                 self.register_parameter('bias', Parameter(zeros(self.input_filters)))
@@ -274,7 +277,10 @@ class GroupNorm(Layer):
             (2, 64, 128, 128)
 
         """
-        super().__init__(in_sequence=in_sequence,name=name)
+        super().__init__(name=name)
+        self.in_sequence = in_sequence
+        if self.in_sequence:
+            self.filter_index = -1
         self.affine=affine
         self.num_groups = num_groups
         self.eps = eps
@@ -374,7 +380,10 @@ class InstanceNorm(Layer):
             (2, 64, 128, 128)
 
         """
-        super().__init__(in_sequence=in_sequence,name=name)
+        super().__init__(name=name)
+        self.in_sequence = in_sequence
+        if self.in_sequence:
+            self.filter_index = -1
         self.eps = _epsilon
         self.momentum = momentum
         self.affine = affine
@@ -392,8 +401,8 @@ class InstanceNorm(Layer):
     def build(self, input_shape:TensorShape):
         if self._built == False:
             if self.affine:
-                self.weight = Parameter(torch.Tensor(self.input_filters))
-                self.bias = Parameter(torch.Tensor(self.input_filters))
+                self.weight = Parameter(torch.Tensor(self.input_filters)).to(get_device())
+                self.bias = Parameter(torch.Tensor(self.input_filters)).to(get_device())
                 init.ones_(self.weight)
                 init.zeros_(self.bias)
             else:
@@ -401,9 +410,9 @@ class InstanceNorm(Layer):
                 self.register_parameter('bias', None)
 
             if self.track_running_stats:
-                self.register_buffer('running_mean', torch.zeros(self.input_filters))
-                self.register_buffer('running_var', torch.ones(self.input_filters))
-                self.register_buffer('num_batches_tracked', torch.tensor(0, dtype=torch.long))
+                self.register_buffer('running_mean', torch.zeros(self.input_filters).to(get_device()))
+                self.register_buffer('running_var', torch.ones(self.input_filters).to(get_device()))
+                self.register_buffer('num_batches_tracked', torch.tensor(0, dtype=torch.long).to(get_device()))
             else:
                 self.register_parameter('running_mean', None)
                 self.register_parameter('running_var', None)
@@ -490,7 +499,9 @@ class LayerNorm(Layer):
         >>> output = m(input)
 
         """
-        super().__init__(in_sequence=in_sequence,name=name)
+        super().__init__(name=name)
+        self.in_sequence=in_sequence
+        self.filter_index = -1
         self.eps = eps
         self.elementwise_affine = elementwise_affine
 
@@ -498,16 +509,14 @@ class LayerNorm(Layer):
 
     def build(self, input_shape:TensorShape):
         if self._built == False :
-            self.register_parameter('weight',Parameter(ones((self.input_filters))))
-            self.register_parameter('bias', Parameter(zeros((self.input_filters))))
+            self.normalized_shape=(input_shape[-1],)
+            self.register_parameter('weight',Parameter(ones(*self.normalized_shape)))
+            self.register_parameter('bias', Parameter(zeros(*self.normalized_shape)))
             self._built=True
     def forward(self, x, **kwargs):
 
-        if hasattr(self,'in_sequence') and self.in_sequence:
-            x = x.permute(0, 2, 1)
-        x= F.layer_norm(x, int_shape( self.weight), self.weight, self.bias, self.eps)
-        if hasattr(self,'in_sequence') and self.in_sequence:
-            x=x.permute(0, 2, 1)
+
+        x= F.layer_norm(x, self.normalized_shape, self.weight, self.bias, self.eps)
         return x
         # mean = x.mean(dim=self.axis, keepdim=True).detach()
         # std = x.std(dim=self.axis, keepdim=True).detach()
@@ -519,7 +528,10 @@ LayerNorm3d=LayerNorm
 
 class L2Norm(Layer):
     def __init__(self,in_sequence=False, axis=1,name=None, **kwargs):
-        super().__init__(in_sequence=in_sequence,name=name)
+        super().__init__(name=name)
+        self.in_sequence = in_sequence
+        if self.in_sequence:
+            self.filter_index = -1
         self.eps=epsilon()
         self.axis=axis
 
@@ -540,6 +552,9 @@ class L2Norm(Layer):
 class PixelNorm(Layer):
     def __init__(self,in_sequence=False,name=None, **kwargs):
         super(PixelNorm, self).__init__(in_sequence=in_sequence,name=name)
+        self.in_sequence = in_sequence
+        if self.in_sequence:
+            self.filter_index = -1
 
     def forward(self, x, **kwargs):
         x = enforce_singleton(x)
@@ -630,11 +645,11 @@ class SpectralNorm(Layer):
         height = w.data.shape[0]
         width = w.view(height, -1).data.shape[1]
 
-        u = Parameter(w.data.new(height).normal_(0, 1), requires_grad=False)
-        v = Parameter(w.data.new(width).normal_(0, 1), requires_grad=False)
+        u = Parameter(w.data.new(height).normal_(0, 1), requires_grad=False).to(get_device())
+        v = Parameter(w.data.new(width).normal_(0, 1), requires_grad=False).to(get_device())
         u.data = l2_normalize(u.data)
         v.data = l2_normalize(v.data)
-        w_bar = Parameter(w.data)
+        w_bar = Parameter(w.data).to(get_device())
 
         del self.module._parameters[self.name]
 
@@ -658,8 +673,11 @@ class SpectralNorm(Layer):
 
 
 class EvoNormB0(Layer):
-    def __init__(self,rank=2,nonlinear=True,momentum=0.9,eps = 1e-5):
+    def __init__(self,rank=2,nonlinear=True,momentum=0.9,eps = 1e-5,in_sequence=False):
         super(EvoNormB0, self).__init__()
+        self.in_sequence = in_sequence
+        if self.in_sequence:
+            self.filter_index = -1
         self.rank=rank
         self.nonlinear = nonlinear
         self.momentum = momentum
@@ -725,13 +743,18 @@ def get_normalization(fn_name):
         return None
     elif isinstance(fn_name,Layer) and 'Norm' in fn_name.__class__.__name__:
         return fn_name
+    elif inspect.isclass(fn_name) and fn_name.__class__.__name__==type:
+        return fn_name()
     elif inspect.isclass(fn_name):
         return fn_name
+
     elif isinstance(fn_name, str):
         if fn_name.lower().strip() in ['instance_norm','instance','in','i']:
             return InstanceNorm()
         elif  fn_name.lower().strip() in ['batch_norm','batch','bn','b']:
             return BatchNorm()
+        elif fn_name.lower().strip() in ['layer_norm', 'layer', 'ln', 'l']:
+            return BatchNorm()
         elif  fn_name.lower().strip() in ['group_norm','group','gn','g']:
             return GroupNorm(num_groups=16)
         elif fn_name.lower().strip() in ['evo_normb0', 'evo-b0', 'evob0']:
@@ -740,9 +763,9 @@ def get_normalization(fn_name):
             return EvoNormS0()
         elif fn_name.lower().strip() in ['spectral_norm','spectral','spec','sp' ,'s']:
             return SpectralNorm
+        elif fn_name.lower().strip() in ['l2_norm','l2']:
+            return L2Norm()
 
-    elif inspect.isclass(fn_name):
-        return fn_name
     fn_modules = ['trident.layers.pytorch_normalizations']
     normalization_fn_ = get_class(fn_name, fn_modules)
     normalization_fn = normalization_fn_
