@@ -44,7 +44,7 @@ class PositionalEmbedding(Layer):
         pe = pe.unsqueeze(0)
         self.register_buffer('pe', pe)
 
-    def forward(self, x):
+    def forward(self, x, **kwargs):
         return self.pe[:, :x.size(1)]
 
 class PositionEmbeddingSine(Layer):
