@@ -12,7 +12,6 @@ if get_backend()=='pytorch':
     from . import pytorch_densenet as densenet
     from . import pytorch_efficientnet as efficientnet
     from . import pytorch_mobilenet as mobilenet
-    from . import pytorch_gan as gan
     from . import pytorch_deeplab as deeplab
     from . import pytorch_arcfacenet as arcfacenet
     from . import pytorch_mtcnn as mtcnn
@@ -20,6 +19,8 @@ if get_backend()=='pytorch':
     from . import pytorch_ssd as ssd
     from . import pytorch_yolo as yolo
     from . import pytorch_embedded as embedded
+    from . import pytorch_inception as inception
+    from . import pytorch_visual_transformer as visual_transformer
 elif get_backend()=='tensorflow':
     from . import tensorflow_vgg as vgg
     from . import tensorflow_resnet as resnet
