@@ -19,6 +19,7 @@ import torch.nn.functional as F
 from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
+from trident.data.vision_transforms import Resize,Normalize
 
 from trident.models.pretrained_utils import _make_recovery_model_include_top
 from trident.backend.common import *
@@ -36,7 +37,7 @@ __all__ = ['efficient_block', 'EfficientNet', 'EfficientNetB0', 'EfficientNetB1'
            'EfficientNetB4', 'EfficientNetB5', 'EfficientNetB6', 'EfficientNetB7']
 
 _session = get_session()
-_device =get_device()
+
 
 _epsilon = _session.epsilon
 _trident_dir = _session.trident_dir
@@ -219,8 +220,8 @@ def EfficientNet(width_coefficient, depth_coefficient, default_size, dropout_rat
               encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
         model.class_names = labels
-    model.preprocess_flow = [resize((default_size[2], default_size[1]), keep_aspect=True), normalize(0, 255),
-                             normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
+    model.preprocess_flow = [Resize((default_size[2], default_size[1]), keep_aspect=True), Normalize(0, 255),
+                             Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
 
     # model.summary()
     return model
@@ -236,13 +237,13 @@ def EfficientNetB0(include_top=True, pretrained=True,freeze_features=False, inpu
     if pretrained == True:
         download_model_from_google_drive('1bxnoDerzoNfiZZLft4ocD3DAgx4v6aTN', dirname, 'efficientnet-b0.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b0.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb0.model = recovery_model
     else:
         effb0.model = _make_recovery_model_include_top( effb0.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb0.model .input_shape = input_shape
-    effb0.model .to(_device)
+    effb0.model .to(get_device())
     return effb0
 
 
@@ -255,13 +256,13 @@ def EfficientNetB1(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1F3BtnAjmDz4G9RS9Q0hqU_K7WWXCni1G', dirname, 'efficientnet-b1.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b1.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb1.model = recovery_model
     else:
         effb1.model = _make_recovery_model_include_top( effb1.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb1.model .input_shape = input_shape
-    effb1.model .to(_device)
+    effb1.model .to(get_device())
     return effb1
 
 
@@ -274,13 +275,13 @@ def EfficientNetB2(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1PjqhB7WJasF_hqOwYtSBNSXSGBY-cRLU', dirname, 'efficientnet-b2.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b2.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb2.model = recovery_model
     else:
         effb2.model = _make_recovery_model_include_top( effb2.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb2.model .input_shape = input_shape
-    effb2.model .to(_device)
+    effb2.model .to(get_device())
     return effb2
 
 
@@ -293,13 +294,13 @@ def EfficientNetB3(include_top=True, pretrained=True,freeze_features=False, inpu
     if pretrained == True:
         download_model_from_google_drive('11tMxdYdFfaEREwnESO4cwjtcoEB42zB_', dirname, 'efficientnet-b3.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b3.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb3.model = recovery_model
     else:
         effb3.model = _make_recovery_model_include_top(effb3.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb3.model.input_shape = input_shape
-    effb3.model.to(_device)
+    effb3.model.to(get_device())
     return effb3
 
 
@@ -312,13 +313,13 @@ def EfficientNetB4(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1X4ZOBR_ETRHZJeffJHvCmWTTy9_aW8SP', dirname, 'efficientnet-b4.pth')
         recovery_model =fix_layer( load(sanitize_path(os.path.join(dirname, 'efficientnet-b4.pth'))))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb4.model = recovery_model
     else:
         effb4.model = _make_recovery_model_include_top(effb4.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb4.model.input_shape = input_shape
-    effb4.model.to(_device)
+    effb4.model.to(get_device())
     return effb4
 
 
@@ -331,13 +332,13 @@ def EfficientNetB5(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('17iTD12G9oW3jYAui84MKtdY4gjd9vpgG', dirname, 'efficientnet-b5.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b5.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb5.model = recovery_model
     else:
         effb5.model = _make_recovery_model_include_top(effb5.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb5.model.input_shape = input_shape
-    effb5.model.to(_device)
+    effb5.model.to(get_device())
     return effb5
 
 
@@ -350,13 +351,13 @@ def EfficientNetB6(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1XJrKmcmMObN_nnjP2Z-YH_BQ3img58qF', dirname, 'efficientnet-b6.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b6.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb6.model = recovery_model
     else:
         effb6.model = _make_recovery_model_include_top(effb6.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb6.model.input_shape = input_shape
-    effb6.model.to(_device)
+    effb6.model.to(get_device())
     return effb6
 
 
@@ -370,11 +371,11 @@ def EfficientNetB7(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1M2DfvsNPRCWSo_CeXnUCQOR46rvOrhLl', dirname, 'efficientnet-b7.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b7.pth')))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb7.model = recovery_model
     else:
         effb7.model = _make_recovery_model_include_top(effb7.model, include_top=include_top, classes=classes, freeze_features=False)
 
     effb7.model.input_shape = input_shape
-    effb7.model.to(_device)
+    effb7.model.to(get_device())
     return effb7
