@@ -32,7 +32,7 @@ from trident.layers.pytorch_layers import *
 from trident.layers.pytorch_normalizations import get_normalization
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import ImageClassificationModel
-
+from trident.data.vision_transforms import Resize,Normalize
 __all__ = ['VGG19','VGG11','VGG13','VGG16']
 
 _session = get_session()
@@ -97,8 +97,8 @@ def make_vgg_layers(cfg, num_classes=1000,input_shape=(3,224,224),include_top=Tr
               encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
         model.class_names = labels
-    model.preprocess_flow = [resize((input_shape[2], input_shape[1]), keep_aspect=True), normalize(0, 255),
-                             normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
+    model.preprocess_flow = [Resize((input_shape[2], input_shape[1]), keep_aspect=True), Normalize(0, 255),
+                             Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
     # model.summary()
 
     return model
@@ -122,7 +122,7 @@ def VGG11(include_top=True,
         recovery_model=load(os.path.join(dirname,'vgg11.pth'))
         recovery_model = fix_layer(recovery_model)
         recovery_model.name='vgg11'
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         vgg11.model = recovery_model
 
     else:
@@ -152,7 +152,7 @@ def VGG13(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1wx67gmQ8eHWXs2mhJmNl-t-cFNw7dJ7O',dirname,'vgg13.pth')
         recovery_model=load(os.path.join(dirname,'vgg13.pth'))
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         vgg13.model = recovery_model
 
     else:
@@ -181,7 +181,7 @@ def VGG16(include_top=True,
         recovery_model=load(os.path.join(dirname,'vgg16.pth'))
         recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'vgg16'
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         vgg16.model = recovery_model
 
     else:
@@ -209,7 +209,7 @@ def VGG19(include_top=True,
         recovery_model=load(os.path.join(dirname,'vgg19.pth'))
         recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'vgg19'
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model,input_shape=input_shape, include_top=include_top, classes=classes, freeze_features=freeze_features)
         vgg19.model = recovery_model
 
     else:
