@@ -19,7 +19,7 @@ from trident.layers.tensorflow_blocks import *
 from trident.layers.tensorflow_layers import *
 from trident.layers.tensorflow_pooling import GlobalAvgPool2d
 from trident.optims.tensorflow_trainer import *
-
+from trident.data.vision_transforms import Resize,Normalize
 
 __all__ = ['efficient_block','EfficientNet','EfficientNetB0','EfficientNetB1','EfficientNetB2','EfficientNetB3','EfficientNetB4','EfficientNetB5','EfficientNetB6','EfficientNetB7']
 
@@ -185,7 +185,7 @@ def EfficientNet(width_coefficient,
         if dropout_rate > 0:
             efficientnet.add_module('top_dropout',Dropout(dropout_rate,name='top_dropout'))
         efficientnet.add_module('fc',Dense(num_classes,activation=None,name='fc'))
-        efficientnet.add_module('softmax', SoftMax(name='softmax'))
+        efficientnet.add_module('softmax', SoftMax(axis=-1,name='softmax'))
     if isinstance(default_size,int):
         default_size=(default_size,default_size,3)
     elif len(default_size)==1:
@@ -195,7 +195,7 @@ def EfficientNet(width_coefficient,
     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)) ,'imagenet_labels1.txt'), 'r', encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
         model.class_names=labels
-    model.preprocess_flow=[resize((default_size[0],default_size[1]),keep_aspect=True),normalize(0,255),normalize([0.485, 0.456, 0.406],[0.229, 0.224, 0.225])]
+    model.preprocess_flow=[Resize((default_size[0],default_size[1]),keep_aspect=True),Normalize(0,255),Normalize([0.485, 0.456, 0.406],[0.229, 0.224, 0.225])]
     return model
 
 
@@ -211,7 +211,7 @@ def EfficientNetB0(include_top=True,
         input_shape=(224, 224,3)
     effb0 = EfficientNet(1.0, 1.0, default_size=input_shape, dropout_rate= 0.2, model_name='efficientnet-b0',include_top=include_top, num_classes=classes)
     with tf.device(get_device()):
-        if pretrained==True:
+        if pretrained:
             download_model_from_google_drive('1pO4wRWY6N4e7U_7E2H-NhBPEF4MlR4ru',dirname,'efficientnet-b0_tf.pth')
             recovery_model=load(os.path.join(dirname,'efficientnet-b0_tf.pth'))
             recovery_model = fix_layer(recovery_model)
@@ -222,6 +222,7 @@ def EfficientNetB0(include_top=True,
             effb0.model = _make_recovery_model_include_top(effb0.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb0.model.input_shape = input_shape
+        effb0.model.name='efficientnet-b0'
     return effb0
 
 
@@ -248,6 +249,7 @@ def EfficientNetB1(include_top=True,
             effb1.model = _make_recovery_model_include_top(effb1.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb1.model.input_shape = input_shape
+        effb1.model.name = 'efficientnet-b1'
     return effb1
 
 
@@ -274,6 +276,7 @@ def EfficientNetB2(include_top=True,
             effb2.model = _make_recovery_model_include_top(effb2.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb2.model.input_shape = input_shape
+        effb2.model.name = 'efficientnet-b2'
     return effb2
 
 
@@ -300,6 +303,7 @@ def EfficientNetB3(include_top=True,
             effb3.model = _make_recovery_model_include_top(effb3.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb3.model.input_shape = input_shape
+        effb3.model.name = 'efficientnet-b3'
     return effb3
 
 
@@ -326,6 +330,7 @@ def EfficientNetB4(include_top=True,
             effb4.model = _make_recovery_model_include_top(effb4.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb4.model.input_shape = input_shape
+        effb4.model.name = 'efficientnet-b4'
     return effb4
 
 
@@ -365,6 +370,7 @@ def EfficientNetB5(include_top=True,
             effb5.model = _make_recovery_model_include_top(effb5.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb5.model.input_shape = input_shape
+        effb5.model.name = 'efficientnet-b5'
     return effb5
 
 
@@ -392,6 +398,7 @@ def EfficientNetB6(include_top=True,
             effb6.model = _make_recovery_model_include_top(effb6.model, include_top=include_top, classes=classes, freeze_features=False)
 
         effb6.model.input_shape = input_shape
+        effb6.model.name = 'efficientnet-b6'
     return effb6
 
 
@@ -407,17 +414,17 @@ def EfficientNetB7(include_top=True,
     else:
         input_shape=(600, 600,3)
     effb7 =EfficientNet(2.0, 3.1, default_size=input_shape, dropout_rate=0.5, model_name='efficientnet-b7',include_top=include_top,num_classes=classes)
-    if pretrained==True:
-        with tf.device(get_device()):
-            if pretrained == True:
-                download_model_from_google_drive('1NcsqfYpnIXme8nk8qrrvNAVQOGK-EOZz', dirname, 'efficientnet-b7_tf.pth')
-                recovery_model = load(os.path.join(dirname, 'efficientnet-b7_tf.pth'))
-                recovery_model = fix_layer(recovery_model)
+    with tf.device(get_device()):
+        if pretrained:
+            download_model_from_google_drive('1NcsqfYpnIXme8nk8qrrvNAVQOGK-EOZz', dirname, 'efficientnet-b7_tf.pth')
+            recovery_model = load(os.path.join(dirname, 'efficientnet-b7_tf.pth'))
+            recovery_model = fix_layer(recovery_model)
 
-                recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
-                effb7.model = recovery_model
-            else:
-                effb7.model = _make_recovery_model_include_top(effb7.model, include_top=include_top, classes=classes, freeze_features=False)
+            recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+            effb7.model = recovery_model
+        else:
+            effb7.model = _make_recovery_model_include_top(effb7.model, include_top=include_top, classes=classes, freeze_features=False)
 
-            effb7.model.input_shape = input_shape
-        return effb7
\ No newline at end of file
+        effb7.model.input_shape = input_shape
+        effb7.model.name = 'efficientnet-b7'
+    return effb7
\ No newline at end of file
