@@ -16,7 +16,7 @@ from trident.backend.common import get_session, addindent, get_time_suffix, get_
 from trident.backend.pytorch_ops import *
 from trident.data.mask_common import mask2trimap
 
-__all__ = ['accuracy','pixel_accuracy','alpha_pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
+__all__ = ['accuracy','recall','pixel_accuracy','alpha_pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
 
 # def accuracy(input, target,axis=1):
 #     input_tensor=input.clone().detach()
@@ -35,12 +35,15 @@ __all__ = ['accuracy','pixel_accuracy','alpha_pixel_accuracy','iou','psnr','mean
 
 
 
-def accuracy(output, target, topk=1,axis=1,exclude_mask=False):
+@torch.no_grad()
+def accuracy(output, target, topk=1,axis=1,ignore_index=-100, exclude_mask=False):
     """Computes the precision@k for the specified values of k
     prec1, prec5 = accuracy(output.data, target, topk=(1, 5))
     """
     input_tensor=output.copy().detach()
     target_tensor=target.copy().detach()
+    num_classes = int_shape(output)[axis]
+
 
     is_logsoftmax = None
     from_logits = None
@@ -68,9 +71,17 @@ def accuracy(output, target, topk=1,axis=1,exclude_mask=False):
     if input_tensor.shape!=target_tensor.shape and topk==1:
         raise  ValueError('input shape {0} is not competable with target shape {1}'.format(input_tensor.shape,target_tensor.shape))
 
+    input_mask=ones_like(input_tensor)
+    if isinstance(ignore_index, int) and 0 <= ignore_index < num_classes:
+        input_mask[input_tensor==ignore_index] = 0
+    elif isinstance(ignore_index, (list, tuple)):
+        for idx in ignore_index:
+            if isinstance(idx, int) and 0 <= idx < int_shape(output)[axis]:
+                input_mask[input_tensor == idx] = 0
+
     batch_size = target_tensor.size(0)
     if topk==1:
-        return input_tensor.eq(target_tensor).float().mean()
+        return (input_tensor.eq(target_tensor).float()*input_mask).sum()/clip((input_mask).float().sum(),min=1)
     else:
         _, pred = input_tensor.topk(topk)
         pred = pred.t()
@@ -80,19 +91,76 @@ def accuracy(output, target, topk=1,axis=1,exclude_mask=False):
 
 
 
+@torch.no_grad()
+def recall(output, target, axis=1,ignore_index=-100):
+    """Computes the precision@k for the specified values of k
+    prec1, prec5 = accuracy(output.data, target, topk=(1, 5))
+    """
+    input_tensor=output.copy().detach()
+    target_tensor=target.copy().detach()
+    num_classes = int_shape(output)[axis]
 
 
+    is_logsoftmax = None
+    from_logits = None
+    output_exp = exp(input_tensor)
+    if (ndim(input_tensor) >= 1 and 'float' in str(input_tensor.dtype) and input_tensor.min() >= 0 and input_tensor.max() <= 1):
+        is_logsoftmax = False
+        from_logits = True
+        input_tensor = clip(input_tensor, min=1e-8, max=1 - 1e-8)
+
+    elif (ndim(output_exp) >= 1 and 'float' in str(output_exp.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
+        is_logsoftmax = True
+        from_logits = True
+        input_tensor =  clip(output_exp, min=1e-8, max=1 - 1e-8)
+    else:
+        is_logsoftmax = False
+        from_logits = False
+
+    if input_tensor.dtype!=torch.int64 :
+        if len(input_tensor.size())==1: #binary
+            input_tensor=input_tensor.gt(0.5).float()
+        else:
+            input_tensor=argmax(input_tensor,axis).squeeze()
+    if target_tensor.dtype!=torch.int64:
+        target_tensor=argmax(target_tensor,axis).squeeze()
+    if input_tensor.shape!=target_tensor.shape :
+        raise  ValueError('input shape {0} is not competable with target shape {1}'.format(input_tensor.shape,target_tensor.shape))
+    target_mask=ones_like(target_tensor)
+    if isinstance(ignore_index, int) and 0 <= ignore_index < num_classes:
+        target_mask[target_tensor==ignore_index] = 0
+    elif isinstance(ignore_index, (list, tuple)):
+        for idx in ignore_index:
+            if isinstance(idx, int) and 0 <= idx < int_shape(output)[axis]:
+                target_mask[target_tensor == idx] = 0
+
+    batch_size = target_tensor.size(0)
+    return (input_tensor.eq(target_tensor).float()*target_mask).sum()/clip((target_mask).float().sum(),min=1)
+
+
+
+
+@torch.no_grad()
 def psnr(output, target):
     input_tensor = output.clone().detach()
     target_tensor = target.clone().detach()
     if input_tensor.shape != target_tensor.shape :
         raise ValueError(
             'input shape {0} is not competable with target shape {1}'.format(input_tensor.shape, target_tensor.shape))
+
+    max_value = 255
+    target_np = to_numpy(target_tensor)
+    if  target_np.min() <0 :
+        target_tensor = (target_tensor + 1) * 0.5
+        input_tensor = (input_tensor + 1) * 0.5
+        max_value = 1
+    elif 0 <= target_np.min() <= 1 and 0 <= target_np.max() <= 1:
+        max_value = 1
     rmse = ((input_tensor - target_tensor) ** 2).mean().sqrt()
-    psnr = 20 * (1 / rmse).log10_()
+    psnr = 20 * ((max_value/ rmse).log10_())
     return psnr
 
-
+@torch.no_grad()
 def mean_absolute_error(output, target):
     input_tensor = output.view(-1).clone().detach()
     target_tensor = target.view(-1).clone().detach()
@@ -103,7 +171,7 @@ def mean_absolute_error(output, target):
     return torch.abs(input_tensor- target_tensor).mean()
 mae=mean_absolute_error
 
-
+@torch.no_grad()
 def mean_squared_error(output, target):
     input_tensor = output.view(-1).clone().detach()
     target_tensor = target.view(-1).clone().detach()
@@ -116,7 +184,7 @@ mse=mean_squared_error
 
 
 
-
+@torch.no_grad()
 def root_mean_squared_error(output, target):
     input_tensor=output.view(-1).clone().detach()
     target_tensor=target.view(-1).clone().detach()
@@ -127,7 +195,7 @@ def root_mean_squared_error(output, target):
 rmse=root_mean_squared_error
 
 
-
+@torch.no_grad()
 def mean_squared_logarithmic_error(output, target):
     input_tensor=output.view(-1).clone().detach()
     target_tensor=target.view(-1).clone().detach()
@@ -138,7 +206,7 @@ def mean_squared_logarithmic_error(output, target):
 msle=mean_squared_logarithmic_error
 
 
-
+@torch.no_grad()
 def pixel_accuracy(output, target):
     input_tensor = output.clone().detach()
     target_tensor = target.clone().detach()
@@ -148,6 +216,7 @@ def pixel_accuracy(output, target):
     pixel_correct = ((input_tensor == target_tensor)*(target_tensor > 0)).sum().float()
     return pixel_correct/max(pixel_labeled,1)
 
+@torch.no_grad()
 def alpha_pixel_accuracy(output, alpha):
     output_tensor = to_numpy(output)
     alpha_tensor =  to_numpy(alpha)
@@ -166,6 +235,7 @@ def alpha_pixel_accuracy(output, alpha):
     pixel_correct = ((output_tensor == alpha_tensor)*(trimap == 1)).sum()+ (np.less(np.abs(output_tensor - alpha_tensor),0.1).astype(np.float32)*(trimap == 0.5)).sum()
     return pixel_correct/max(pixel_labeled,1)
 
+@torch.no_grad()
 def iou(output, target):
     input_tensor = output.clone().detach()
     target_tensor = target.clone().detach()
@@ -178,7 +248,6 @@ def iou(output, target):
     return intersection/max(union,1)
 
 
-
 def get_metric(metric_name):
     if metric_name is None:
         return None
