@@ -1,6 +1,9 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+
+import math
+
 import numpy as np
 import tensorflow as tf
 from tensorflow.python.framework import ops
@@ -8,11 +11,17 @@ from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops.losses import loss_reduction
 from tensorflow.python.ops.losses import util as tf_losses_utils
+
+
+from trident import context
+
 from trident.backend.common import camel2snake, get_class, epsilon, PrintException
 from trident.backend.tensorspec import *
 from trident.backend.tensorflow_backend import *
 from trident.backend.tensorflow_ops import *
 from trident.optims.losses import Loss
+from trident.backend.common import dtype
+
 # def cosine_similarity(target, output):
 #     assert target.ndim == 2
 #     assert output.ndim == 2
@@ -29,8 +38,8 @@ from trident.optims.losses import Loss
 #             [0 ] *0, axis=-1)
 
 
-__all__ = ['get_loss','_ClassificationLoss', 'CrossEntropyLoss', 'MSELoss', 'EdgeLoss', 'NLLLoss', 'F1ScoreLoss', '_ClassificationLoss',
-           'FocalLoss','DiceLoss','L1Loss','L2Loss','SmoothL1Loss','WingLoss','AdaptiveWingLoss']
+__all__ = ['get_loss','_ClassificationLoss', 'CrossEntropyLoss','BCELoss', 'MSELoss', 'EdgeLoss', 'NLLLoss', 'F1ScoreLoss', '_ClassificationLoss',
+           'FocalLoss','DiceLoss','L1Loss','L2Loss','SmoothL1Loss','WingLoss','AdaptiveWingLoss','IoULoss','SoftIoULoss']
 
 
 
@@ -39,7 +48,7 @@ __all__ = ['get_loss','_ClassificationLoss', 'CrossEntropyLoss', 'MSELoss', 'Edg
 class _ClassificationLoss(Loss):
     """Calculate loss for  complex classification task."""
 
-    def __init__(self, axis=-1, sample_weight=None, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', name=None, **kwargs):
+    def __init__(self, axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name=None, **kwargs):
         """
 
         Args:
@@ -72,7 +81,7 @@ class _ClassificationLoss(Loss):
             label_smooth (bool):If True, mean we will apply label-smoothing in loss calculation.
 
         """
-        super(_ClassificationLoss, self).__init__(reduction=reduction, sample_weight=sample_weight,axis=axis,name=name)
+        super(_ClassificationLoss, self).__init__(reduction=reduction, sample_weight=sample_weight,axis=axis,enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, namename=name)
         self._set_name_scope()
         self.need_target_onehot = True
         self.is_multiselection = False
@@ -81,6 +90,19 @@ class _ClassificationLoss(Loss):
         self.is_logsoftmax = False
         self.ignore_index = ignore_index
         self.ignore_index_weight=None
+        self.auto_balance = auto_balance
+        self.auto_balance = auto_balance
+        if self.auto_balance:
+            self.label_statistics = None
+            ctx = context._context()
+            if hasattr(ctx._thread_local_info, 'data_providers') and len(ctx._thread_local_info.data_providers) > 0:
+                dp = list(ctx._thread_local_info.data_providers.values())[0]
+                if dp.traindata.label.__class__.__name__ == 'LabelDataset':
+                    unique, counts = np.unique(np.array(dp.traindata.label.items), return_counts=True)
+                    reweights = np.clip(counts, 1, np.inf) / np.sum(counts).astype(np.float32)
+
+                    reweights1 = np.max(reweights) / reweights
+                    self.label_statistics = reweights1
         if cutoff is not None and not 0 < cutoff < 1:
             raise ValueError('cutoff should between 0 and 1')
         self.cutoff = cutoff
@@ -123,18 +145,20 @@ class _ClassificationLoss(Loss):
 
     def flatten_check(self, output, target):
         "Check that `out` and `targ` have the same number of elements and flatten them."
-        if ndim(output) > 2 and len(output) == len(target)+1:
+        if ndim(output) > 2 and ndim(output) == ndim(target)+1 :
             shp = int_shape(output)
             output = output.reshape((shp[0], -1, shp[-1]))
-            target = target.reshape((shp[0], -1))
+            target = cast(target.reshape((shp[0], -1)),'int64')
             return output, target
-        elif ndim(output) > 2 and len(output) == len(target):
+        elif ndim(output) > 2 and ndim(output) == ndim(target):
             shp = int_shape(output)
             output = output.reshape((shp[0], -1, shp[-1]))
             if ndim(target) > 2 :
                 target = target.reshape((shp[0], -1, shp[-1]))
             return output, target
-        elif ndim(output) == 2 and len(output) == len(target):
+        elif ndim(output) <= 2 and ndim(output) == ndim(target):
+            return output, target
+        elif ndim(output) <= 2 and ndim(output) == ndim(target)+1:
             return output, target
         else:
             raise ValueError('output and target have diffent elements.')
@@ -154,8 +178,9 @@ class _ClassificationLoss(Loss):
         if self.num_classes is None:
             self.num_classes = int_shape(output)[self.axis]
         output, target = self.flatten_check(output, target)
+
         if self.sample_weight is None:
-            self.sample_weight = ones(self.num_classes, requires_grad=False)
+            self.sample_weight = ones(self.num_classes)
         elif len(self.sample_weight) != self.num_classes:
             raise ValueError('weight should be 1-D tensor and length equal to numbers of filters')
         else:
@@ -166,17 +191,25 @@ class _ClassificationLoss(Loss):
         if (ndim(output) >= 1 and 'float' in str(output.dtype) and output.min() >= 0 and output.max() <= 1 ):
             self.is_logsoftmax = False
             self.from_logits = True
+            if self.auto_balance and self.label_statistics is not None:
+                output=output*to_tensor(self.label_statistics.copy())
             output = clip(output, min=1e-8, max=1 - 1e-8)
 
         elif (ndim(output) >=1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1 ):
             self.is_logsoftmax = True
             self.from_logits = True
+            if self.auto_balance and self.label_statistics is not None:
+                output = output +to_tensor(np.log(self.label_statistics.copy()))
             output = clip(output, max=- 1e-8)
         else:
             self.is_logsoftmax = False
             self.from_logits = False
 
-        self.ignore_index_weight=ones_like(self.sample_weight,requires_grad=False,dtype=output.dtype)
+
+        if (ndim(target) == ndim(output)  and 'float' in str(target.dtype) and target.min() >= 0 and target.max() <= 1 ):
+            self.is_target_onehot = True
+
+        self.ignore_index_weight=np.ones_like(self.sample_weight)
         # ignore_index
 
         if isinstance(self.ignore_index, int) and 0 <= self.ignore_index < self.num_classes:
@@ -185,6 +218,7 @@ class _ClassificationLoss(Loss):
             for idx in self.ignore_index:
                 if isinstance(idx, int) and 0 <= idx < int_shape(output)[self.axis]:
                     self.ignore_index_weight[idx] = 0
+        self.ignore_index_weight=to_tensor(self.ignore_index_weight,dtype=output.dtype,device=output.device)
         if self.label_smooth:
             self.need_target_onehot = True
         if target.dtype == str2dtype('long'):
@@ -197,15 +231,10 @@ class _ClassificationLoss(Loss):
         if  target.dtype==tf.int64 and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis)
             if self.label_smooth:
-                target = target #* (torch.Tensor(target.size()).uniform_(0.9, 1))
+                target = target +random_normal_like(target)
                 self.is_target_onehot = True
 
-
-        # setting cutoff
-        # if self.cutoff is not None:
-        #     mask = (output > self.cutoff).to(output.dtype)
-        #     output = output * mask
-        return output, tf.stop_gradient(target)
+        return output, target
 
     def calculate_loss(self, output, target, **kwargs):
         """ Calculate the unaggregate loss.
@@ -244,7 +273,7 @@ class _ClassificationLoss(Loss):
 class _PairwiseLoss(Loss):
     """Calculate loss for  complex classification task."""
 
-    def __init__(self, axis=-1,  reduction='batch_mean', name=None, **kwargs):
+    def __init__(self, axis=-1,  reduction='batch_mean',enable_ohem=False, ohem_ratio=3.5,name=None, **kwargs):
         """
 
         Args:
@@ -277,7 +306,7 @@ class _PairwiseLoss(Loss):
             label_smooth (bool):If True, mean we will apply label-smoothing in loss calculation.
 
         """
-        super(_PairwiseLoss, self).__init__(reduction=reduction,axis=axis,name=name)
+        super(_PairwiseLoss, self).__init__(reduction=reduction,axis=axis,enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,name=name)
         self._set_name_scope()
 
 
@@ -320,16 +349,17 @@ class _PairwiseLoss(Loss):
 
     def flatten_check(self, output, target):
         "Check that `out` and `targ` have the same number of elements and flatten them."
+        out_shp = int_shape(output)
+        tar_shp = int_shape(target)
         if ndim(output) > 2:
-            output = output.view(-1, output.size(-1))
-        if ndim(target) > 2:
+            output = reshape(output,(out_shp[0],-1,out_shp[-1]))
+        if ndim(target) >2 and len(tar_shp)-1==len(tar_shp)  :
+            target =reshape(target,(tar_shp[0],-1))
+        elif ndim(target) >2 and len(tar_shp)==len(tar_shp)  :
+            target = reshape(target, (tar_shp[0], -1, tar_shp[-1]))
 
-            target = target.view(-1, target.size(-1))
+        return output, target
 
-        if len(output) == len(target):
-            return output, target
-        else:
-            raise ValueError('output and target have diffent elements.')
 
     def preprocess(self, output: Tensor, target: Tensor, **kwargs):
         """
@@ -362,6 +392,37 @@ class _PairwiseLoss(Loss):
         ##dont do aggregation
         raise NotImplementedError
 
+    def _do_ohem(self, output: Tensor, target: Tensor):
+        if self.enable_ohem:
+            output_ = output.clone()
+            target_ = target.clone()
+            num_hard = 0
+            num_easy = 0
+            hard_mask = None
+            reduce_axis = list(range(output_.ndim))[1:]
+            base_losses = pow(output_ - target, 2).mean(axis=reduce_axis) if len(reduce_axis) > 0 else pow(output_ - target, 2)
+            if target.dtype == dtype.int64:
+                hard_mask = target < 0
+                num_hard = reduce_sum(hard_mask).numpy()
+                num_easy = int(self.ohem_ratio * num_hard)
+            elif target.shape == output.shape:
+                hard_mask = target < 0
+                num_hard =reduce_sum(hard_mask).numpy()
+                num_easy = int(self.ohem_ratio * num_hard)
+
+            if num_hard == 0:
+                return output, target
+            base_losses[hard_mask] = math.inf
+
+            easy_cases = topk(base_losses, k=clip(int(num_easy + num_hard),1,len(base_losses)))
+            idxs = easy_cases
+
+            output_hn = output.index_select(0, idxs)
+            target_hn = target.index_select(0, idxs)
+            return output_hn, target_hn
+        else:
+            return output, target
+
     def __call__(self, output: Tensor, target: Tensor, **kwargs):
         result = self.forward(output, target, **kwargs)
         return result
@@ -396,33 +457,38 @@ class CrossEntropyLoss(_ClassificationLoss):
 
     Examples:
     >>> output=to_tensor([[0.1, 0.7 , 0.2],[0.3 , 0.6 , 0.1],[0.9 , 0.05 , 0.05],[0.3 , 0.4 , 0.3]])
-    >>> print(output.shape)
+    >>> print(int_shape(output))
     (4, 3)
     >>> target=make_onehot([1,0,1,2],3,axis=-1)
     >>> print(target.shape)
     (4, 3)
     >>> CrossEntropyLoss(reduction='mean')(output,target)
-    <tf.Tensor: shape=(), dtype=float32, numpy=1.1305244>
+    <tf.Tensor: shape=(), dtype=float32, numpy=1.1305245>
     >>> CrossEntropyLoss(reduction='sum')(output,target)
-    <tf.Tensor: shape=(), dtype=float32, numpy=4.5220976>
+    <tf.Tensor: shape=(), dtype=float32, numpy=4.522098>
     >>> CrossEntropyLoss(label_smooth=True,reduction='mean')(output,target)
-    <tf.Tensor: shape=(), dtype=float32, numpy=1.1305244>
+    <tf.Tensor: shape=(), dtype=float32, numpy=1.1305245>
     >>> CrossEntropyLoss(sample_weight=to_tensor([1.0,1.0,0.5]),reduction='mean')(output,target)
-    <tf.Tensor: shape=(), dtype=float32, numpy=0.9888908>
+    <tf.Tensor: shape=(), dtype=float32, numpy=0.98889095>
     >>> CrossEntropyLoss(ignore_index=2,reduction='mean')(output,target)
-    <tf.Tensor: shape=(), dtype=float32, numpy=0.84725726>
+    <tf.Tensor: shape=(), dtype=float32, numpy=0.8472573>
+    >>> target = to_tensor([[1., 0., 0.],[ 0., 1., 0.],[ 0., 0., 1.]])
+    >>> out = to_tensor([[.9, .05, .05],[ .05, .89, .06],[ .05, .01, .94]])
+    >>> CrossEntropyLoss(reduction='mean',axis=-1)(out,target)
+    tf.Tensor(0.33333334, shape=(), dtype=float32)
+    >>> CrossEntropyLoss(reduction='mean',axis=-1)(out,argmax(target,-1))
+    tf.Tensor(0.33333334, shape=(), dtype=float32)
 
 
 
 
     """
 
-    def __init__(self, axis=-1, sample_weight=None, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False,
-                 reduction='mean', name='CrossEntropyLoss'):
-        super().__init__(axis, sample_weight, from_logits, ignore_index, cutoff, label_smooth, reduction, name)
+    def __init__(self, axis=-1, sample_weight=None, auto_balance=False,from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False,
+                 reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='CrossEntropyLoss'):
+        super().__init__(axis, sample_weight,auto_balance, from_logits, ignore_index, cutoff, label_smooth, reduction, enable_ohem,ohem_ratio,name)
         self._built = True
-        self.need_target_onehot=True
-
+        self.need_target_onehot =None
 
     def calculate_loss(self, output, target, **kwargs):
         """
@@ -447,21 +513,48 @@ class CrossEntropyLoss(_ClassificationLoss):
         # if self.ignore_index_weight is not None and ndim(self.ignore_index_weight)==1:
         #     self.ignore_index_weight=expand_dims(self.ignore_index_weight,0)
         with self._name_scope:
-            sample_weight = cast(self.sample_weight,output.dtype)*cast( self.ignore_index_weight,output.dtype)
-            if ndim(output) == 2:
-                pass
-            else:
-                reshape_shape = [1] * ndim(output)
-                reshape_shape[self.axis] = self.num_classes
-                sample_weight = self.sample_weight.reshape(reshape_shape)
-                if self.is_logsoftmax == True:
-                    sample_weight = self.sample_weight.reshape(reshape_shape) * self.ignore_index_weight.reshape(reshape_shape)
-            # -sum([p[i] * log2(q[i]) for i in range(len(p))])
-            if self.is_logsoftmax == False:
-                loss = -reduce_sum(target * log_softmax(output, axis=self.axis) * tf.stop_gradient(sample_weight),axis=-1)
-            else:
-                loss = -reduce_sum(target * output * sample_weight,axis=-1)
-            return loss
+            sample_weight =expand_dims(cast(self.sample_weight,output.dtype)*cast( self.ignore_index_weight,output.dtype),0)
+            n_=ndim(output)-ndim(sample_weight)
+            for n in range(n_):
+                sample_weight=expand_dims(sample_weight,1)
+            loss=None
+            if  self.is_target_onehot :
+                # -sum([p[i] * log2(q[i]) for i in range(len(p))])
+                if not self.is_logsoftmax:
+                    loss = -reduce_sum(target * log_softmax(output, axis=self.axis,keepdims=True) * sample_weight,axis=-self.axis)
+
+                else:
+
+                    loss = -reduce_sum(target * output * sample_weight,axis=self.axis)
+
+                if ndim(loss)>1:
+                    reduce_axes = list(range(loss.ndim))
+                    reduce_axes.remove(0)
+                    if len(reduce_axes) == 0:
+                        reduce_axes = None
+                    return reduce_mean(loss,axis=reduce_axes)
+                else:
+                    return loss
+            elif not self.is_target_onehot and ndim(target)==ndim(output)-1:
+                if not self.is_logsoftmax:
+                    output=log_softmax(output,axis=self.axis,keepdims=True)
+                target=expand_dims(cast(target,cast_dtype=dtype.int64),axis=self.axis)
+
+                loss=gather(-1*output,gather_axis=self.axis,indices=target)
+
+                loss = reduce_sum(loss, self.axis)
+                if ndim(loss)>1:
+                    reduce_axes = list(range(ndim(loss)))
+                    reduce_axes.remove(0)
+                    if len(reduce_axes) == 0:
+                        reduce_axes = None
+                    return reduce_mean(loss,axis=reduce_axes)
+                else:
+                    return loss
+
+
+
+
 
 
 class NLLLoss(_ClassificationLoss):
@@ -491,10 +584,14 @@ class NLLLoss(_ClassificationLoss):
 
     """
 
-    def __init__(self, axis=-1, loss_weights=None, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False,
-                 reduction='mean', name='NLLLoss'):
-        super().__init__(axis, loss_weights, from_logits, ignore_index, cutoff, label_smooth, reduction, name)
+    def __init__(self, axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False,
+                 reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='NLLLoss'):
+        super().__init__(axis, sample_weight, auto_balance, from_logits, ignore_index, cutoff, label_smooth, reduction, enable_ohem,ohem_ratio,name)
         self._built = True
+        self.need_target_onehot=True
+
+
+
 
     def calculate_loss(self, output, target, **kwargs):
         """
@@ -510,8 +607,19 @@ class NLLLoss(_ClassificationLoss):
         with self._name_scope:
             reshape_shape = [1] * ndim(output)
             reshape_shape[self.axis] = self.num_classes
-            loss = reduce_sum(-target * output * reshape(tf.stop_gradient(self.sample_weight), reshape_shape),axis=-1)
-            return loss
+            sample_weight = reshape(cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype), reshape_shape)
+
+
+            loss = reduce_sum(-target * output * self.sample_weight,axis=self.axis)
+            if ndim(loss) > 1:
+                reduce_axes = list(range(loss.ndim))
+                reduce_axes.remove(0)
+                if len(reduce_axes) == 0:
+                    reduce_axes = None
+                return reduce_mean(loss, axis=reduce_axes)
+            else:
+                return loss
+
 
 
 class F1ScoreLoss(_ClassificationLoss):
@@ -541,31 +649,30 @@ class F1ScoreLoss(_ClassificationLoss):
 
     """
 
-    def __init__(self,num_classes=None, beta=1, axis=-1, loss_weights=None, from_logits=False, ignore_index=-100, cutoff=None,
-                 label_smooth=False, reduction='mean', name='CrossEntropyLoss'):
-        super().__init__(axis, loss_weights, from_logits, ignore_index, cutoff, label_smooth, reduction, name)
+    def __init__(self, beta=1, axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='CrossEntropyLoss'):
+        super().__init__(axis, sample_weight, auto_balance, from_logits, ignore_index, cutoff, label_smooth, reduction, enable_ohem,ohem_ratio,name)
         self.beta = beta
+
+        self.need_target_onehot = True
         self._built = True
-        self.num_classes=num_classes
 
     def calculate_loss(self, output, target, **kwargs):
-        """
 
-        Args:
-            output ():
-            target ():
-            **kwargs ():
+        if self.is_logsoftmax:
+            output = clip(exp(output), 1e-8, 1 - 1e-8)
+            self.from_logits = True
+        if not self.from_logits:
+            output = softmax(output, self.axis)
+        if target.dtype == tf.int64 or self.is_target_onehot == False:
+            target = cast(make_onehot(target, self.num_classes, axis=1),output.dtype)
 
-        Returns:
-
-        """
-        with self._name_scope:
-            reshape_shape = [1] * ndim(output)
-            reshape_shape[self.axis] = self.num_classes
-            correct_predictions = reduce_sum(output * target, axis=self.axis, keepdims=True)
-            precision = correct_predictions / reduce_sum(output, axis=self.axis, keepdims=True)
-            recall = correct_predictions / reduce_sum(target, axis=self.axis, keepdims=True)
-            return 1 - (1 + self.beta ** 2) * precision * recall / (self.beta ** 2 * precision + recall)
+        tp = (target * output*self.sample_weight * self.ignore_index_weight).sum(axis=self.axis)
+        # tn = ((1 - target) * (1 - output))
+        # fp = ((1 - target) * output)
+        # fn = (target * (1 - output))
+        precision =true_divide(tp ,reduce_sum(output,axis=self.axis))
+        recall =true_divide(tp ,reduce_sum(target,axis=self.axis))
+        return 1-(1 + self.beta ** 2) * precision * recall /( self.beta ** 2*precision + recall)
 
 
 class FocalLoss(_ClassificationLoss):
@@ -580,14 +687,15 @@ class FocalLoss(_ClassificationLoss):
         https://github.com/open-mmlab/mmdetection/blob/master/mmdet/core/loss/losses.py
     """
 
-    def __init__(self, alpha=0.5, gamma=2, normalized=False, threshold=None, axis=-1, loss_weights=None,
+    def __init__(self, alpha=0.5, gamma=2, normalized=False, threshold=None, axis=-1, sample_weight=None,auto_balance=False,
                  from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean',
-                 name='FocalLoss'):
-        super().__init__(axis, loss_weights, from_logits, ignore_index, cutoff, label_smooth, reduction, name)
+                 enable_ohem=False, ohem_ratio=3.5 , name='FocalLoss'):
+        super().__init__(axis, sample_weight,auto_balance, from_logits, ignore_index, cutoff, label_smooth, reduction, enable_ohem,ohem_ratio,name)
         self.alpha = alpha
         self.gamma = gamma
         self.threshold = threshold
         self.normalized = normalized
+        self.need_target_onehot=True
 
     def calculate_loss(self, output, target, **kwargs):
         """
@@ -622,6 +730,38 @@ class FocalLoss(_ClassificationLoss):
 
             return loss
 
+
+
+class BCELoss(_ClassificationLoss):
+    def __init__(self, axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='BCELoss'):
+        super().__init__(axis=axis, sample_weight=sample_weight,auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff, label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem,ohem_ratio=ohem_ratio,name=name)
+        self._built = True
+        self.num_classes = 1
+        self.is_logsoftmax = False
+        self.need_target_onehot = True
+        self.is_target_onehot = False
+
+    def calculate_loss(self, output, target, **kwargs):
+        """
+
+        Args:
+            output ():
+            target ():
+            **kwargs ():
+
+        Returns:
+
+        """
+        with self._name_scope:
+            if self.is_logsoftmax:
+                output = exp(output)
+            # Compute cross entropy from probabilities.
+            bce = target * log(output + epsilon())
+            bce += (1 - target) * log(1 - output + epsilon())
+            return -bce
+
+
+
 class DiceLoss(_ClassificationLoss):
     r"""This criterion combines :func:`nn.LogSoftmax` and :func:`nn.NLLLoss` in one single class.
 
@@ -663,7 +803,7 @@ class DiceLoss(_ClassificationLoss):
 
     """
 
-    def __init__(self, smooth=1., axis=-1, sample_weight=None, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', name='DiceLoss'):
+    def __init__(self, smooth=1., axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='DiceLoss'):
         """
         Args:
             axis (int): the axis where the class label is.
@@ -676,7 +816,7 @@ class DiceLoss(_ClassificationLoss):
             name (stringf):
         """
 
-        super().__init__(axis, sample_weight, from_logits, ignore_index, cutoff, label_smooth, reduction, name)
+        super().__init__(axis, sample_weight,auto_balance, from_logits, ignore_index, cutoff, label_smooth, reduction, enable_ohem,ohem_ratio,name)
         self.smooth = smooth
         self.is_logsoftmax = False
         self.need_target_onehot = True
@@ -694,21 +834,24 @@ class DiceLoss(_ClassificationLoss):
         Returns:
 
         """
-        with self._name_scope:
-            if self.is_logsoftmax:
-                output=exp(output)
-            reduce_axes=list(range(target.ndim))
-            axis=self.axis if self.axis>=0 else target.ndim+self.axis
-            reduce_axes.remove(0)
-            reduce_axes.remove(axis)
-            loss_weights=self.sample_weight.copy()
-            # for k in range(target.ndim-self.loss_weights.ndim):
-            #     loss_weights=loss_weights.expand_dims(0)
-            intersection = reduce_sum(target * output, axis=reduce_axes)*loss_weights
-            den1 = reduce_sum(output, axis=reduce_axes)*loss_weights
-            den2 = reduce_sum(target, axis=reduce_axes)*loss_weights
-            dice = 1.0 - ((2.0 * intersection + self.smooth) / (den1 + den2 + self.smooth))
-            return dice
+        #with self._name_scope:
+        if self.is_logsoftmax:
+            output = exp(output)
+        reduce_axes = list(range(ndim(output)))
+        axis = self.axis if self.axis >= 0 else target.ndim + self.axis
+        reduce_axes.remove(0)
+        sample_weight =cast(expand_dims( self.sample_weight* self.ignore_index_weight,0),output.dtype)
+        n_ = ndim(output) - ndim(sample_weight)
+        for n in range(n_):
+            sample_weight = expand_dims(sample_weight, 0)
+
+        target=cast(target,output.dtype)
+        pred=greater_equal(output[:,:,1:],0.5,dtype=output.dtype)
+
+        intersection = reduce_sum(target *pred* sample_weight, axis=reduce_axes)
+        den = reduce_sum((pred+target)* sample_weight, axis=reduce_axes)
+        dice = 1.0 - math_ops.div_no_nan((2.0 * intersection + self.smooth), (den+ self.smooth))
+        return dice
 
 
 
@@ -719,7 +862,7 @@ class L1Loss(_PairwiseLoss):
 
      See :class:`~torch.nn.L1Loss` for details.
      """
-    def __init__(self, reduction='mean', name='L1Loss'):
+    def __init__(self, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='L1Loss'):
         super(L1Loss, self).__init__(reduction)
         self.name = name
         self.reduction = reduction
@@ -745,7 +888,7 @@ class L2Loss(_PairwiseLoss):
 
         See :class:`~torch.nn.MSELoss` for details.
         """
-    def __init__(self, reduction='mean', name='MSELoss'):
+    def __init__(self, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='MSELoss'):
         super(L2Loss, self).__init__(reduction)
         self.name = name
         self.reduction = reduction
@@ -762,7 +905,7 @@ class L2Loss(_PairwiseLoss):
 
         """
         with self._name_scope:
-            return 0.5*tf.math.square(output-target,name='l2_loss')
+            return tf.nn.l2_loss(output-target,name='l2_loss')
 
 
 #
@@ -772,7 +915,7 @@ class L2Loss(_PairwiseLoss):
 #
 #     See :class:`~torch.nn.SmoothL1Loss` for details.
 #     """
-#     def __init__(self, reduction='mean', name='SmoothL1Loss'):
+#     def __init__(self, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='SmoothL1Loss'):
 #         super(SmoothL1Loss, self).__init__(reduction=reduction)
 #         self.name = name
 #         self.reduction = reduction
@@ -830,8 +973,8 @@ class SmoothL1Loss(_PairwiseLoss):
 
     See :class:`~torch.nn.SmoothL1Loss` for details.
     """
-    def __init__(self, reduction='mean', name='SmoothL1Loss'):
-        super(SmoothL1Loss, self).__init__(reduction=reduction)
+    def __init__(self, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='SmoothL1Loss'):
+        super(SmoothL1Loss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,reduction=reduction,name=name)
         self.name = name
         self.reduction = reduction
         self.huber_delta = 0.5
@@ -858,8 +1001,8 @@ class SmoothL1Loss(_PairwiseLoss):
 
 
 class WingLoss(_PairwiseLoss):
-    def __init__(self, omega=10, epsilon=2, name='WingLoss'):
-        super(WingLoss, self).__init__()
+    def __init__(self, omega=10, epsilon=2, reduction='mean',enable_ohem=False, ohem_ratio=3.5, name='WingLoss'):
+        super(WingLoss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,reduction=reduction,name=name)
         self.name = name
         self.omega = omega
         self.epsilon = epsilon
@@ -875,18 +1018,20 @@ class WingLoss(_PairwiseLoss):
         Returns:
 
         """
-        with self._name_scope:
-            delta_y = tf.math.abs(target - output)
-            delta_y1 = delta_y[delta_y < self.omega]
-            delta_y2 = delta_y[delta_y >= self.omega]
-            loss1 = self.omega * tf.math.log(1 + delta_y1 / self.epsilon)
-            C = self.omega - self.omega * tf.math.log(1 + self.omega / self.epsilon)
-            loss2 = delta_y2 - C
-            return (tf.reduce_sum(loss1) + tf.reduce_sum(loss2)) / (len(loss1) + len(loss2))
+        delta_y = (target - output).abs()
+        c = self.omega * (1.0 - log(1.0 + self.omega / self.epsilon))
 
+        losses = where(
+            greater(delta_y, self.omega, dtype=dtype.bool),
+            self.omega * log(1.0 + delta_y / self.epsilon),
+            delta_y- c
+        )
+
+
+        return reduce_mean(losses,[1,2])
 
 class AdaptiveWingLoss(_PairwiseLoss):
-    def __init__(self, omega=14, theta=0.5, epsilon=1, alpha=2.1, name='AdaptiveWingLoss'):
+    def __init__(self, omega=14, theta=0.5, epsilon=1, alpha=2.1  , enable_ohem=False, ohem_ratio=3.5, name='AdaptiveWingLoss'):
         super(AdaptiveWingLoss, self).__init__()
         self.name = name
         self.omega = omega
@@ -925,7 +1070,7 @@ class AdaptiveWingLoss(_PairwiseLoss):
 
 
 class EdgeLoss(_PairwiseLoss):
-    def __init__(self, name='EdgeLoss'):
+    def __init__(self,  enable_ohem=False, ohem_ratio=3.5,name='EdgeLoss'):
         self.name = name
         super(EdgeLoss, self).__init__()
 
@@ -945,6 +1090,65 @@ class EdgeLoss(_PairwiseLoss):
             return loss1 + loss2
 
 
+class IoULoss(_ClassificationLoss):
+    def __init__(self,axis=-1, sample_weight=None,auto_balance=False, from_logits=False, ignore_index=0, cutoff=None, label_smooth=False, reduction='mean' , enable_ohem=False, ohem_ratio=3.5, name='lou_loss'):
+        super(IoULoss, self).__init__(raxis=axis, sample_weight=sample_weight,auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff, label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem,ohem_ratio=ohem_ratio,name=name)
+        self.need_target_onehot=True
+        self.is_multiselection = False
+        self.is_logsoftmax=False
+        self._built = True
+
+    def calculate_loss(self, output, target, **kwargs):
+        with self._name_scope:
+            reshape_shape = [1] * ndim(output)
+            reshape_shape[self.axis] = self.num_classes
+            sample_weight = reshape(cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype), reshape_shape)
+
+            _dtype=output.dtype
+            if self.is_logsoftmax:
+                output = clip(exp(output), 1e-8, 1 - 1e-8)
+                self.from_logits = True
+            if not self.from_logits:
+                output = softmax(output, self.axis)
+
+            intersection = reduce_sum(output*target*sample_weight)
+            union = reduce_sum((output+target)*sample_weight)-intersection
+            loss = -log(intersection / union)
+            return loss
+
+
+class SoftIoULoss(Loss):
+    def __init__(self, n_classes, reduction="mean", reduced=False):
+        super(SoftIoULoss, self).__init__()
+        self.n_classes = n_classes
+        self.reduction = reduction
+        self.reduced = reduced
+
+    def calculate_loss(self, output, target, **kwargs):
+        # logit => N x Classes x H x W
+        # target => N x H x W
+
+        N = len(output)
+
+        pred =softmax(output, dim=1)
+        target_onehot = make_onehot(target, self.n_classes)
+
+        # Numerator Product
+        inter = pred * target_onehot
+        # Sum over all pixels N x C x H x W => N x C
+        inter = inter.view(N, self.n_classes, -1).sum(2)
+
+        # Denominator
+        union = pred + target_onehot - (pred * target_onehot)
+        # Sum over all pixels N x C x H x W => N x C
+        union = union.view(N, self.n_classes, -1).sum(2)
+
+        loss = inter / (union + 1e-16)
+
+        # Return average loss over classes and batch
+        return -loss.mean()
+
+
 def get_loss(loss_name):
     if loss_name is None:
         return None
