@@ -19,6 +19,11 @@ from tensorflow.python.eager import context, tape, function
 from tensorflow.python.eager import forwardprop
 from tensorflow.python.eager.backprop import GradientTape
 from tensorflow.python.ops.losses import util as tf_losses_utils
+from trident.data.mask_common import color2label
+
+from trident.data.transform import Transform
+
+from trident.backend.opencv_backend import array2image, image2array
 
 from trident import __version__
 from trident.backend.common import *
@@ -29,6 +34,7 @@ from trident.backend.tensorflow_ops import is_tensor
 from trident.backend.tensorflow_serialization import save, load, load_pthtar
 from trident.callbacks.lr_schedulers import get_lr_scheduler, AdjustLRCallbackBase, AdjustLRCallback
 from trident.data.image_common import *
+from trident.data.vision_transforms import *
 from trident.backend.tensorspec import *
 from trident.layers.tensorflow_layers import SoftMax
 
@@ -40,7 +46,7 @@ from trident.optims.tensorflow_regularizers import *
 
 # from tensorflow.python.framework.ops import EagerTensor
 
-__all__ = ['Model', 'ImageClassificationModel', 'ImageDetectionModel', 'ImageSegmentationModel', 'LanguageModel']
+__all__ = ['Model', 'ImageClassificationModel','ImageRegressionModel', 'ImageDetectionModel', 'ImageSegmentationModel','FaceLandmarkModel','FaceRecognitionModel', 'LanguageModel']
 _session = get_session()
 _backend = get_backend()
 working_directory = _session.working_directory
@@ -68,6 +74,8 @@ class Model(ModelBase):
         self.summary_writer = None
 
     def _initial_graph(self, inputs=None, input_shape=None, output=None, initializer=None):
+        if isinstance(input_shape,numbers.Integral):
+            input_shape=(input_shape,)
         if output is None:
             raise ValueError('There is at least one output')
 
@@ -133,13 +141,13 @@ class Model(ModelBase):
             # output.cpu()
             if output.built and hasattr(output, '_output_shape') and output._output_shape is not None:
                 self._model = output
-                self.signature = None
-                if self.signature is not None and hasattr(self.signature, "outputs"):
-                    self._outputs = OrderedDict()
+                if self._model.signature.maybe_not_complete():
+                    self._model.signature = None
+                if self._model.signature is not None and hasattr(self._model.signature, "outputs"):
+                    self._outputs['output'] = self._model.signature.outputs
                     self._targets = OrderedDict()
-                    for name, spec in self.signature.outputs.item_list:
-                        self._outputs[name] = spec
-                        self._targets[name.replace("output", "target")] = spec
+                    for k, v in self._model.signature.outputs.item_list:
+                        self._targets[k.replace("output", "target")] = v
 
             else:
                 out = None
@@ -161,6 +169,7 @@ class Model(ModelBase):
                     out = output(dummay_input)
 
                 self._model = output
+                self._model.input_spec = TensorSpec(shape=self._model.input_shape, dtype=self._model.weights[0].value().dtype)
                 if isinstance(out, Tensor):
                     self._outputs['output'] = TensorSpec(shape=tensor_to_shape(out), name='output')
                     self._targets['target'] = TensorSpec(shape=tensor_to_shape(out), name='target')
@@ -356,7 +365,7 @@ class Model(ModelBase):
                     argnames = get_signature(self._losses[alias].forward, alias)
                 else:
                     argnames = get_signature(self._losses[alias].__call__, alias)
-        elif inspect.isclass(loss) and inspect._is_type(loss):
+        elif inspect.isclass(loss) and  loss.__class__.__name__=="type":
             alias = loss.__class__.__name__ if alias is None or len(alias) == 0 else alias
             if alias in self._losses:
                 dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
@@ -377,6 +386,7 @@ class Model(ModelBase):
                 argnames = get_signature(self._losses[alias].forward, alias)
             else:
                 argnames = get_signature(self._losses[alias], alias)
+            self._losses[alias].signature = argnames
         elif inspect.isfunction(loss):
             if alias in self._losses:
                 dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
@@ -387,6 +397,7 @@ class Model(ModelBase):
             else:
                 self._losses[alias] = partial(loss, **kwargs)
             argnames = get_signature(loss, alias)
+            self._losses[alias].signature = argnames
 
         # create signature
         if hasattr(self._losses[alias], 'signature') and self._losses[alias].signature is not None:
@@ -435,7 +446,7 @@ class Model(ModelBase):
             else:
                 self._metrics[alias] = partial(metric_fn, **kwargs)
             argnames = get_signature(metric_fn, alias)
-        elif inspect.isclass(metric) and inspect._is_type(metric):
+        elif inspect.isclass(metric) and metric.__class__.__name__=="type":
             alias = metric.__class__.__name__ if len(alias) == 0 else alias
             if alias in self._metrics:
                 dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
@@ -616,31 +627,36 @@ class Model(ModelBase):
         self.eval()
 
     def do_on_epoch_start(self):
-        self.training_context['time_epoch_start'] = time.time()
         if self.training_context['steps'] == 0:
-            self.training_context['time_epoch_progress'] = self.training_context['time_epoch_start']
+            self.training_context['time_epoch_start'] = time.time()
+            self.training_context['time_epoch_progress'] = 0
 
         if self.warmup > 0 and self.training_context['current_epoch'] < self.warmup:
             lr = 1e-6 * (self.training_context['current_epoch'] + 1)
             self.optimizer.adjust_learning_rate(self.base_lr, verbose=True)
             self.training_context['current_lr'] = lr
-        elif self.warmup > 0 and self.training_context['current_epoch'] == self.warmup:
+        elif 0 < self.warmup == self.training_context['current_epoch']:
             self.optimizer.adjust_learning_rate(self.base_lr, verbose=True)
             self.training_context['current_lr'] = self.base_lr
-        gc.collect()
+
 
     def do_on_epoch_end(self):
-        self.training_context['time_epoch_end'] = time.time()
+        pass
 
     def do_on_batch_start(self):
-        self.training_context['time_batch_start'] = time.time()
         if self.training_context['steps'] == 0:
-            self.training_context['time_batch_progress'] = self.training_context['time_batch_start']
+            self.training_context['time_batch_progress'] = 0
+        self.training_context['time_batch_start'] = time.time()
+
+        if self.model.device == 'cuda':
+            gc.collect()
 
     def do_on_batch_end(self):
         self.training_context['time_batch_end'] = time.time()
+        diff=( time.time() -self.training_context['time_batch_start'] )
+        self.training_context['time_batch_progress']+=diff
+        self.training_context['time_epoch_progress'] += diff
         self.training_context['steps'] += 1
-
         if self.training_context['steps'] % 100 == 0:
             gc.collect()
         if self.training_context['steps'] % 200 == 0:
@@ -648,17 +664,16 @@ class Model(ModelBase):
                 self._model.cpu()
                 self._model.cuda()
 
-        if self.training_context['steps'] % _session.epoch_equivalent == 0:
-            if self.warmup > 0 and self.warmup == self.training_context['steps'] // _session.epoch_equivalent:
+        if (self.training_context['steps'] + 1) % _session.epoch_equivalent == 0:
+            if self.warmup > 0 and self.warmup == (self.training_context['steps'] + 1) // _session.epoch_equivalent:
                 self.adjust_learning_rate(self.training_context['base_lr'])
                 self.warmup = 0
-
         if self.training_context['current_batch'] == 0 and self.training_context['is_print_batch_progress'] == True:
             temp = OrderedDict()
             for k in self.training_context['losses'].key_list:
                 if len(self.training_context['losses'][k]) > 0:
                     temp[k] = self.training_context['losses'][k][-1][-1]
-            print(temp)
+            print('{ ' + ', '.join(['{0}: {1}'.format(k, adaptive_format(v,value_type='loss')) for k, v in temp.items()]) + ' }')
 
     def do_on_data_received(self, train_data, test_data):
         if train_data is None and test_data is None:
@@ -695,7 +710,7 @@ class Model(ModelBase):
                                 available_fields.remove('input')
                             elif len(self._model.signature.inputs.key_list) == 1:
                                 for item in available_fields:
-                                    data_shape = train_data[item].shape if len(train_data[item].shape) > 2 else TensorShape([None])
+                                    data_shape = tensor_to_shape(train_data[item])
                                     if 'target' not in item and 'output' != item and data_shape == inshapes[0].shape:
                                         data_feed[arg] = item
                                         available_fields.remove(item)
@@ -727,12 +742,12 @@ class Model(ModelBase):
                                 elif len(available_fields) > 0:
                                     target_shape = outshapes
                                     for item in available_fields:
-                                        data_shape = list(train_data[item].shape) if len(train_data[item].shape) > 1 else [None]
+                                        data_shape = tensor_to_shape(train_data[item])
                                         if target_shape == data_shape:
                                             data_feed[arg] = item
                                             available_fields.remove(item)
                                         elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
-                                                train_data[item].dtype)) and target_shape[:-1] == data_shape:
+                                                train_data[item].dtype)) and target_shape== data_shape:
                                             data_feed[arg] = item
                                             available_fields.remove(item)
                                         else:
@@ -800,11 +815,11 @@ class Model(ModelBase):
                 callback.on_optimization_step_start(self.training_context)
 
             if self.training_context['stop_update'] == 0:
-                self.optimizer.step(self.optimizer.grads_and_vars)
+                self.optimizer.step(self.training_context['grads_and_vars'])
 
             elif 0 < self.training_context['stop_update'] < 1:
                 if random.random() <= self.training_context['stop_update']:
-                    self.optimizer.step(self.optimizer.grads_and_vars)
+                    self.optimizer.step(self.training_context['grads_and_vars'])
             else:
                 self.training_context['stop_update'] = self.training_context['stop_update'] - 1
 
@@ -814,7 +829,7 @@ class Model(ModelBase):
     def do_post_gradient_update(self):
 
         self.training_context['tmp_losses'].collect('total_losses', self.training_context['steps'], to_numpy(self.training_context['current_loss']).mean())
-        if self.training_context['is_collect_data'] == True:
+        if self.training_context['is_collect_data']:
             steps, values = self.training_context['tmp_losses'].get_series('total_losses')
             self.training_context['losses'].collect('total_losses', self.training_context['steps'], float(to_numpy(values).mean()))
             self.training_context['tmp_losses'].reset()
@@ -945,24 +960,55 @@ class Model(ModelBase):
         folder, filename, ext = split_path(file_path)
         if filename == '':
             filename = self.name
-
-        ext = '.pth.tar'
-        save_path = os.path.join(folder, filename + ext)
-        if not os.path.exists(save_path):
-            save_path = os.path.join(working_directory, filename + ext)
-        pretrained_dict = load_pthtar(file_path)
         state_dict = None
-        if "state_dict" in pretrained_dict.keys():
-            state_dict = pretrained_dict['state_dict']
+        pretrained_dict = None
+        if ext == '.pth.tar':
+            state_dict = load_pthtar(file_path)
+        elif ext == '.pth':
+            load_path = file_path
+            if not os.path.exists(file_path):
+                if os.path.exists(file_path.replace(ext, '.pth.tar')):
+                    load_path = file_path.replace(ext, '.pth.tar')
+                elif os.path.exists(os.path.join(working_directory, filename + ext)):
+                    load_path = os.path.join(working_directory, filename + ext)
+            recovery_pth = load_pthtar(load_path)
 
-            # pretrained_dict = remove_prefix(pretrained_dict, 'module.')
-        if check_keys(self._model, state_dict):
-            self._model.load_state_dict(state_dict, strict=False)
+            if isinstance(recovery_pth, dict):
+                state_dict = recovery_pth
+
+            elif isinstance(recovery_pth, Layer):
+                state_dict = recovery_pth.state_dict()
+
+        if 'backend' in state_dict and state_dict['backend'] != 'tensorflow':
+            raise RuntimeError(
+                'The model archive {0} is a {1}-based model, but current backend is Tensorflow, so cannot load model properly.'.format(file_path, state_dict['backend']))
+
+        if "state_dict" in state_dict.keys():
+            pretrained_dict = state_dict['state_dict']
+        else:
+            pretrained_dict = state_dict
+
+        if check_keys(self._model, pretrained_dict):
+            has_abnormal = False
+            for key in pretrained_dict.keys():
+                value = pretrained_dict[key]
+                if is_tensor(value) and any_abnormal_number(value):
+                    has_abnormal = True
+                    pretrained_dict[key] = where(is_nan(value), random_normal_like(value, mean=0, std=0.02).to(get_device()).cast(value.dtype), value)
+                if is_tensor(value) and ndim(value) == 0:
+                    pretrained_dict[key] = to_tensor(value.item())
+
+            if has_abnormal:
+                sys.stderr.write(self._model._name + '  has_abnormal detected and  fixed!!\n')
+            self._model.load_state_dict(pretrained_dict, strict=False)
             print('Model loaded!')
+            # must switch to evluate first beforeinference or training
+            # Dropout and Batch normalization will behavior change!!!
 
-        self.signature = None
-        if 'signature' in pretrained_dict:
-            self.signature = pretrained_dict['signature']
+            self._model.eval()
+        if "signature" in state_dict.keys() and (self._model.signature is None or state_dict['signature'] != self._model.signature):
+            self._model.signature = state_dict['signature']
+        self._model.to(get_device())
 
     def merge_grads(self, old_grads, new_grades):
         if isinstance(old_grads, list) and isinstance(new_grades, list) and len(old_grads) == len(new_grades):
@@ -1018,7 +1064,7 @@ class Model(ModelBase):
                 for callback in self.callbacks:
                     callback.on_data_received(self.training_context)
 
-                if accumulate_grads == False:
+                if not accumulate_grads:
                     self.training_context['current_loss'] = to_tensor(0.0, requires_grad=True)
                     self.do_preparation_for_loss()
                     self.training_context['optimizer'] = self.optimizer
@@ -1044,7 +1090,7 @@ class Model(ModelBase):
                                         self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
                                 else:
                                     self.train_data[self.outputs.key_list[0]] = output
-                                    if self.use_output_as_loss == True:
+                                    if self.use_output_as_loss:
                                         this_loss = output.sum()
                                         self.training_context['losses'].collect(self.outputs.key_list[0], self.training_context['steps'], this_loss)
                                         self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
@@ -1053,14 +1099,15 @@ class Model(ModelBase):
                             PrintException()
                             if isinstance(self._model, Layer) and any_abnormal_number(self._model):
                                 for para in self._model.parameters():
-                                    if any_abnormal_number(para):
+                                    if para is not None and any_abnormal_number(para):
                                         para.data.copy_(where(is_nan(para), random_normal_like(para, mean=0, std=0.02).to(get_device()), para))
 
                     # write output in to data
 
                     # confirm singleton
                     # output=unpack_singleton(output)
-
+                    for callback in self.callbacks:
+                        callback.on_loss_calculation_start(self.training_context)
                     # losss
                     for k, v in self._losses.items():
                         if not hasattr(v, 'start_epoch') or (hasattr(v, 'start_epoch') and v.start_epoch <= self.training_context['current_epoch']):
@@ -1104,7 +1151,7 @@ class Model(ModelBase):
                     for callback in self.callbacks:
                         callback.on_loss_calculation_end(self.training_context)
 
-                    if accumulate_grads == False:
+                    if not accumulate_grads:
                         # regularizer
                         for k, v in self._regs.items():
                             this_loss = to_tensor(0.0)
@@ -1122,8 +1169,10 @@ class Model(ModelBase):
                                 self.training_context['losses'].collect(k + '_Loss', self.training_context['steps'], this_loss)
 
                 vars = grad_tape.watched_variables()
-                grads = grad_tape.gradient(self.training_context['current_loss'], vars, unconnected_gradients=tf.UnconnectedGradients.NONE)
+                grads = grad_tape.gradient(self.training_context['current_loss'], vars, unconnected_gradients=tf.UnconnectedGradients.ZERO)
                 # grads = tuple([where(is_nan(grad), zeros_like(grad), grad) for grad in grads])
+
+                self.training_context['grads_and_vars']= zip(grads, vars);
                 self.optimizer.grads_and_vars = zip(grads, vars)
                 # self.training_context['grads'] = grads
                 # self.training_context['vars'] = vars
@@ -1191,9 +1240,21 @@ class Model(ModelBase):
                     # aggregate tmp data and move to metrics history
                     for k, v in self.training_context['tmp_metrics'].items():
                         steps, values = self.training_context['tmp_metrics'].get_series(k)
-                        self.training_context['metrics'].collect(k, self.training_context['steps'], float(to_numpy(values).mean()))
+
+                        # check
+                        if len(steps) > 0:
+                            values = np.mean(to_numpy(values))
+                            self.training_context['metrics'].collect(k, self.training_context['steps'], values)
                     self.training_context['tmp_metrics'].reset()
 
+                    for k, v in self.training_context['tmp_losses'].items():
+                        steps, values = self.training_context['tmp_losses'].get_series(k)
+
+                        if len(steps) > 0:
+                            values = np.mean(to_numpy(values))
+                            self.training_context['losses'].collect(k, self.training_context['steps'], values)
+                    self.training_context['tmp_losses'].reset()
+
                 # ON_BATCH_END
                 self.do_on_batch_end()
                 for callback in self.training_context['callbacks']:
@@ -1246,9 +1307,10 @@ class Model(ModelBase):
                             self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.array(metric_values).mean())
                             self.training_context['metrics'].last_aggregate_idx = len(metric_values)
                         else:
-                            self.epoch_metric_history.collect(k, self.training_context['current_epoch'],
-                                                              np.array(metric_values[self.training_context['metrics'].last_aggregate_idx:]).mean())
-                            self.training_context['metrics'].last_aggregate_idx = len(metric_values)
+                            if self.training_context['metrics'].last_aggregate_idx < len(metric_values):
+                                self.epoch_metric_history.collect(k, self.training_context['current_epoch'],
+                                                                  np.array(metric_values[self.training_context['metrics'].last_aggregate_idx:]).mean())
+                                self.training_context['metrics'].last_aggregate_idx = len(metric_values)
 
                     if is_print_epoch_progress:
                         self.do_on_progress_start()
@@ -1419,12 +1481,17 @@ class Model(ModelBase):
 
 class ImageClassificationModel(Model):
     def __init__(self, inputs=None, input_shape=None, output=None):
-        super(ImageClassificationModel, self).__init__(inputs, input_shape, output)
+        super(ImageClassificationModel, self).__init__(inputs,input_shape,output)
 
         self._class_names = []
-        self.preprocess_flow = []
         self._idx2lab = {}
         self._lab2idx = {}
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list) > 0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.outputs.value_list) > 0 and self._model.signature.outputs.value_list[0].object_type is None:
+            self._model.signature.outputs.value_list[0].object_type = ObjectType.classification_label
 
     @property
     def class_names(self):
@@ -1457,14 +1524,15 @@ class ImageClassificationModel(Model):
         if self._model.built:
             if isinstance(self._model, Layer):
                 self._model.eval()
+
             img = image2array(img)
             if img.shape[-1] == 4:
                 img = img[:, :, :3]
             for func in self.preprocess_flow:
-                if inspect.isfunction(func) and func is not image_backend_adaption:
-                    img = func(img)
+                if (inspect.isfunction(func) or isinstance(func, Transform)) and func is not  image_backend_adaption:
+                    img = func(img, spec=self._model.input_spec)
             img = image_backend_adaption(img)
-            inp = to_tensor(np.expand_dims(img, 0))
+            inp = to_tensor(np.expand_dims(img, 0)).to(self._model.device)
             result = self._model(inp)
             result = to_numpy(result)[0]
             if self.class_names is None or len(self.class_names) == 0:
@@ -1484,34 +1552,81 @@ class ImageClassificationModel(Model):
             raise ValueError('the model is not built yet.')
 
 
+class ImageRegressionModel(Model):
+    def __init__(self, inputs=None, input_shape=None, output=None):
+        super(ImageRegressionModel, self).__init__(inputs,input_shape,output)
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list)>0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+
+
+
+    def infer_single_image(self, img):
+        if self._model.built:
+            self._model.eval()
+            img = image2array(img)
+            img_shp = img.shape
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+            rescale_scale = 1.0
+            for func in self.preprocess_flow:
+                if (inspect.isfunction(func) or isinstance(func,Transform)) and func is not image_backend_adaption:
+                    img = func(img, spec=self._model.input_spec)
+                    if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op' ) or( isinstance(func,Transform) and  func.name=='resize') :
+                        rescale_scale = func.scale
+            img = image_backend_adaption(img)
+            if isinstance( self._model,Layer):
+                inp = to_tensor(np.expand_dims(img, 0)).to(self._model.device).to(self._model.weights[0].data.dtype)
+                result = self._model(inp)
+                result = to_numpy(result)
+                return result.astype(np.int32)
+            else:
+
+                raise ValueError('the model is not layer.')
+
+        else:
+            raise ValueError('the model is not built yet.')
+
+
+
 class ImageDetectionModel(Model):
     def __init__(self, inputs=None, input_shape=None, output=None):
-        super(ImageDetectionModel, self).__init__(inputs, input_shape, output)
-        self.preprocess_flow = []
-        self.detection_threshould = 0.5
+        super(ImageDetectionModel, self).__init__(inputs,input_shape,output)
+        object.__setattr__(self, 'detection_threshold', 0.5)
+        object.__setattr__(self, 'nms_threshold', 0.3)
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list)>0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+
 
     def infer_single_image(self, img, scale=1):
         if self._model.built:
             self._model.to(self.device)
             self._model.eval()
+
             img = image2array(img)
             if img.shape[-1] == 4:
                 img = img[:, :, :3]
-
+            rescale_scale = 1
             for func in self.preprocess_flow:
-                if inspect.isfunction(func):
-                    img = func(img)
-
-            result = self._model(to_tensor(np.expand_dims(img, 0)))
+                if (inspect.isfunction(func) or isinstance(func,Transform)) and func is not image_backend_adaption:
+                    img = func(img, spec=self._model.input_spec)
+                    if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op') or (isinstance(func, Transform) and func.name == 'resize'):
+                        rescale_scale = func.scale
+            img = image_backend_adaption(img)
+            inp = to_tensor(np.expand_dims(img, 0)).to(get_device()).to( self._model.weights[0].value().dtype)
+            result = self._model(inp)
             self._model._set_inputs(to_tensor(np.expand_dims(img, 0)))
-            bboxes = self.generate_bboxes(*result, threshould=self.detection_threshould, scale=scale)
+            bboxes = self.generate_bboxes(*result, threshold=self.detection_threshold, scale=scale)
             bboxes = self.nms(bboxes)
             # idx=int(np.argmax(result,-1)[0])
             return bboxes
         else:
             raise ValueError('the model is not built yet.')
 
-    def generate_bboxes(self, *outputs, threshould=0.5, scale=1):
+    def generate_bboxes(self, *outputs, threshold=0.5, scale=1):
         raise NotImplementedError
 
     def nms(self, bboxes):
@@ -1520,8 +1635,196 @@ class ImageDetectionModel(Model):
 
 class ImageSegmentationModel(Model):
     def __init__(self, inputs=None, input_shape=None, output=None):
-        super(ImageSegmentationModel, self).__init__(inputs, input_shape, output)
+        super(ImageSegmentationModel, self).__init__(inputs,input_shape,output)
+        self.palette= OrderedDict()
+        self._class_names = []
+        self._idx2lab = {}
+        self._lab2idx = {}
+        if self.input_spec is not None and self.input_spec.object_type is None:
+            self.input_spec.object_type = ObjectType.rgb
+
+    @property
+    def class_names(self):
+        return self._class_names
+
+    @class_names.setter
+    def class_names(self, value):
+        if self._class_names is not None and self._class_names != value:
+            self._class_names = value
+            self._lab2idx = {v: k for k, v in enumerate(self._class_names)}
+            self._idx2lab = {k: v for k, v in enumerate(self._class_names)}
+
+    def index2label(self, idx: int):
+        if self._idx2lab is None or len(self._idx2lab.items()) == 0:
+            raise ValueError('You dont have proper mapping class names')
+        elif idx not in self._idx2lab:
+            raise ValueError('Index :{0} is not exist in class names'.format(idx))
+        else:
+            return self._idx2lab[idx]
+
+    def label2index(self, label):
+        if self._lab2idx is None or len(self._lab2idx.items()) == 0:
+            raise ValueError('You dont have proper mapping class names')
+        elif label not in self._lab2idx:
+            raise ValueError('label :{0} is not exist in class names'.format(label))
+        else:
+            return self._lab2idx[label]
+
+    def infer_single_image(self, img):
+        if isinstance(self._model, Layer) and self._model.built:
+            self._model.eval()
+            if self._model.input_spec.object_type is None:
+                self._model.input_spec.object_type = ObjectType.rgb
+            img = image2array(img)
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+            rescale_scale = 1.0
+            for func in self.preprocess_flow:
+                if (inspect.isfunction(func) or isinstance(func, Transform)) and not isinstance(func, image_backend_adaption):
+                    img = func(img, spec=self._model.input_spec)
+
+                    if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op' ) or( isinstance(func,Transform) and  func.name=='resize') :
+                        rescale_scale = func.scale
+
+            img = image_backend_adaption(img)
+            inp = cast(to_tensor(np.expand_dims(img, 0)).to(self._model.get_root().device),self._model.weights[0].data.dtype)
+            result = argmax(self._model(inp)[0], axis=-1)
+            result = to_numpy(result)
+            if self.class_names is None or len(self.class_names) == 0:
+                return result
+            else:
+                if len(self.palette) > 0:
+                    color_result = color2label(result, self.palette)
+                    return color_result
+                else:
+                    return result
+        else:
+            raise ValueError('the model is not built yet.')
+
+
+class ImageGenerationModel(Model):
+    def __init__(self, inputs=None, input_shape=None, output=None):
+        super(ImageGenerationModel, self).__init__(inputs,input_shape,output)
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list)>0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.outputs.value_list) > 0 and self._model.signature.outputs.value_list[0].object_type is None:
+            self._model.signature.outputs.value_list[0].object_type = ObjectType.rgb
+
+    @property
+    def reverse_preprocess_flow(self):
+        return_list = []
+        for i in range(len(self.preprocess_flow)):
+            fn = self.preprocess_flow[-1 - i]
+            if fn.__qualname__ == 'normalize.<locals>.img_op':
+                return_list.append(unnormalize(fn.mean, fn.std))
+        return return_list
+
+    def infer_single_image(self, img):
+        if self._model.built:
+            self._model.eval()
+            if self._model.input_spec.object_type is None:
+                self._model.input_spec.object_type = ObjectType.rgb
+            img = image2array(img)
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+
+            for func in self.preprocess_flow:
+                if (inspect.isfunction(func) or isinstance(func,Transform)) and func is not image_backend_adaption:
+                    img = func(img, spec=self._model.input_spec)
+            img = image_backend_adaption(img)
+            inp = to_tensor(np.expand_dims(img, 0)).to(self._model.device).to(self._model.weights[0].data.dtype)
+            result = self._model(inp)
+            result = to_numpy(result)[0]
+
+            for func in self.reverse_preprocess_flow:
+                if inspect.isfunction(func):
+                    result = func(result)
+            result = array2image(result)
+            return result
+
+class FaceLandmarkModel(Model):
+    def __init__(self, inputs=None, input_shape=None, output=None):
+        super(FaceLandmarkModel, self).__init__(inputs,input_shape,output)
+
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list) > 0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.outputs.value_list) > 0 and self._model.signature.outputs.value_list[0].object_type is None:
+            self._model.signature.outputs.value_list[0].object_type = ObjectType.landmarks
+
+
+    def infer_single_image(self, img):
+        if self._model.built:
+            self._model.eval()
+            if self._model.input_spec.object_type is None:
+                self._model.input_spec.object_type=ObjectType.rgb
+            img = image2array(img)
+            img_shp=img.shape
+
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+            rescale_scale=1.0
+            for func in self.preprocess_flow:
+                if (inspect.isfunction(func) or isinstance(func, Transform)) and not isinstance(func, image_backend_adaption):
+                    img = func(img,spec=self._model.input_spec)
+                    if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op' ) or( isinstance(func,Transform) and  func.name=='resize') :
+                        rescale_scale = func.scale
+
+            img = image_backend_adaption(img)
+            if isinstance(self._model, Layer):
+                inp = to_tensor(np.expand_dims(img, 0)).to(self._model.device).to(self._model.weights[0].data.dtype)
+                result = self._model(inp)
+                result = to_numpy(result)/ rescale_scale
+                result[:,:, 0::2] =clip(result[:,:, 0::2] ,0,img_shp[1])
+                result[:,:, 1::2] =clip(result[:,:, 1::2],0,img_shp[0])
+                return result.astype(np.int32)
+            else:
+
+                raise ValueError('the model is not layer.')
+
+        else:
+            raise ValueError('the model is not built yet.')
+
+
+class FaceRecognitionModel(Model):
+    def __init__(self, inputs=None, input_shape=None, output=None):
+        super(FaceRecognitionModel, self).__init__(inputs,input_shape,output)
+
+
         self.preprocess_flow = []
+        if self._model.input_spec.object_type is None:
+            self._model.input_spec.object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.inputs.value_list)>0 and self._model.signature.inputs.value_list[0].object_type is None:
+            self._model.signature.inputs.value_list[0].object_type = ObjectType.rgb
+        if self._model.signature is not None and len(self._model.signature.outputs.value_list) > 0 and self._model.signature.outputs.value_list[0].object_type is None:
+            self._model.signature.outputs.value_list[0].object_type = ObjectType.embedding
+
+
+    def infer_single_image(self, img):
+
+        if isinstance(self._model, Layer) and self._model.built:
+            self._model.eval()
+
+            img = image2array(img)
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+            rescale_scale = 1.0
+            for func in self.preprocess_flow:
+                if (inspect.isfunction(func) or isinstance(func, Transform)) and func is not image_backend_adaption:
+                    img = func(img, spec=self._model.input_spec)
+                    if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op') or (isinstance(func, Transform) and func.name == 'resize'):
+                        rescale_scale = func.scale
+            img = image_backend_adaption(img)
+            inp = to_tensor(np.expand_dims(img, 0)).to(self._model.device).to(self._model.weights[0].data.dtype)
+            result = self._model(inp)[0]
+            embedding = to_numpy(result)
+            return embedding
+
+        else:
+            raise ValueError('the model is not built yet.')
 
 
 class LanguageModel(Model):
